#ifndef SBW_BROKER_INSTANCE
#define SBW_BROKER_INSTANCE

#include "InstanceThread.h"
#include "Host.h"
#include "IntegerHashtable.h"
#include "Instance.h"
#include "BrokerInterface.h"
#include "Properties.h"
#include "StringUtil.h"

#include "SBW/DataBlockReader.h"
#include "SBW/SBWLowLevel.h"
#include "SBW/SBWException.h"

#include <string>
#include <iostream>

#include "BrokerDefines.h"

using namespace SystemsBiologyWorkbench::Broker;
namespace SystemsBiologyWorkbench
{
	namespace Broker
	{

		class BrokerInstance : public InstanceThread
		{
		private:
			/** The identifier for this instance on the remote broker. **/
			int myRemoteId;
			/** The id for the local broker as it's known by the remote broker. **/
			int remoteIdForLocalBroker;
			/** A proxy object for this broker's service methods. **/
			BrokerInterface *brokerInterface;
			/** The local (not remote) host.  **/
			Host *localHost;
			IntegerHashtable<int> remoteIdMap;
			bool connectStreams();
			bool exchangeIds();
		public:
			BrokerInstance(SOCKET socket, Host *host, InstanceThread::Hook *hook) :
			InstanceThread(socket, "broker", hook), transmitMutex("brokerinstance")
			{
				myRemoteId = BROKER_MODULE;
				InstanceBase::myBrokerId = InstanceBase::myId;
				InstanceBase::myName = "BROKER";
				localHost = host;

				startupWaitTime = DEFAULT_BROKER_STARTUP_WAIT;

				setLocalIdForRemote(myRemoteId, myId);
				brokerInterface = NULL;

				// remoteHost, myFullName & myInternalName are set in exchangeIds().
			}
			static InstanceThread::Factory *getFactory(InstanceThread::Hook *hook,Host *host);
			bool connect(std::string key);
			void deliverMessage(int destId, Instance *destInstance, unsigned char* message, int length);
			virtual void transmit(unsigned char *data, int length);
			void transmitDirect(char *raw, int offset, int length, bool addLength = true);
			DataBlockReader readDirect();
			void reportTransmitFailed(int length, int destModuleId, unsigned char* message, SBWException *ex);
			BrokerInterface *getBrokerInterface();
			int getRemoteId();
			int getRemoteIdForLocalBroker();
			int getLocalIdForRemote(int remoteModuleId);
			void setLocalIdForRemote(int remoteModuleId, int localId);
			void removeLocalIdForRemote(int remoteModuleId);
			SBWOSMutex transmitMutex ;

			class Factory : public InstanceThread::Factory 
			{
			public:
				Factory(Host *host, InstanceThread::Hook *hook) { _host = host; _hook = hook; }
				Factory() { _host = NULL; _hook = NULL; }
				InstanceThread *newInstance(SOCKET socket, Host *host, InstanceThread::Hook *hook);
				virtual InstanceThread *newInstance(SOCKET socket);
				virtual std::string getType() { return "broker"; }
			private:
				Host *_host;
				InstanceThread::Hook *_hook;

			};
			class Hook : public InstanceThread::Hook 
			{
			public :
				/**
				* Callback invoked as the last step by the {@link InstanceThread}
				* class constructor.
				* <p>
				* @param instance the instance
				**/
				virtual void onCreation(InstanceThread *instance);

				/**
				* Callback invoked as the first step by the {@link InstanceThread#run}
				* method.
				* <p>
				* @param instance the instance
				**/
				virtual void onStartup(InstanceThread *instance);

				/**
				* Callback invoked after the {@link InstanceThread} message loop
				* has stopped being operational.
				* <p>
				* @param instance the instance
				**/
				virtual void onShutdown(InstanceThread *instance);
			};

		};
	}
}

#endif
