#ifndef SBW_INTEGER_HASHTABLE
#define SBW_INTEGER_HASHTABLE

#include <vector>

#include "SBW/SBWApplicationException.h"

#define DEFAULT_CAPACITY 31

namespace SystemsBiologyWorkbench
{
	namespace Broker
	{		
		template<class T> class Entry
		{
		private:
			int key;                // The key for the entry.
			T value;           // The value stored with this key.
			Entry<T> *next;             // Pointer to next entry in list.
		public:
			/** Constructor. **/
			Entry(int key, T value) 
			{ 
				this->key = key; 
				this->value = value; 
			}
			/** Get the key for this entry. **/
			int getKey() 
			{
				return key; 
			}
			/** Get the value for this entry. **/
			T getValue() 
			{
				return value; 
			}
			/** Set the value for this entry and return the old value. **/
			T setValue(T newValue) 
			{ 
				T old = value; 
				value = newValue; 
				return old; 
			}
			/** Reset the value for this entry to something else. **/
			void changeValue(T newValue) 
			{ 
				value = newValue; 
			} 
		};
		template<class T>class IntegerHashtable
		{
		public:
			IntegerHashtable() 
			{
				capacity = DEFAULT_CAPACITY;
				m_size = 0;
			}
			IntegerHashtable(int capacity)
			{
				this->capacity = capacity;
				m_size = 0;
			}
			int size() 
			{
				return table.size(); 
			}
			bool exists(int key)
			{
				return find(key) != NULL;
			}
			T get(int key)
			{
				Entry<T> *e = find(key);
				return (e == NULL ? NULL : e->getValue());				
			}
			std::vector<T> getAll()
			{
				std::vector<T> oList;

				for (unsigned int i = 0; i < table.size(); i++)
					oList.push_back(table[i]->getValue());
				return oList;
			}
			void put(int key, T val)
			{
				table.push_back( new Entry<T>(key,val) );
			}
			T remove(int key)
			{
				std::vector<Entry<T> *> ::iterator oIterator;
				for ( oIterator = table.begin(); oIterator != table.end(); oIterator ++)
				{
					if ((*oIterator)->getKey() == key)
					{
						table.erase(oIterator);
						return (*oIterator)->getValue();
					}
				}
				return NULL;
			}
		private:
			Entry<T> *find(int key)
			{
				for (unsigned int i = 0; i < table.size(); i++) 
				{
					Entry<T> *oTemp = table[i];
					if(oTemp->getKey() == key)
						return oTemp;
				}
				return NULL;					
			}
			/** Default capacity of the hash table, in terms of number of buckets. **/
			/** Set capacity of the hash table, in terms of number of buckets. **/
			int capacity;
			/** Current size of the hash table (i.e., number of entries in it). **/
			int m_size;
			/** The hash table itself. **/
			std::vector<Entry<T> *> table;			
		};
	}
}

#endif

