#ifndef SBW_BROKER_MODULE_INSTANCE
#define SBW_BROKER_MODULE_INSTANCE

#include "SBW/SBWException.h"

#include "Host.h"
#include "Instance.h"
#include "InstanceThread.h"

#include <string>
#include <iostream>

namespace SystemsBiologyWorkbench
{
	namespace Broker
	{
		class ModuleInstance :
			public InstanceThread
		{
		public:
			class Factory : public InstanceThread::Factory 
			{
			public:
				Factory(Host *host, InstanceThread::Hook *hook) 
				{ 
					_host = host; _hook = hook; 
				}
				Factory() 
				{ 
					_host = NULL; _hook = NULL; 
				}
				InstanceThread *newInstance(SOCKET socket, Host *host, InstanceThread::Hook *hook)
				{ 
					return new ModuleInstance(socket, host, hook); 
				}

				virtual InstanceThread *newInstance(SOCKET socket) 
				{
					return new ModuleInstance(socket, _host, _hook); 
				} ;
				virtual std::string getType() { 
					return "module"; 
				}
			private:
				Host *_host;
				InstanceThread::Hook *_hook;
			};
			class Hook : public InstanceThread::Hook 
			{
			public :
				/**
				* Callback invoked as the last step by the {@link InstanceThread}
				* class constructor.
				* <p>
				* @param instance the instance
				**/
				virtual void onCreation(InstanceThread *instance);
				/**
				* Callback invoked as the first step by the {@link InstanceThread#run}
				* method.
				* <p>
				* @param instance the instance
				**/
				virtual void onStartup(InstanceThread *instance);

				/**
				* Callback invoked after the {@link InstanceThread} message loop
				* has stopped being operational.
				* <p>
				* @param instance the instance
				**/
				virtual void onShutdown(InstanceThread *instance);
			};

		
			ModuleInstance(SOCKET socket, Host *host, InstanceThread::Hook *hook);
		private:
			void reportTransmitFailed(int length, int destModuleId, unsigned char* message, SBWException *ex);
			SBWOSMutex transmitMutex ;
		public:
			static InstanceThread::Factory *getFactory(InstanceThread::Hook *hook,
				Host *host);
			bool connect(std::string key);
			void deliverMessage(int destId, Instance *destInstance, unsigned char* message, int length);
			virtual void transmit(unsigned char *data, int length);

		};
	}
}

#endif

