#include "SBW/SBWOSSocket.h"
#include "SBW/DataBlockWriter.h"
#include "SBW/ServiceImpl.h"
#include "SBW/ModuleImpl.h"
#include "SBW/MethodImpl.h"
#include "SBW/Handler.h"
#include "SBW/Module.h"
#include "SBW/MethodTable.h"
#include "SBW/SBWException.h"
#include "SBW/SBWApplicationException.h"
#include "SBW/SBWMethodNotFoundException.h"

#include "ObjectOrientedReceiver.h"
#include "BrokerDefines.h"
#include "Sys.h"

#include <sstream>

using namespace std;
using namespace SystemsBiologyWorkbench;
using namespace SystemsBiologyWorkbench::Broker;


ObjectOrientedReceiver::ObjectOrientedReceiver(std::string moduleName)
{
	this->moduleName = moduleName;
	this->moduleId = -1;
}
void ObjectOrientedReceiver::addService(std::string name, std::string displayName,
										std::string category, SystemsBiologyWorkbench::Broker::Broker  *impl, std::string help)
{
	ServiceImpl* oImpl = new ServiceImpl(name, displayName, category, help);
	std::vector<std::string> oMethods = getBrokerMethods();
	for (unsigned int i = 0; i < oMethods.size(); i ++ )
		oImpl->addMethod(NULL,oMethods[i]);
	services.push_back(oImpl);
	numServices = services.size();
	m_oBroker = impl;
}
DataBlockWriter ObjectOrientedReceiver::receive(Integer fromModuleId, Integer serviceId,
												Integer methodId, DataBlockReader args)
{
	if (serviceId == SYSTEM_SERVICE)
	{
		DataBlockWriter writer;
		try
		{
			// See SBWLowLevel.java for the list of system methods
			// that must be handled.
			switch (methodId)
			{
			case GET_SERVICES_METHOD:
				{
					writer << getServices();
					return writer;
				}
			case GET_METHODS_METHOD:
				{
					int queryServiceId;
					args >> queryServiceId;
					ServiceImpl *srvImpl = getService(queryServiceId);
					writer << srvImpl ->getSignatureStrings();
					return writer;
				}

			case GET_METHOD_HELP_METHOD: 
				{
					int targetServiceId,  targetMethodId; 
					args >> targetServiceId >> targetMethodId;
					ServiceImpl *srvImpl = getService(targetServiceId);
					writer << srvImpl->getMethod(targetMethodId)->getHelp();
					return writer;
				}

			case SHUTDOWN_METHOD:
			case ON_OTHER_MODULE_INSTANCE_SHUTDOWN_METHOD:
			case ON_OTHER_MODULE_INSTANCE_STARTUP_METHOD:
			case ON_REGISTRATION_CHANGE_METHOD:
				writer.add(0);
				return writer;

			default:
				{
					std::stringstream sTemp;
					sTemp << "Method "  <<  methodId << " does not exist on service " <<  serviceId;
					throw new SBWMethodNotFoundException(sTemp.str() );
				}
			}
		}
		catch (...)
		{
			//writer.release();
			throw new SBWApplicationException("receive failed ...");
		}
	}
	else
	{
		// Since ConfigVars is 'final', the following should get optimized
		// out completely when not compiled in debug mode.
		ServiceImpl *srvImpl = getService(serviceId);
#ifdef _DEBUG

		if (srvImpl == NULL)
		{
			TRACE("Module " << fromModuleId << " invoked" <<  " nonexistent service " <<  serviceId);
		}
		else
		{
			std::string name = srvImpl->getName();
			if (name != "BROKER") 
			{
				std::vector<std::string> sigs = srvImpl->getSignatureStrings();
				if (methodId < 0 || methodId >= (int)sigs.size())
				{
					TRACE("Module " << fromModuleId  << " passed in" <<  " out-of-range method id " << methodId)
				}
				else
				{
					TRACE("Module " <<  fromModuleId << " invoking method " <<  methodId <<  " ('"\
						<< sigs[methodId] << "') on service " << serviceId << " ('" << name << "') of module "\
						<< moduleId << " (name '" << moduleName << "')")
				}
			}
		}
#endif
		if (srvImpl != NULL)
		{
			try
			{	
				if ( srvImpl->getName() == "BROKER")
				{
					TRACE("Module " <<  fromModuleId << " invoking method " <<  methodId <<  " ('"\
						<< getBrokerMethods()[methodId] << "') on service " << serviceId << " ('" << srvImpl->getName() << "') of module "\
						<< moduleId << " (name '" << moduleName << "')");

					DataBlockWriter oWriter;
					switch (methodId)
					{
					case 0:
						oWriter << m_oBroker->getVersion();
						break;
					case 1:
						{
							std::string sTemp; 
							bool bTemp;
							args >> sTemp >> bTemp;
							oWriter << m_oBroker->findServices(sTemp,bTemp);
							break;
						}
					case 2:
						{
							std::string sTemp; 
							bool bTemp;
							args >> sTemp >> bTemp;
							oWriter << m_oBroker->getModuleDescriptor(sTemp,bTemp);
							break;
						}
					case 3:
						{
							int nTemp;
							args >> nTemp;						
							oWriter << m_oBroker->getModuleDescriptor(nTemp);
							break;
						}
					case 4:
						{						
							bool bTemp1,bTemp2;
							args >> bTemp1 >> bTemp2;
							oWriter << m_oBroker->getModuleDescriptors(bTemp1,bTemp2);
							break;
						}
					case 5: 
						{
							std::string sName;
							args >> sName;					
							oWriter << m_oBroker->getModuleInstance(sName);
							break;
						}
					case 6:
						{
							std::string sName;
							args >> sName;					
							oWriter << m_oBroker->getServiceCategories(sName);
							break;
						}
					case 7:
						{
							std::string sName;
							args >> sName;					
							if (!Sys::isSecure() || !m_oBroker->isRemoteInstance(fromModuleId))
								oWriter << m_oBroker->linkBroker(sName);
							else throw new SBWApplicationException("insecure execution denied", "linking the remote broker to another computer is not allowed.");
							break;
						}
					case 8:
						{
							oWriter << m_oBroker->getExistingModuleInstanceIds();
							break;
						}
					case 9:
						{
							int nTemp;
							std::string sTemp;
							args >> nTemp >> sTemp;					
							if (!Sys::isSecure() || !m_oBroker->isRemoteInstance(fromModuleId))
								m_oBroker->changeModuleName(nTemp, sTemp);
							else throw new SBWApplicationException("insecure execution denied", "changing the remote registry is not allowed");
							break;
						}
					case 10:
						{
							int nTemp1, nTemp2;
							args >> nTemp1 >> nTemp2;					
							oWriter << m_oBroker->getServiceDescriptor(nTemp1, nTemp2);
							break;
						}
					case 11:
						{
							int nTemp;
							std::string sTemp;
							args >> nTemp >> sTemp;					
							oWriter << m_oBroker->getServiceDescriptor(nTemp, sTemp);
							break;
						}
					case 12:
						{
							int nTemp;
							args >> nTemp;					
							oWriter << m_oBroker->getServiceDescriptors(nTemp);
							break;
						}
					case 13:
						{
							std::string sTemp;
							args >> sTemp;					
							oWriter << m_oBroker->getServiceDescriptors(sTemp);
							break;
						}
					case 14:
						{
							std::string s1,s2,s3,s4;
							int n1;
							args >> s1 >> s2 >> n1 >> s3 >> s4;					
							if (!Sys::isSecure() || !m_oBroker->isRemoteInstance(fromModuleId))
								m_oBroker->registerModule(s1,s2,n1,s3,s4);
							else throw new SBWApplicationException("insecure execution denied", "changing the remote registry is not allowed");
							break;
						}
					case 15:
						{
							std::string s1,s2,s3,s4,s5;
							args >> s1 >> s2 >> s3 >> s4 >> s5;					
							if (!Sys::isSecure() || !m_oBroker->isRemoteInstance(fromModuleId))
								m_oBroker->registerService(s1,s2,s3,s4,s5);
							else throw new SBWApplicationException("insecure execution denied", "changing the remote registry is not allowed");
							break;
						}
					case 16:
						if (!Sys::isSecure() || !m_oBroker->isRemoteInstance(fromModuleId))
							m_oBroker->shutdownBroker();
						else 
							throw new SBWApplicationException("insecure execution denied", "shutdown of remote broker is not supported.");
						break;
					case 17:
						{
							std::string s1;
							args >> s1;					
							if (!Sys::isSecure() || !m_oBroker->isRemoteInstance(fromModuleId))
								m_oBroker->unregisterModule(s1);
							else throw new SBWApplicationException("insecure execution denied", "changing the remote registry is not allowed");
							break;
						}
					case 18:
						{
							oWriter << m_oBroker->getListOfModules();
							break;
						}
					case 19:
						{
							int n1,n2;
							args >> n1 >> n2;
							oWriter << m_oBroker->getMethodIds(n1,n2);
							break;
						}
					case 20:
						{
							string s1,s2;
							args >> s1 >> s2;
							oWriter << m_oBroker->getMethodIds(s1,s2);
							break;
						}

					default:
						TRACE("the method is not yet supported: " << methodId)
							break;
					}
					return  oWriter;
				} 
				else
				{
					MethodImpl *oMethod = srvImpl->getMethod(methodId);
					if (oMethod != NULL)
						return oMethod->invoke(Module(fromModuleId), args);
				}
			}
			catch (SBWException *e) 
			{	
				throw e;
			}
			catch (...) 
			{
				throw new SBWApplicationException("error calling method.");
			}
		}
		return DataBlockWriter();
	}
}
void ObjectOrientedReceiver::setModuleId(int moduleId)
{
	this -> moduleId = moduleId;
}
std::vector<std::string> ObjectOrientedReceiver::getServices()
{
	std::vector<std::string> sResult;

	for (unsigned int i = 0; i < services.size(); i++)
	{
		ServiceImpl *service = services[i];

		if (service == NULL)
			sResult.push_back("");
		else
			sResult.push_back(service->getName());
	}

	return sResult;
}
ServiceImpl *ObjectOrientedReceiver::getService(int serviceId)
{
	if (serviceId < 0 || serviceId >= numServices) {
		std::stringstream sTemp;
		sTemp << "Unknown service id " << serviceId;
		throw new SBWApplicationException(sTemp.str());
	}

	ServiceImpl *service = services[serviceId];

	if (service == NULL)
	{
		std::stringstream sTemp;
		sTemp << "Service id " << serviceId << " no longer supported"
			<< "The service identifier " << serviceId << " is not known to this"
			<<	 " module.  Either it is invalid or the module no longer"
			<<  "offers that service.";
		throw new SBWApplicationException(sTemp.str());
	}
	else
		return service;
}
std::vector<std::string> ObjectOrientedReceiver::getBrokerMethods() 
{
	if (m_oMethods.size() == 0)
	{
		m_oMethods.push_back("string getVersion()");
		m_oMethods.push_back("{}[] findServices(string, boolean)");
		m_oMethods.push_back("{} getModuleDescriptor(string, boolean)");
		m_oMethods.push_back("{} getModuleDescriptor(int)");
		m_oMethods.push_back("{}[] getModuleDescriptors(boolean, boolean)");
		m_oMethods.push_back("int getModuleInstance(string)");
		m_oMethods.push_back("string[] getServiceCategories(string)");
		m_oMethods.push_back("int linkBroker(string)");
		m_oMethods.push_back("int[] getExistingModuleInstanceIds()");
		m_oMethods.push_back("void changeModuleName(int, string)");
		m_oMethods.push_back("{} getServiceDescriptor(int, int)");
		m_oMethods.push_back("{} getServiceDescriptor(int, string)");
		m_oMethods.push_back("{}[] getServiceDescriptors(int)");
		m_oMethods.push_back("{}[] getServiceDescriptors(string)");
		m_oMethods.push_back("void registerModule(string, string, int, string, string)");
		m_oMethods.push_back("void registerService(string, string, string, string, string)");
		m_oMethods.push_back("void shutdownBroker()");
		m_oMethods.push_back("void unregisterModule(string)");
		m_oMethods.push_back("{} getListOfModules()");
		m_oMethods.push_back("{} getMethodIds(int,int)");
		m_oMethods.push_back("{} getMethodIds(string,string)");
	}
	return m_oMethods;
}
std::vector<std::string> ObjectOrientedReceiver::m_oMethods;
