#ifndef SBW_OBJECT_ORIENTED_RECEIVER
#define SBW_OBJECT_ORIENTED_RECEIVER

#include "Broker.h"

#include "SBW/DataBlockWriter.h"
#include "SBW/DataBlockReader.h"
#include "SBW/ServiceImpl.h"
#include "SBW/Receiver.h"

#include "BrokerInterface.h"


#include <string>
#include <vector>

namespace SystemsBiologyWorkbench
{
	namespace Broker
	{
		
		class ObjectOrientedReceiver :  public SystemsBiologyWorkbench::Receiver
		{
		public:
			ObjectOrientedReceiver(std::string moduleName);
			void addService(std::string name, std::string displayName,
				std::string category, Broker *impl, std::string help);
			virtual DataBlockWriter receive(Integer fromModuleId, Integer serviceId,
				Integer methodId, DataBlockReader args);
			void setModuleId(int moduleId);

			static std::vector<std::string> getBrokerMethods();

		private:
			/**
			* Returns an array of strings, one for each service implemented by this
			* receiver.  This is intended to be used for the GET_SERVICES_METHOD
			* and is not a high performance item.
			* <p>
			* @return an array of strings listing the services offered
			**/
			std::vector<std::string> getServices();
			ServiceImpl *getService(int serviceId);
			/** Contains ServiceImpl implementing services. **/
			std::vector<ServiceImpl *> services;

			/** Number of services in 'services' array, cached for speedy lookup. **/
			int numServices;

			/** Name of the module that this receiver represents. **/
			std::string moduleName;

			/** Module id of the module that this receiver represents. **/
			int moduleId;

			// broker implemetation
			SystemsBiologyWorkbench::Broker::Broker  *m_oBroker;

			static std::vector<std::string> m_oMethods;
		};

	}
}
#endif
