#ifndef SBW_BROKER_SRANDOM
#define SBW_BROKER_SRANDOM

#include <string>

#define ALG_NULL  0// No digest
#define ALG_SHA1  1// SHA1 Digest
#define ALG_MD5   2// MD5 digest
#define SHA1DigestLength  20 // 160 bits
#define seedLength  20


namespace SystemsBiologyWorkbench
{
	namespace Broker
	{
		// A test hash algorithm
		class SHA1Hash {
		public:
			static std::string name;

			// Algorithm types

			SHA1Hash();
		private: 
			static std::string myname;
		public:
			std::string getAlgorithmName();
		private:
			bool debug;			
		public:
			void setDebug() { debug = true; }			
		public:
			void clearDebug() { debug = false; }
		private:
			int blockLength;
			int nStates;
			int *block;
			int blockIndex;
			int *dd;
			int *state;
			long count;
			char type;
			char len;

			// Temporary variables used in R0-R4
			int tmp;
		public: 
			char getAlgorithm();			
			char getDigestLength();			
			void reset();			
		private:
			void R0(int data[], int v, int w, int x, int y, int z, int i);
			void R1(int data[],	int v, int w, int x, int y, int z, int i);
			void R2(int data[],	int v, int w, int x, int y, int z, int i);
			void R3(int data[],	int v, int w, int x, int y, int z, int i);
			void R4(int data[],	int v, int w, int x, int y, int z, int i);
			void transform();
			char *digest();
		public:
			SHA1Hash* clone();
			void update(char buf[], int off, int len);
			int doFinal(char inBuf[], int inOff,int inLen, char outBuf[],int outOff);
		};
		class SRandom {
		private:
			static char seed[16];
			SHA1Hash *sha1;
			long count;
			int timeLength;

		private:
			void PRandom();
		public:
			SRandom();
			static int longSize;
			SRandom(long lseed);

			std::string whoami() { return "Jxta Random"; }			
		private:
			char *getChurn();
		public:
			void generateData(char buffer[], int offset, int length);
			char *generateData(int length);
			void setSeed(char buffer[], int offset, int length);
			void setSeed(long lseed);
		protected:
			int next(int bits);
		public:
			void nextBytes(char buff[]);			
			static int intSize;

			int nextInt();
			long nextLong();
		};
	}
}

#endif

