#ifndef WIN32
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#endif

#include "ServerSocket.h"
#include "SBW/SBWException.h"
#include "SBW/SBWCommunicationException.h"

using namespace SystemsBiologyWorkbench;
using namespace SystemsBiologyWorkbench::Broker;

ServerSocket::ServerSocket (int nPort, std::string sHost /*= ""*/)
{
	m_nPort = nPort;

	// create socket for connections
	if ((m_nServerSocket = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0)
		throw new SBWCommunicationException("socket() failed ...");

	// consturct adress structure
	memset(&oServerAddr, 0, sizeof(oServerAddr));
	oServerAddr.sin_family = AF_INET;
	oServerAddr.sin_addr.s_addr = htonl(INADDR_ANY);
	oServerAddr.sin_port = htons(m_nPort);

	// bind to local address
	if (bind(m_nServerSocket, (struct sockaddr *) &oServerAddr, sizeof(oServerAddr)) < 0)
		throw new SBWCommunicationException("bind() failed ...");

	// listen for connections
	if (listen(m_nServerSocket, SOMAXCONN) < 0 )
		throw new SBWCommunicationException("listen() failed ...");

	m_nClientSocket = 1;	
	m_nClientLength = 0;
}

SOCKET ServerSocket::acceptClient()
{
	m_nClientLength = sizeof(this->oClientAddr);
	memset(&oClientAddr, 0, sizeof(oClientAddr));
#ifndef WIN32
	m_nClientSocket = accept( (SOCKET)m_nServerSocket, (struct sockaddr *) &oClientAddr, (socklen_t *) &m_nClientLength);
#else
	m_nClientSocket = accept( (SOCKET)m_nServerSocket, (struct sockaddr *) &oClientAddr, &m_nClientLength);
#endif
	if (m_nClientSocket < 0)
		throw new SBWCommunicationException("accept() failed ...");

	return m_nClientSocket;
}

std::string ServerSocket::getCurrentClientAddress()
{
	std::string sAddress( inet_ntoa(oClientAddr.sin_addr ));
	return sAddress;
}
