#!/usr/bin/perl -w
use strict;
use DBI;
use XML::Writer;
use IO::File;
use Getopt::Std;

my %option = ();
my ($dsn) = "DBI:mysql:biowarehouse:localhost";
my ($username) = "biospice";
my ($password) = "bio.spice";
my ($Pathway) = "wild_type";

getopts("hd:u:p:o:P:",\%option);

sub usage() {
  print STDERR "biowarehouse2sbml.pl -d dsn -u username -p password -P pathway\n";
  exit(1);
}



if ($option{h}) {
  usage();
}
if ($option{d}) {
  $dsn = $option{d};
}
if ($option{u}) {
  $username = $option{u};
}
if ($option{p}) {
  $password = $option{p};
}


#if ($option{o}) {
#  $output = new IO::File(">$option{o}");
#}

##print  $output "dsn: ", $dsn, "  username: ", $username, "  password ", $password;
#$dsn = "DBI:mysql:biowarehouse:localhost";
#$username = "biospice";
#$password = "bio.spice";
#print  $output "dsn: ", $dsn, "  username: ", $username, "  password ", $password;

my ($dbh, $sth);
my (@ary);

$dbh = DBI->connect( $dsn, $username, $password, { RaiseError => 1} );




##my $writer = new XML::Writer(OUTPUT => STDOUT, DATA_MODE => 1, DATA_INDENT => 2);
my $writer = new XML::Writer(DATA_MODE => 1, DATA_INDENT => 2);
my %sql =     ("DBID" => qq{
			    select XID, OtherWID
			    from DBID
			   },
	       "fetch-dataset" => qq{
                           select XID, Name, Version, HomeURL
                           from DBID, DataSet
                           where DataSet.WID = DBID.OtherWID
                           and DBID.XID = ?
                    },
	    "fetch-list-of-chemicals" => qq{
                            select distinct XID, Compartment
                            from DBID, Reactant
                            where DBID.OtherWID = Reactant.OtherWID
                            and   Reactant.ReactionWID in (%s) 
                            union
                            select distinct XID, Compartment
                            from DBID, Product
                            where DBID.OtherWID = Product.OtherWID
                            and   Product.ReactionWID in (%s)   
                          },
	    "fetch-list-of-transported-chemicals" => qq{
                            select distinct XID
                            from DBID, Reactant
                            where DBID.OtherWID = Reactant.OtherWID
                            and   Reactant.ReactionWID in (%s) 
                            and Reactant.Compartment != "cytoplasm"
                            union
                            select distinct XID
                            from DBID, Product
                            where DBID.OtherWID = Product.OtherWID
                            and   Product.ReactionWID in (%s)   
                            and Product.Compartment != "cytoplasm"
                          },

            "fetch-reactions-of-pathway" => qq {
                            select ReactionWID 
                            from PathwayReaction, DBID
                            where DBID.OtherWID = PathwayReaction.PathwayWID
                            and DBID.XID = ?
                           },

	       "fetch-all-enzymatic-reactions" => qq {
                             select distinct ReactionWID
                             from EnzymaticReaction
                           },
	       "fetch-list-of-proteins" => qq {
                            select protein.XID, Protein.Name
                            from DBID protein, Protein, EnzymaticReaction
                            where protein.OtherWID = Protein.WID
                            and Protein.WID = EnzymaticReaction.ProteinWID
                            and EnzymaticReaction.ReactionWID in (%s)
                           },
	       "fetch-list-of-compartments" => qq{
                            select distinct Compartment 
                            from Reactant
                            where Compartment is not null
                            and ReactionWID in (%s)
                            union
                            select distinct Compartment 
                            from Product
                            where Compartment is not null
                            and ReactionWID in (%s)

                           },
	       "fetch-list-of-reactions" => qq{
	                    select distinct XID, ReactionDirection
	                    from DBID, EnzymaticReaction
                            where DBID.OtherWID = ReactionWID
                            and ReactionWID in (%s)
                            },
               "fetch-enzymatic-reactions" => qq{
                            select  rxn.XID, enzyme.XID, complex.XID ReactionDirection
                            from DBID rxn, DBID enzyme, DBID complex, EnzymaticReaction
                            where rxn.OtherWID = ReactionWID
                            and  enzyme.OtherWID =  ProteinWID
                            and complex.OtherWID = ComplexWID
                            },
               "fetch-list-of-reactants" => qq {
                            select reactant.XID, Coefficient, Compartment
                            from DBID reactant, Reactant, DBID rxn
                            where Reactant.OtherWID = reactant.OtherWID
                            and ReactionWID = rxn.OtherWID
                            and rxn.XID = ?
               },
               "fetch-list-of-products" => qq {
                            select product.XID, Coefficient, Compartment
                            from DBID product, Product, DBID rxn
                            where Product.OtherWID = product.OtherWID
                            and ReactionWID = rxn.OtherWID
                            and rxn.XID = ?
               },
               "fetch-list-of-modifiers" => qq {
                            select protein.XID
                            from DBID protein, EnzymaticReaction, DBID rxn
                            where protein.OtherWID = ProteinWID
                            and ReactionWID = rxn.OtherWID
                            and rxn.XID = ?
               });

if ($option{P}) {
  $Pathway = $option{P};

} 
my $rxn_list = fetch_reactions_of_pathway( $dbh, \%sql, $Pathway );

sub fetch_list_of_compartments {
  my ($dbh, $sql, $rxn_list) = @_;
  my $compartment;
  my @list_of_compartments = ();
  my $statement = sprintf( $$sql{'fetch-list-of-compartments'}, $rxn_list, $rxn_list );
  my $sth = $dbh->prepare( $statement ); 
  $sth->execute();
  $sth->bind_col(1, \$compartment);
  while ($sth->fetch ) {
    push @list_of_compartments, $compartment;
  }
  return @list_of_compartments;
}

sub fetch_reactions_of_pathway { 
  my ($dbh, $sql, $pathway) = @_; 

  my $sth = $dbh->prepare( $$sql{"fetch-reactions-of-pathway"} ); 
  $sth->execute($pathway); 
  my $rxn; 
  my @reactions; 
  $sth->bind_col(1, \$rxn); 
  while( $sth->fetch ) { 
    push @reactions, $rxn; 
  }
  return join(',', @reactions );
}

sub fetch_all_rxns {
  my ($dbh, $sql, $pathway) = @_;
  
  my $sth = $dbh->prepare( $$sql{"fetch-all-enzymatic-reactions"} );
  $sth->execute();
  my $rxn;
  my @reactions;
  $sth->bind_col(1, \$rxn);
  while ($sth->fetch ) {
    push @reactions, $rxn;
  }
  return join(',',@reactions );
}

sub print_model {
  my ($writer, $dbh, $sql, $Pathway) = @_;
  $writer->startTag( 'model', 'id' => $Pathway );
}

sub print_sbml {
  my ($writer, $dbh, $sql, $rxn_list) = @_;
  my $namespace = "http://www.sbml.org/sbml/level2";
  my $org_id = $Pathway;
  my $name = "Biowarehouse generated Ecoli Version 8.0";
 
  $writer->xmlDecl( 'UTF-8' );
  $writer->startTag( 'sbml', 'xmlns' => $namespace, 'version' => 1, 'level' => 2);
 print_model( $writer, $dbh, $sql, $Pathway );
#  $writer->startTag( 'model', 'id' => $org_id, 'name' => $name );
  print_list_of_compartments( $writer, $dbh, $sql, $rxn_list );  
  print_list_of_species( $writer, $dbh, $sql, $rxn_list );
  print_list_of_reactions( $writer, $dbh, $sql, $rxn_list );
  $writer->endTag( 'model' );
  $writer->endTag( 'sbml' );
}

print_sbml($writer, $dbh, \%sql, $rxn_list );

sub xid2sid {
  my $xid = shift;
  my $sid = $xid;
  $sid =~ s/([^0-9A-Za-z_])/"__" . ord( $1 ) . "__"/ge;
  $sid =~ s/^([0-9])/_$1/;
  
  return $sid;
}

sub sid2xid {
  my $sid = shift;
  my $xid = $sid;
  $xid =~ s/^_(\d)/$1/;
  $xid =~ s/__(\d+)__/chr($1)/eg;
  
  return $xid;
}

sub fetch_species_reference {
  my ($dbh, $sth, $rxn) = @_;
  my ($specie, $stoichiometry, $compartment);
  my @speciesReference = ();
  
  $sth->execute( $rxn );
  $sth->bind_columns( \($specie, $stoichiometry, $compartment) );
  while ($sth->fetch) {
    my %row = ();
    if ($compartment ne "cytoplasm") {
      $row{specie} = $specie . "_$compartment";
    } else {
      $row{specie} = $specie;
    }
    $row{stoichiometry} = $stoichiometry;
    push @speciesReference, \%row;
  }
  return @speciesReference;
}

sub fetch_modifier_species_reference {
  my ($dbh, $sth, $rxn) = @_;
  my ($specie, $coefficient, $compartment);
  my @modifierSpeciesReference = ();
  
  $sth->execute( $rxn );
  $sth->bind_col(1, \$specie );
  while ($sth->fetch) {
      push @modifierSpeciesReference, $specie;
  }
  return @modifierSpeciesReference;
}

sub fetch_list_of_reactions {
  my ($dbh, $sql, $list_of_rxns) = @_;
  my ($rxn, $rxn_direction);
  my @reactions;
  my $statement = sprintf $$sql{'fetch-list-of-reactions'}, $list_of_rxns;
  my $sth = $dbh->prepare( $statement );
  $sth->execute( );
  $sth->bind_columns(\$rxn, \$rxn_direction);
  while ($sth->fetch) {
    my %row = ();
    if ($rxn_direction eq "REVERSIBLE") {
      $row{reversible} = "true";
    } else {
      $row{reversible} = "false";
    }
    $row{id} = $rxn;
    push @reactions, \%row;
  }
  return @reactions;
}

sub print_species_reference {
  my ($dbh, $sth, $rxn) = @_;
  foreach my $speciesReference (fetch_species_reference( $dbh, $sth, $rxn )) {
    $writer->emptyTag( 'speciesReference',
		       'species' => xid2sid( $$speciesReference{specie} ),
		       'stoichiometry' => $$speciesReference{stoichiometry});
  }
}

sub print_modifier_species_reference {
  my ($dbh, $sth, $rxn) = @_;
  foreach my $speciesReference (fetch_modifier_species_reference( $dbh, $sth, $rxn )) {
    $writer->emptyTag( 'modifierSpeciesReference',
		       'species' => xid2sid( $speciesReference ));
  }
}
    


# does not do names or stoichiometries or reaction direction yet
sub print_list_of_reactions {
  my ($writer, $dbh, $sql, $rxn_list) = @_;
  my $rth = $dbh->prepare( $$sql{'fetch-list-of-reactants'} );
  my $pth = $dbh->prepare( $$sql{'fetch-list-of-products'} );
  my $mth = $dbh->prepare( $$sql{'fetch-list-of-modifiers'} );

  $writer->startTag('listOfReactions');
  foreach my $rxn (fetch_list_of_reactions( $dbh, $sql, $rxn_list )) {
    $writer->startTag( 'reaction',
		       'id' => xid2sid( $$rxn{id} ),
		       'metaid' => "_$$rxn{id}",
		       'name' => $$rxn{id},
                       'reversible' => $$rxn{reversible});

#    print_flux_limits( $writer, $$rxn{reversible} );
		       
    my @reactantlist = fetch_species_reference ( $dbh, $rth, $$rxn{id} );
    if (scalar(@reactantlist) > 0) {
      $writer->startTag( 'listOfReactants' );
      print_species_reference( $dbh, $rth, $$rxn{id} );
      $writer->endTag();
    }
    
    my @productlist = fetch_species_reference ( $dbh, $pth, $$rxn{id} );
    if (scalar(@productlist) > 0) {
      $writer->startTag( 'listOfProducts' );
      print_species_reference( $dbh, $pth, $$rxn{id} );
      $writer->endTag();
    }

#    my @modifierlist = fetch_modifier_species_reference ( $dbh, $mth, $$rxn{id} );
#    if (scaler(@modifierlist) > 0) {
      $writer->startTag( 'listOfModifiers' );
      print_modifier_species_reference( $dbh, $mth, $$rxn{id} );
      $writer->endTag();
#   }
    $writer->endTag( 'reaction' );
   }
  $writer->endTag( 'listOfReactions' );
}

sub print_flux_limits {
  my ($writer, $reversible) = @_;

  $writer->startTag( 'annotation',
		       'xmlns:flux' => 'http://arep.med.harvard.edu/fluxns' );
  if ( $reversible eq 'false' ) {
    $writer->emptyTag( 'flux:limitation',
		       'lower' => '0',
		       'upper' => 'INF' );
  } else {
    $writer->emptyTag( 'flux:limitation',
		       'lower' => '-INF',
		       'upper' => 'INF' );
  }
  $writer->endTag();
  
}
			  
sub print_list_of_species {
  my ($writer, $dbh, $sql, $rxn_list) = @_;
  
  $writer->startTag('listOfSpecies');
  foreach my $species (fetch_list_of_species($dbh, $sql, $rxn_list)) {
    $writer->emptyTag( 'species', 
		       'id' => xid2sid( $$species{chemical} ), 
		       'metaid' =>  "_$$species{chemical}",
		       'name' => $$species{chemical},
                       'compartment' => $$species{compartment} ); 
  }
  $writer->endTag();
}

sub fetch_list_of_species {
  my ($dbh, $sql, $rxn_list) = @_;
  my @list_of_species = ();
  push @list_of_species, fetch_list_of_chemicals( $dbh, $sql, $rxn_list );
  push @list_of_species, fetch_list_of_transported_chemicals( $dbh, $sql, $rxn_list );
#  push @list_of_species, fetch_list_of_genes( $dbh, $sql, $rxn_list );
  push @list_of_species, fetch_list_of_proteins( $dbh, $sql, $rxn_list );
  return @list_of_species;
}

sub print_list_of_compartments {
  my ($writer, $dbh, $sql, $rxn_list) = @_;
  $writer->startTag( "listOfCompartments" );
  foreach my $compartment (fetch_list_of_compartments( $dbh, \%sql, $rxn_list ) ) {
    $writer->emptyTag( "compartment", 'id' => $compartment );
  }
  $writer->endTag();
}



sub fetch_list_of_genes {
  my ($dbh, $sql, $rxn_list) = @_;
  my $sth = $dbh->prepare( sprintf $$sql{"fetch-list-of-genes"}, $rxn_list )
  
}

sub fetch_list_of_proteins {
  my ($dbh, $sql, $rxn_list) = @_;
  my $statement = sprintf $$sql{"fetch-list-of-proteins"}, $rxn_list;
  my $sth = $dbh->prepare( $statement );
  my @proteins = ();
  my $protein;
  $sth->execute();
  $sth->bind_col(1, \$protein);
  while( $sth->fetch ) {
    my %row = ();
    $row{chemical} = $protein;
# hack - hardcoded compartment
    $row{compartment} = "cytoplasm";
    push @proteins, \%row;
  }
  return @proteins;
}




sub fetch_list_of_chemicals {
  my ($dbh, $sql, $rxn_list) = @_;
  my $statement = sprintf $$sql{'fetch-list-of-chemicals'}, ($rxn_list, $rxn_list);
  my $sth = $dbh->prepare( $statement );
  my ($chemical, $compartment);
  $sth->execute();
  my @chemicals = ();
  $sth->bind_columns(\($chemical, $compartment) );
  while( $sth->fetch ) {
    my %row = ();
    if ( $compartment ne "cytoplasm" ) {
      $row{chemical} = $chemical . "_$compartment";
    } else {
      $row{chemical} = $chemical;
    }
    $row{compartment} = $compartment;
    push @chemicals, \%row;
  }
  return @chemicals;
  
}

# Hack. hardcoded compartment
sub fetch_list_of_transported_chemicals {
  my ($dbh, $sql, $rxn_list) = @_;
  my $statement = sprintf $$sql{'fetch-list-of-transported-chemicals'}, ($rxn_list, $rxn_list);
  my $sth = $dbh->prepare( $statement );
  my $chemical;
  $sth->execute();
  my @chemicals = ();
  $sth->bind_col(1, \$chemical);
  while( $sth->fetch ) {
    my %row = ();
    $row{chemical} = $chemical . "_periplasm";
    $row{compartment} = "periplasm";
    push @chemicals, \%row;
  }
  return @chemicals;
  
}

sub create_list_of_reactions{}
sub create_list_of_products {}
sub create_list_of_reactants {}
sub create_list_of_modifiers {}


