#include "QpGenData.h"
#include "QpGenVars.h"
#include "QpGenResiduals.h"
#include "GondzioSolver.h"
#include "QpGenSparseMa27.h"
#include <iostream.h>
#include <math.h>
#include <string>
#include <fstream>
#include "qps.h"

#include "data.h"
// #include "e.h"

void compare_I(int * x, int * y, int k, string s);
void compare_C(char * x, char * y, int k, string s);
void compare_D(double * x, double * y, int k, string s);

int main (int a, char * b[])
{
  
	int quiet = 1;

	QPS X(b[1]);
	X.Export_I( nxD, myD,  mzD, nnzQD, nnzAD,nnzCD);

	cD = new double[nxD];


	xuppD= new double[nxD];
	ixuppD= new char[nxD];;

	xlowD= new double[nxD];
	ixlowD= new char[nxD];

	bAD       = new double[myD];;
	irowAD    = new int[nnzAD];
	jcolAD    = new int[nnzAD];
	dAD       = new double[nnzAD];

	string *Cname = new string[nxD];
        string *Rname = new string[myD];

	cout << " nx = " << nxD << endl;
	cout << " my = " << myD << endl;
										
	X.Export_D(cD,xlowD,ixlowD,xuppD,ixuppD,irowAD,jcolAD,dAD,bAD,Cname,Rname);
	sort(irowAD, jcolAD, nnzAD, myD, dAD);

	/*compare_D(&c[0], cD, nxD, "Objective function " );
	compare_D(&xupp[0], xuppD, nxD, "Low Bound values " ) ;
	compare_C(&ixupp[0], ixuppD, nxD, "Upper Bound flag " );
	compare_C(&ixlow[0], ixlowD, nxD, "Lower Bound flag " );

	cout << " ---- Lower Bound Flag for ixlowD[100] = " << ixlowD[100] << endl;

	for (int i=0;i<nnzA;i++)
	{
		if ((fabs(dAD[i] - dA[i]) > 0.00001)
				|| (irowAD[i] != irowA[i]) 
			        || (jcolAD[i] != jcolA[i]))
		{
			cout << "[(" << irowAD[i] << "," << jcolAD[i] << "): " << dAD[i] << "] v.s. " ;
			cout << "[(" << irowA[i]  << "," << jcolA[i]  << "): " << dA[i]  << "]" << endl;
		}
	}
*/
	// set up dimensions
 	QpGenSparseMa27 * qp = new QpGenSparseMa27( nxD, myD, mzD, nnzQD, nnzAD, nnzCD );

	// make data
	
  	QpGenData      * prob = (QpGenData *) qp->copyDataFromSparseTriple(
        cD,      irowQD,  nnzQD,   jcolQD,  dQD,
        xlowD,   ixlowD,  xuppD,   ixuppD,
        irowAD,  nnzAD,   jcolAD,  dAD,     bAD,
        irowCD,  nnzCD,   jcolCD,  dCD,
        clowD,   iclowD,  cuppD,   icuppD );
/*
	c[742] = -1.0;
  	QpGenData      * prob = (QpGenData *) qp->copyDataFromSparseTriple(
        &c[0],      &irowQ[0],  nnzQ,   &jcolQ[0],  &dQ[0],
        &xlow[0],   &ixlow[0],  &xupp[0],   &ixupp[0],
        &irowA[0],  nnzA,   &jcolA[0],  &dA[0],     &bA[0],
        &irowC[0],  nnzC,   &jcolC[0],  &dC[0],
        &clow[0],   &iclow[0],  &cupp[0],   &icupp[0] );
*/
	// make variable, given input data
  	QpGenVars      * vars  = (QpGenVars *) qp->makeVariables( prob );
  	QpGenResiduals * resid = (QpGenResiduals *) qp->makeResiduals( prob );
  
	// solver object
  	GondzioSolver  * s     = new GondzioSolver( qp, prob );
  
  	if( !quiet ) s->monitorSelf();
  	int ierr = s->solve(prob,vars, resid);
 	cout << "ierr = " << ierr << endl; 
  	if( ierr == 0 ) {
    		cout.precision(4);
    		cout << "Solution: \n";
    		vars->x->writefToStream( cout, "x[%{index}] = %{value}" );
  	} 
	else 
	{
    		cout << "Could not solve the problem.\n";
  	}
	
	exit(1);

	// for (int i=0;i<nx;i++)
	// {
	//	cout << "Primal name (" << i << "): " << Cname[i];
	//       cout << "\t\t" << xlow[i] << "\t" << xupp[i]	<< endl;
	// }
	/*
        for (int i=0;i<my;i++)
       		cout << "Row name    (" << i << "): " << Rname[i] << endl;	       
	*/
	return 0;
}

void compare_I(int * x, int * y, int k, string s)
{
	for (int i=0; i< k; i++)
		if (x[i] != y[i])
			cout << s << " :" << i << " (" << x[i] << "," << y[i] << ") " << endl; 
}

void compare_C(char * x, char * y, int k, string s)
{
	for (int i=0; i< k; i++)
		if (x[i] != y[i])
			cout << s << " :" << i << " (" << (char ) x[i] << "," << y[i] << ") " << endl;
}

void compare_D(double * x, double * y, int k, string s)
{
	for (int i=0; i< k; i++)
		if (fabs(x[i] -  y[i]) > 0.0001)
			cout << s << " :" << i <<" (" << x[i] << "," << y[i] << ") " << endl;
}

