/*******************************************
File qps.h: Header file for Class QPS

This class is used to parse the the input
file in the QPS (or MPS) format and return
input data  in the appropriate form 
specified by the OOQP package
********************************************/

#ifndef _QPS_H
#define _QPS_H

const int NROWS    = 1000;		// maximun number of matrix row allowed
const int NCOLUMNS = 1000;		// maximun number of matrix columns allowed
const int NNB      = 60;		// maximun number of edges for each node allowed 

class Cnode
{
   public:
        Cnode();
        ~Cnode();
	void insert(string s, double v);
	string id;
	string row_name[NNB];
        double values[NNB];
        int nr;
} ;

class QPS
{ 
  public: 
	// Constructors: take filename as argument
        QPS(char * ifilename);

	void Export_I(int & nx, int &my, int &mz, int &nnzQ, int &nnzA, int &nnzC);
	void Export_D(double *c, double *xlow, char *ixlow, double *xupp, char *ixupp, 
			         int *irowA, int *jcolA, double *dA, double *b, string *Cname, string *Rname);

	// destructor for QPS class
        ~QPS();
  private:	
	// this function must be called first. Read in the NAME card
	void START();

	// read in the ROWS section of the input file. Make sure that this
	// is called imediately after START()
	void ROWS();

	// read in the COLUMN section of the input file. Make sure that this
	// is called imediately after ROWS()
	void COLUMNS();

	// read in the RANGES section of the input file.
	void RANGES();

	// read in the RHS section of the input file. For the current purpose
	// this function set vector b to zero:  S*v = 0
	void RHS();

	// read in the BOUND section of the input file
	void BOUNDS();

	void QSECTION();

	void CSECTION();

	// read in the QUADOBJ section of the input file
	void QUADOBJ();

	// set all flag to make sure that the reading is fine
	void ENDATA();

	// this function will convert all read data into appropriate array
	void PROCESS();

	void print_col_vs_row();
	void print_data();
	void function_selection(int i);
	inline int set_calling_flag(const string & s);

	string clean_word(string ss);

	string problem_name;
	string Obj_var;				// name of the objective variable
	double Obj_fun[NCOLUMNS];		// vector of objective function, used internally
	ifstream input;				// input MPS file name
	char Ctype[NROWS];			// constraint type: E, G, L, N...
	string row_name[NROWS];			// row name
	string bound_name[NROWS];		// bounded variables (flux names)
	Cnode  col_name[NCOLUMNS];		// colume node vs. row node
	int    nrows, ncols;			// Current number of rows and culumns for a given QPS file
	int    glinenum;			// global number of line read so far: used to check number of lines read
	int    N_BOUND;				// Number of bounds in the MPS file
	int search_row(const string s);
	int search_bound(const string s);
	int search_col(string s);
	int BFL[NROWS], BFU[NROWS];		// BF = bound type: 0=FREE, 1=otherwise; L=lower bound, U=upper
	double LB[NROWS], UB[NROWS];		// Lower bound and upper bound of flux vector
						// LB ~= XLOW; UB ~=XUPP
	// Now all following variables below are for the OOQP
	double *C;		// objective function
	int    NX;		// size of objective function;

	char *IXUPP, *IXLOW;	// bound flag for primal vectors
	double *XUPP, *XLOW;	// bound values for primal vector
				// will copy LB-> XLOW; UP->XUPP

	int NNZA;		// size of vector A: number of non-zero values
	int *IROWA, *JCOLA;	// i-row and j-col indices of matrix A
	double *DA;		// Matrix A in 1D containing non-zero elements
	
	int MY;			// size of RHS vector b
	double *B;		// Vector B of the RHS

	// used variables, but needed for OOQP to work correctly
	int MZ;
	double *CLOW, *CUPP;
	char *ICLOW, *ICUPP;
	int NNZC;
	int *IROWC, *JCOLC;
	double *DC;

	// these for matrix Q used in quadratic programming
	int NNZQ;
	int *IROWQ, *JCOLQ;
	double *DQ;

}; 
#endif

