/* OOQP                                                               *
 * Authors: E. Michael Gertz, Stephen J. Wright                       *
 * (C) 2001 University of Chicago. See Copyright Notification in OOQP */

#include "QpGenData.h"
#include "QpGenVars.h"
#include "QpGenResiduals.h"
#include "GondzioSolver.h"
#include "QpGenSparseMa27.h"

#include <iostream.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>

const int nx   = 6;
// double    c[]  = { 1, -1, 1, 1, 1, 1 };
double    c[]  = { 1, 1, 0, 0, 1, 1 };

double  xupp[] = {  -1,   -1,      100, 100,            -1, -1 };
char   ixupp[] = {  1,   1,      1, 1,            1, 1 };

double  xlow[] = {  -100, -100,      1, 1,            -100, -100 };
char   ixlow[] = {  1,   1,          1, 1,            1,     1 };

const int nnzQ  = 0;
int   * irowQ  ; 
int   * jcolQ  ;
double *   dQ  ;


int myb        = 4;
double   b[]   = { 0, 0, 0, 0 };

int nnzA       = 12;
int  irowA[]   = { 0, 0, 0, 1, 1, 1, 2, 2, 2, 3, 3, 3 };
int  jcolA[]   = { 0, 1, 2, 0, 3, 4, 1, 3, 4, 2, 4, 5 };
double   dA[]  = { 1, 1, 1,-1, 1, 1,-1,-1, 1,-1,-1,-1 };

const int mz   = 0;
double *clow   = 0;
char  * iclow  = 0;

double * cupp  = 0;
char  * icupp  = 0;

const int nnzC = 0;
int   * irowC  = 0;
int   * jcolC  = 0;
double  *  dC  = 0;

int main( int argc, char * argv[] )
{
  int usage_ok = 1, quiet = 0;

  if( argc > 2 ) usage_ok = 0;
  if( argc == 2 ) {
    if( 0 == strcmp( "--quiet", argv[1] ) ) {
      quiet = 1;
    } else {
      usage_ok = 0;
    }
  } 
  if( !usage_ok ) {
    cerr << "Usage: " <<  argv[0] << " [ --quiet ]\n";
    return 1;
  }
    
  QpGenSparseMa27 * qp 
    = new QpGenSparseMa27( nx, myb, mz, nnzQ, nnzA, nnzC );
  
  QpGenData      * prob = (QpGenData * ) qp->copyDataFromSparseTriple(
        c,      irowQ,  nnzQ,   jcolQ,  dQ,
        xlow,   ixlow,  xupp,   ixupp,
        irowA,  nnzA,   jcolA,  dA,     b,
        irowC,  nnzC,   jcolC,  dC,
        clow,   iclow,  cupp,   icupp );

  QpGenVars      * vars 
    = (QpGenVars *) qp->makeVariables( prob );
  QpGenResiduals * resid 
    = (QpGenResiduals *) qp->makeResiduals( prob );
  
  GondzioSolver  * s     = new GondzioSolver( qp, prob );
  
  if( !quiet ) s->monitorSelf();
  int ierr = s->solve(prob,vars, resid);
  
  if( ierr == 0 ) {
    cout.precision(4);
    cout << "Solution: \n";
    vars->x->copyIntoArray(c);
    vars->x->writefToStream( cout, "x[%{index}] = %{value}" );
  } else {
    cout << "Could not solve the problem.\n";
  }
  double * qq = new double[5];

	qq[1] = 0.2;
	qq[2] = 0.9;
	qq[3] = 0.8;
	qq[4] = 1.0;

  cout << " ----- Distances ---- " << endl;

	c[0] = sqrt(1000.0*qq[1]*qq[2]/fabs(c[0]));
	c[1] = sqrt(1000.0*qq[1]*qq[3]/fabs(c[1]));
	c[2] = sqrt(1000.0*qq[1]*qq[4]/fabs(c[2]));
	c[3] = sqrt(1000.0*qq[2]*qq[3]/fabs(c[3]));
	c[4] = sqrt(1000.0*qq[2]*qq[4]/fabs(c[4]));
	c[5] = sqrt(1000.0*qq[3]*qq[4]/fabs(c[5]));




  for (int i=0;i<nx;i++) {
	cout << c[i] << endl;
  }
  return ierr;
}
