/* OOQP                                                               *
 * Authors: E. Michael Gertz, Stephen J. Wright                       *
 * (C) 2001 University of Chicago. See Copyright Notification in OOQP */

#include "OblioQpGen.h"
#include "QpGenData.h"
#include "QpGenSpLinsys.h"

#include "SparseLinearAlgebraPackage.h"
#include "SparseGenMatrix.h"
#include "SimpleVector.h"
#include "OblioSolver.h"
#include "SparseLinearAlgebraPackage.h"

OblioQpGen::OblioQpGen( int nx_, int my_, int mz_,
					  int nnzQ_, int nnzA_, int nnzC_ ) :
  QpGenSparseSeq( nx_, my_, mz_, nnzQ_, nnzA_, nnzC_ )
{
  la = SparseLinearAlgebraPackage::soleInstance();
}


LinearSystem * OblioQpGen::makeLinsys( Data * prob_in )
{
  QpGenData * prob = (QpGenData *) prob_in;
  int n = nx + my + mz;

  OblioSolver * solver =
    new OblioSolver( n, my + mz + nx + 2 * nnzQ + 2 * nnzA + 2 * nnzC );
  SparseGenMatrixOldMat = solver->makeSharedGenMatrix();

  SimpleVectorOld v(n);
  v->setToZero();
  Mat->setToDiagonal(v);

  prob->putQIntoAt( Mat, 0, 0 );
  prob->putAIntoAt( Mat, nx, 0);
  prob->putCIntoAt( Mat, nx + my, 0 );

  
  
  int info;
  Mat->symmetrize( info );
  assert( info == 0 ); // Has to be Ok, we allocated the matrix at max size.

  
  return new QpGenSpLinsys( this, la, Mat,
			    solver, nx, my, mz,
			    prob->ixlowerBound(), prob->ixupperBound(),
			    prob->islowerBound(), prob->isupperBound() );
}

