#include <iostream.h>
#include <string>
#include <fstream.h>
#include <sstream>
#include <math.h>

#include "qps.h"
#include "QpManipulator.h"
#include "cQpGenSparse.h"

const int PRINT_LEVEL=0;

using namespace std;
QpManipulator::QpManipulator()
{
	c=NULL; nx=0;
	irowQ=NULL; nnzQ=0; jcolQ=NULL; dQ=NULL;
	irowQMOMALP=NULL; nnzQMOMALP=0; jcolQMOMALP=NULL; dQMOMALP=NULL;
	irowQMOMAQP=NULL; nnzQMOMAQP=0; jcolQMOMAQP=NULL; dQMOMAQP=NULL;
        irowA=NULL; nnzA=0; jcolA=NULL; dA=NULL;
        b=NULL; my=0;
        irowC=NULL; nnzC=0; jcolC=NULL; dC=NULL; clow=NULL; mz=0; iclow=NULL; cupp=NULL; icupp=NULL;
}
QpManipulator::QpManipulator(char *mpsFileName, int &nnx, int &mmy, int &mmz)
{
	string inputfilename = (string) mpsFileName;
	int index = inputfilename.find(".mps",0);

	if (index >=0)
	{
		qps = new QPS(mpsFileName);
		setBasicData();
		nnx     = nx;
		mmy     = my;
		mmz     = mz;
	}

	else if (  (inputfilename.find(".xml",0) >=0) || (inputfilename.find(".sbml",0) >=0) )
	{
		// call sbmlReader object
		//
		SB = new sbmlReader(mpsFileName);
		// SB->print();
		extractDataFromSBML();

		nnx     = nx;
	        mmy     = my;
		mmz     = mz;
	  		
		// exit(1);
	}
}

void QpManipulator::setBasicData()
{
	qps->Export_I(nx, my,  mz, nnzQ,  nnzA,  nnzC);
	c = new double[nx];
        // x = new double[nx];

        XUPP = new double[nx];
        IXUPP= new char[nx];

        XLOW = new double[nx];
        IXLOW= new char[nx];

	b        = new double[my];
        irowA    = new int[nnzA];
        jcolA    = new int[nnzA];
        dA       = new double[nnzA];

	FluxName = new string[nx];
        MetaboliteName = new string[my];

	// now get the double-typed data
	qps->Export_D(c,XLOW,IXLOW,XUPP,IXUPP,irowA,jcolA,dA,b,FluxName,MetaboliteName);

	generateMatrixQforMOMA();
}
void QpManipulator::extractDataFromSBML()
{
	SB->getDimension(nx,my,nnzA);
	mz = 0;
	nnzQ = 0;
	nnzC = 0;

	c = new double[nx];
	XUPP = new double[nx];
	IXUPP= new char[nx];
	
	XLOW = new double[nx];
        IXLOW= new char[nx];

	b        = new double[my];
        irowA    = new int[nnzA];
        jcolA    = new int[nnzA];
        dA       = new double[nnzA];
	
	FluxName = new string[nx];
	MetaboliteName = new string[my];


	//	SB->getData(c, irowA, jcolA, dA ,IXLOW,XLOW,IXUPP,XUPP ,MetaboliteName,FluxName);
	for (int i=0; i<my; i++)
		b[i] = 0.0;
	generateMatrixQforMOMA();
	

	
	//for (int i=0; i< nnzA; i++)
	//	cout << "(row " << irowA[i] << ",col " << jcolA[i] << ") =\t" << dA[i] << endl;

	/*
 	cout << " -------- Objective function ------------------- " << endl;
	cout << "variable i, c[i], ixlow[i], xlow[i], ixupp[i], xupp[i] \n";
	for (int i=0; i<nx;i++)
	{
		cout << "variable [" << i << "]= " <<c[i] << ", " << (int) IXLOW[i] << ", " << XLOW[i] << ", " << (int) IXUPP[i] << ", " << XUPP[i] << ", " << FluxName[i]<< endl;
	}
	*/
	// for (int i=0; i<my;i++)
	// 	cout << "Metabolite " << i << "=\t" << MetaboliteName[i] << " --- b[i] = " << b[i] << endl;
	
}

void QpManipulator::writeFluxToSBML(double *fluxes, const int nelements)
{
	SB->writeToSBML(fluxes, nelements);
}

void QpManipulator::writeFluxToSBML(char * filename, double *fluxes, const int nelements)
{
	SB->writeToSBML(filename, fluxes, nelements);
}
	

void QpManipulator::initializeArraysAndCopyData(string *FN, double *x, double *gamma, double *phi, double *tempObj, double *objFunc, 
						double *WTSol, double *xlow, double *xupp, char *ixlow, char *ixupp, double *y, 
						double *z, double *lambda, double *pi)
{
	// now initilize them
	for (int i=0; i< nx; i++)
	{
		x[i] 		= 0.0;
		gamma[i]	= 0.0;
		phi[i]		= 0.0;
		tempObj[i] 	= 0.0;
		objFunc[i]	= c[i];
		WTSol[i]	= 0.0;
		xlow[i]		= XLOW[i];
		xupp[i]		= XUPP[i];
		ixlow[i]	= IXLOW[i];
		ixupp[i]	= IXUPP[i];
		FN[i]		= FluxName[i];
	}

	for (int i=0; i<my; i++)
	{
		y[i]	= 0.0;
	}
	
	if (mz >0)
	{
		z	= new double[mz];
		lambda	= new double[mz];
		pi	= new double[mz];

		for (int i=0; i<mz; i++)
		{
			z[i]	= 0.0;
			lambda[i]=0.0;
			pi[i] 	= 0.0;
		}
	}
}

QpManipulator::~QpManipulator() {; }
void QpManipulator::setBasicData(double *c, int nx, int *irowQ, int nnzQ, int *jcolQ, double *dQ,
                        int *irowA, int nnzA, int *jcolA, double *dA,
                        double *b, int my,
                        int *irowC, int nnzC, int *jcolC, double *dC, double *clow, int mz, char *iclow, double *cupp, char *icupp)
{
	this->c = c; this->nx= nx;
	this->irowQ=irowQ; this->nnzQ=nnzQ; this->jcolQ=jcolQ; this->dQ=dQ;
	this->irowA=irowA; this->nnzA=nnzA; this->jcolA=jcolA; this->dA=dA;
	this->b=b;this->my=my;
	this->irowC=irowC; this->nnzC=nnzC; this->jcolC=jcolC; this->dC=dC; this->clow=clow; this->mz=mz; this->iclow=iclow; this->cupp=cupp; this->icupp=icupp;

	generateMatrixQforMOMA();
}

void QpManipulator::generateMatrixQforMOMA()
{
	irowQMOMAQP = new int[nx];
	jcolQMOMAQP = new int[nx];
	   dQMOMAQP = new double[nx];
	 nnzQMOMAQP = nx;
	
	for (int i=0; i< nnzQMOMAQP; i++)
        {
                irowQMOMAQP[i] = i;
                jcolQMOMAQP[i] = i;
                   dQMOMAQP[i] = 1.0;
        }
}
double QpManipulator::CalLP(double *obj,double *xlow,char *ixlow,double *xupp,char *ixupp,double *x,double *gamma, double *phi,double *y,double *z,double *lambda,double *pi)
{
        int ierr;
        double ObjValue=0.0;
        qpsolvesp(obj , nx, irowQMOMALP, nnzQMOMALP, jcolQMOMALP, dQMOMALP, xlow, ixlow, xupp, ixupp,
                     irowA, nnzA, jcolA, dA, b, my,
                     irowC, nnzC, jcolC, dC,
                     clow,  mz,   iclow, cupp, icupp,
                     x, gamma, phi,
                     y,
                     z, lambda, pi, PRINT_LEVEL, &ierr );

        if (ierr ==0)
        {
		ObjValue=0.0;	
		for (int i=0; i<nx; i++)
		{
			ObjValue += c[i]*x[i];
		}
			
                // ObjValue=x[742];
        }
        else
                ObjValue=-1.0;

        return ObjValue;
}

// QP here
double QpManipulator::CalQP(double *obj,double *xlow,char *ixlow,double *xupp,char *ixupp,double *x,double *gamma, double *phi,double *y,double *z,double *lambda,double *pi)
{
        int ierr;
        double ObjValue=0.0;
        qpsolvesp(obj , nx, irowQMOMAQP, nnzQMOMAQP, jcolQMOMAQP, dQMOMAQP, xlow, ixlow, xupp, ixupp,
                     irowA, nnzA, jcolA, dA, b, my,
                     irowC, nnzC, jcolC, dC,
                     clow,  mz,   iclow, cupp, icupp,
                     x, gamma, phi,
                     y,
                     z, lambda, pi, PRINT_LEVEL, &ierr );

        if (ierr ==0)
        {
		ObjValue=0.0;
		for (int i=0; i<nx; i++)
		{
			ObjValue += c[i]*x[i];
		}
        }
        else
                ObjValue=-1.0;

        return ObjValue;
}

