#include <stdio.h>     /* for printf */
#include <stdlib.h>    /* for exit */
#include <getopt.h>
#include <iostream.h>
#include <string>
#include <fstream.h>
#include <math.h>
#include "header.h"
#include "supergene.h"
#include "qps.h"
#include "cQpGenSparse.h"
#include "annotationData.h"
#include "sbmlReader.h"
#define MAX_BUFF_SIZE 10000000


void usage() {
  printf("Usage: fluxor [-i inputfile] [-o outputfile] [-h]\n");
  exit(1);
}
void newQp( double *& c,      int nx,
		     int    *& irowQ,  int nnzQ,  int  *& jcolQ,  double *& dQ,
		     double *& xlow,              char *& ixlow,
		     double *& xupp,              char *& ixupp,
		     int    *& irowA,  int nnzA,  int  *& jcolA,  double *& dA,
		     double *& b,      int my,
		     int    *& irowC,  int nnzC,  int  *& jcolC,  double *& dC,
		     double *& clow,   int mz,    char *& iclow,
		     double *& cupp,              char *& icupp,
	    int    *  ierr ){
  c = new double[nx];
  irowQ = new int[nnzQ];  jcolQ = new int[nnzQ];  dQ = new double[nnzQ];
  xlow = new double[nx];  ixlow = new char[nx];
  xupp = new double[nx];  ixupp = new char[nx];
  irowA = new int[nnzA];  jcolA = new int[nnzA];  dA = new double[nnzA];
  b = new double[my];
  irowC = new int[nnzC];  jcolC = new int[nnzC];  dC = new double[nnzC];
  clow = new double[mz];  iclow = new char[mz];
  cupp = new double[mz];  icupp = new char[mz];
  ierr = 0;
}

void getSBMLFromSTDOUT(char * buffer ){
     int c; int i = 0;
     while((c = getchar()) != EOF ){
       buffer[i] = (char)c;
       i++;
     }

     buffer[i] = '\0';
     //printf( buffer );
}


using namespace std;
int main(int argc, char *argv[])
{
   double * c;      int NX;
   int    * irowQ;  int NNZQ;  int  * jcolQ;  double * dQ;
   double * xlow;              char * ixlow;
   double * xupp;              char * ixupp;
   int    * irowA;  int NNZA;  int  * jcolA;  double * dA;
   double * b;      int MY;
   int    * irowC;  int NNZC;  int  * jcolC;  double * dC;
   double * clow;   int MZ;    char * iclow;
   double * cupp;              char * icupp;
   double *x= 0; double *gamma = 0; double *phi = 0;
   double *y = 0;
   double *z = 0; double *lambda = 0; double *pi = 0;
   int PRINT_LEVEL = 0;
   int   ierr; 
   int toFile_p = -1;
   int fromString_p = 1;
   annotationData *AD;

   sbmlReader *S;
   char * outputfile = new char[MAX_BUFF_SIZE];
   char * sbml = new char[MAX_BUFF_SIZE];

   
   if( argc == 1 ) {
     fromString_p = 1;

   } else {
     int opt = 0;
     //cout << "Getting opt " << endl;
     while( ( opt = getopt(argc, argv, "i:o:h")) != -1 ) {
       switch( opt ) {
       case 'i':
	 //cout << "Case i" << endl;
	 fromString_p = 0;
	 strcpy( sbml, optarg );
	 break;
	 //	 cout << sbml << endl;
       case 'o':
	 //cout << "Case 0:" <<endl;
	 strcpy( outputfile, optarg );
	 //cout << "copied outputfile "<< endl;
	 toFile_p = 1;
	 break;
       case 'h':
	 usage();
       case '?':
	 usage();
       default:
	 printf("?? getopt returned character code 0%o ??\n", c);
       }
     }


   }
   if( fromString_p ) {
     getSBMLFromSTDOUT( sbml );
   }
   S = new sbmlReader( sbml, fromString_p );

   int numAnalyses = S->getNumAnalyses();   
   //cerr << "Read SBML " <<endl;
   for(int analysis = 0; analysis < NANALYSES; analysis++ ){
     S->getDimension( NX, MY, MZ, NNZA, NNZQ, NNZC, analysis);
     //cerr << "Got dimensions " <<endl;
     //cerr << "NX: " << NX << " MY: " << MY << " MZ: " << MZ << " NNZA: " << NNZA <<" nnzQ: " << NNZQ << " nnzC " << NNZC << endl;
     
     newQpGenSparse(&c, NX,
		    &irowQ,  NNZQ ,&jcolQ, &dQ,
		    &xlow, &ixlow,
		    &xupp, &ixupp,
		    &irowA, NNZA, &jcolA, &dA,
		    &b,     MY,
		    &irowC, NNZC, &jcolC, &dC,
		    &clow, MZ, &iclow,
		    &cupp,  &icupp,
		    &ierr );
     //cerr << "Initialized input arrays" << endl;
     x     = (double *) calloc( NX, sizeof( double ) );
     gamma = (double *) calloc( NX, sizeof( double ) );
     phi   = (double *) calloc( NX, sizeof( double ) );
     if( MY > 0 ) y = (double *) calloc( MY, sizeof( double ) );
     if( MZ > 0 ) {
       z      = (double *) calloc( MZ, sizeof( double ) );
       lambda = (double *) calloc( MZ, sizeof( double ) );
       pi     = (double *) calloc( MZ, sizeof( double ) );
     }
     
    //cerr << "Initialized output arrays" <<endl;
     
     S->getBounds( NX, ixlow, xlow, ixupp, xupp, analysis );
     
     //cerr << "Got bounds" << endl;
     S->getObjective( c, NX, analysis );
     
     //cerr << "Got objective" << endl;
     S->getQuadraticMatrix(irowQ, NNZQ, jcolQ, dQ, analysis);
     //cerr << "Got quadratic" << endl;
     S->getLinearEqualityMatrix(irowA, NNZA, jcolA, dA, b, MY, analysis);
     //cerr << "Got Stoichiometric matrix" <<endl;
     S->getLinearInequalityMatrix(irowC, NNZC, jcolC, dC, analysis );
     //cerr << "Got Linear Inequality Matrix dC" << endl;
     S->getLinearInequalityBounds(MZ, iclow, clow, icupp, cupp, analysis);
     //cerr << "Got Linear Inequality Bounds " << endl;
     qpsolvesp( c, NX, irowQ, NNZQ, jcolQ, dQ,
		xlow, ixlow, xupp, ixupp,
		irowA, NNZA, jcolA, dA, b, MY,
		irowC, NNZC, jcolC, dC,
		clow, MZ, iclow, cupp, icupp,
		x, gamma, phi, y,
		z, lambda, pi, PRINT_LEVEL, &ierr );
     //cerr << "Ran qpsolvesp" <<endl;
     //cerr << "Ierror: " << ierr <<endl;
     S->setPrediction( x, NX, analysis );
     for( int i = 0; i < NX; i++ ){
       //cerr << "x[" << i << "]: " << x[i] << endl;
     }
     

     freeQpGenSparse( &c, &irowQ, &jcolQ, &dQ,
		   &xlow,  &ixlow, &xupp,  &ixupp,
		   &irowA, &jcolA, &dA, &b,
		   &irowC, &jcolC, &dC, &clow,  &iclow, &cupp,  &icupp );

     free( x ); free( gamma ); free( phi ); free( y ); free( z ); 
     free( lambda ); free( pi );
   }
   //cerr << "writing to sbml" << endl;
   if( toFile_p ) {
     S->writeToSBML( outputfile, fromString_p );
   } else {
     S->writeToSBML( fromString_p );
   }
   //cerr<< "wrote to sbml" << endl;
   return 0;
}
