/*
 * This code was writen to parse the annotation section of the SBML file
 * for flux limits, coefficient for objective function and computation type 
 * (i.e. FBA/MOMA).
 *
 * Author: Dat H. Nguyen
 *         Department of Genetics
 *         Harvard Medical School
 *         Boston, MA 02115
 */

#include <stdio.h>
#include <expat.h>
#include <string.h>
#include <ctype.h>
#include <assert.h>
#include "annotationData.h"

#define BUFFSIZE 10000



char * toUpper(const char * lower ) {
  int len = strlen( lower );
  char * upper = (char *) calloc( len, sizeof( char ) );
  int i;
  for( i = 0; i < len; i++ ){
    upper[i] = (char)toupper( lower[i] );
  }
  return upper;
}

static void startElement(void *userData, const char *element, const char **attr)
{
  int i;
  int j;
  annotationData  *nAnnotationPtr = (annotationData *) userData;

  j = nAnnotationPtr->index;
  if( j > NANALYSES ) {
    //fprintf(stderr, "Error: number of analyses in this annotation tag is %d.  Should be less than %d\n", j, NANALYSES );
  }
  for (i = 0; attr[i]; i += 2) 
    {
      if (!strcmp( toUpper( attr[i] ),"XMLNS:FLUX"))
	{
	  nAnnotationPtr->index =0;
	  //fprintf(stderr, "XMLNS:FLUX = %s \n", attr[i + 1] );
	}
      else if (!strcmp( toUpper( attr[i] ),"ANALYSIS"))
	{
	  if (j > 0)
	    {
	      nAnnotationPtr->fbaType[j-1] = (char *) calloc(strlen(attr[i+1])+1,sizeof(char));
	      strcpy(nAnnotationPtr->fbaType[j-1] , attr[i+1]);
	    }	
	  //fprintf(stderr, "Analysis = %s\n fbaType=%s\n", attr[i+1], nAnnotationPtr->fbaType[j-1] );
	}
      else if (!strcmp(toUpper( attr[i] ),"UPPER"))
	{
	  if (j > 0)
	    {
	      //nAnnotationPtr->upperLimit[j-1] = (char *) calloc(strlen(attr[i+1])+1,sizeof(char));
	      if(!strcmp( toUpper( attr[i+1] ), "INF" ) ){
		nAnnotationPtr->iupperLimit[j-1] = (char) 0;
		//nAnnotationPtr->upperLimit[j-1] = atof( attr[i+1] );
		nAnnotationPtr->upperLimit[j-1] = 0;
	      } else {
		nAnnotationPtr->iupperLimit[j-1] = (char) 1;
		nAnnotationPtr->upperLimit[j-1] = atof( attr[i+1] );
	      }
	      
	      //fprintf(stderr, "%s:\tiUpperLimit = %d\tUpperLimit = %f\n", attr[i + 1], (int)nAnnotationPtr->iupperLimit[j-1], nAnnotationPtr->upperLimit[j-1]  );
	    }
	}
      else if (!strcmp( toUpper( attr[i] ),"LOWER"))
	{

	  if (j > 0)
	    {
	      
	      if(!strcmp( toUpper( attr[i+1] ), "-INF" ) ){
		nAnnotationPtr->ilowerLimit[j-1] = (char) 0;
		nAnnotationPtr->lowerLimit[j-1] = 0;
	      } else {
		nAnnotationPtr->ilowerLimit[j-1] = (char) 1;
		nAnnotationPtr->lowerLimit[j-1] = atof( attr[i+1] );
	      }
	      
	      //fprintf(stderr, "%s:\tiLowerLimit = %d\tLowerLimit = %f\n\n", attr[i + 1], (int)nAnnotationPtr->ilowerLimit[j-1], nAnnotationPtr->lowerLimit[j-1]  );
	    }
	}
      else if (!strcmp( toUpper( attr[i] ),"OBJECTIVE"))
	{
	  if (j > 0)
	    {
	      nAnnotationPtr->objectiveValue[j-1] = atof(attr[i+1]);
	      //fprintf(stderr, "%s:\tobjectiveValue = %f\n", attr[i + 1], nAnnotationPtr->objectiveValue[j-1]  );
	    }
	}
    }
	nAnnotationPtr->index += 1;
 	//fprintf(stderr, "\n------index = %d ---------------\n",nAnnotationPtr->index);
}

static void endElement(void *data, const char *element) { ; }

void parseAnnotation(const char * annotationString, annotationData * AData)
{
	int len;
	int nAnnotation=0;
	XML_Parser p = XML_ParserCreate(NULL);
	if (! p)
	{
		//fprintf(stderr, "Couldn't  create parser\n"); 
		exit(1);
	}
	XML_SetUserData(p, AData);
	XML_SetElementHandler(p, startElement, endElement);

	len = strlen(annotationString);

	if (XML_Parse(p, annotationString, len, 1) == XML_STATUS_ERROR)
	{
		//fprintf(stderr, "Parse error: %s\n",XML_ErrorString(XML_GetErrorCode(p)));
		exit(1);
	}
	XML_ParserFree(p);
   
}

/*
int main(int argc, char *argv[])
 {
	 int i=0;
	 char Buff[BUFFSIZE];
	 struct annotationData AD;

	 strcpy(Buff, "<annotation xmlns:flux=\"http://arep.med.harvard.edu/fluxns\"> 		      <flux:limit analysis=\"wild_type\" upper=\"iNF\" lower=\"0\" objective=\"-1\"/> 		<flux:limit analysis=\"mutant_fba\" upper=\"1000\" lower=\"-InF\" objective=\"0\" /> 		<flux:limit analysis=\"mutant_moma\" upper=\"INf\" lower=\"-Inf\" objective=\"34\"/> 		</annotation> "); 

	AD.index = 0; 
 	parseAnnotation(Buff, &AD);

	for (i=0; i<3;i++)
	{
	  printf("Data = %f, %f, %s, %f \n",AD.lowerLimit[i], AD.upperLimit[i], AD.fbaType[i], AD.objectiveValue[i]);
	}
  	return 0;
}
    
*/
