#include <iostream.h>
#include <string>
#include "sbmlReader.h"

extern "C" int sbmlParser(int fromString_p, char *SBMLFileName, int *NX, int *MY, int *NNZA,  int *IROWA, int *JCOLA, double *DA, annotationData *AD, char **MNAME, char ** RNAME );
extern "C" void writeFluxToSBML(const char * sbmldoc, const char * def_output, double * flux, int nelement); 
extern "C" void writeAnnotatedFluxToSBML(const char * sbmldoc, const char * def_output, annotationData *AD, int nelement); 


#include "jnode.h"

using namespace std;

sbmlReader::sbmlReader(char *sbml, int string_p) {
  initialize();
  sbmlDoc = sbml;
  fromString_p = string_p;
  int i = sbmlParser(fromString_p, sbml, &NX, &MY, &NNZA, IROWA, JCOLA, DA, AD, MetaboliteName, ReactionName);
  // cout << "Parsed sbml" <<endl;

  sortJcolA();
}

sbmlReader::sbmlReader(char *filename)
{
  sbmlReader( filename, 0 );
	// now, we need to sort the jcolA and DA
	
}

void sbmlReader::writeToSBML(double *fluxes, int fromString_p)
{
	writeFluxToSBML(sbmlDoc, defaultOutput.c_str(), fluxes, fromString_p);

}

void sbmlReader::writeToSBML(char * outputFileName, double *fluxes, int fromString_p)
{
	writeFluxToSBML(sbmlDoc, outputFileName, fluxes, fromString_p);
}
void sbmlReader::writeToSBML(char *outputFileName, int fromString_p ){
  writeAnnotatedFluxToSBML( sbmlDoc, outputFileName, AD,  fromString_p );
}

void sbmlReader::writeToSBML(int fromString_p ){
  writeAnnotatedFluxToSBML( sbmlDoc, 0 , AD,  fromString_p );
}

void sbmlReader::sortJcolA()
{
	int irowa, jcola, index;
        double da;

	NodeList * JN = new NodeList[MY];
        
	for (int i=0; i< NNZA; i++)
	{
		irowa = IROWA[i];
		jcola = JCOLA[i];
		da    = DA[i];
		if (irowa >=MY)
		{
			cerr << " ERROR in sbmlReader::sortJcolA " << endl; exit(1);
		}
		JN[irowa].insert(irowa,jcola,da);
	}

	index=0;
	for (int i=0; i<MY; i++)
	{
		while (1)
		{
			if (JN[i].getData(irowa, jcola, da))
			{
				if (irowa != i)
				{
					cerr << "ERROR ----------- in sbmlReader::sortA \n" ; exit(1); 
				}
				IROWA[index] = irowa;
				JCOLA[index] = jcola;
				DA[index]    = da;
				index++;
			}
			else
				break;
		}
	}
	if (index != NNZA)
	{
		cerr << " ERROR in sbmlReader::sortJcolA function \n"; exit(1);
	}
}
				
void sbmlReader::initialize()
{
	NX=0; MY=0; NNZA=0;
	sbmlDoc = NULL;
	for (int i=0; i< N_METABOLITES;i++)
                MetaboliteName[i] = new char[50] ;

	for (int i=0; i<N_REACTIONS; i++)
                ReactionName[i]   = new char[80];
}
void sbmlReader::getDimension (int &nx, int &my, int& mz){
}
void sbmlReader::getDimension (int &nx, int &my, int& mz, int &nnzA, int &nnzQ, int &nnzC, int index)
{
	nx = NX;
	my = MY;
	mz = MZ;
	nnzA = NNZA;
	if( index == 2 ){
	  nnzQ = NX;
	} else {
	  nnzQ = 0;
	}
	nnzC = 0;
	//cout << "In getDimension: NX: " << nx << " MY: " << my << " MZ: " << mz << " NNZA: " << nnzA <<" nnzQ: " << nnzQ << " nnzC " << nnzC << endl;
  
}
sbmlReader::~sbmlReader() { ; }

void sbmlReader::getData(int *irowA, int *jcolA, double *da ,string * metName, string * reactionName )
{
  
  for(int i=0; i < NX; i++){
    //cout << "Reaction name: " << ReactionName[i] << endl;
    reactionName[i] = (string) ReactionName[i];
  }
  for (int i=0; i< NNZA; i++)
    {
      irowA[i] = IROWA[i];
      jcolA[i] = JCOLA[i];
      da[i]    = DA[i];
    }
  for (int i=0; i< MY; i++)
    {
      metName[i] = (string) MetaboliteName[i];
    }
}

void sbmlReader::setPrediction(double *prediction, int nx, int index){
  if( nx == NX ){
    if( index == WILD_TYPE ) {
      for( int j = 1; j < NANALYSES; j++ ){
	if( getAnalysisType( j ) == MOMA ) {
	  for( int i = 0; i < NX; i++ ){
	    AD[i].objectiveValue[j] = prediction[i];
	  }
	}
      }
    }
    for(int i = 0; i < NX; i++){
      AD[i].prediction[index] = prediction[i];
    }
  } else {
    //cout << "nx is " << nx << ". Should be " << NX << endl;
  }
}

void sbmlReader::getPrediction(double *prediction, int nx, int index){
  if( nx != NX) {
    //cout << "nx is " << nx << ". Should be " << NX << endl;
  } else {
    for(int i = 0; i < NX; i++){
      prediction[i] = AD[i].prediction[index];
    }
  }
}

int sbmlReader::getAnalysisType( int analysis ) {
  if(analysis == MUTANT_MOMA) {
    return MOMA;
  } else {
    return FBA;
  }
}
void sbmlReader::getLinearInequalityBounds(int mz, char *iclow, double *clow, char *icupp, double *cupp, int index){
  if(( index < NANALYSES ) && (MZ == mz ) ){
    for( int i = 0; i < MZ; i++ ){
      iclow[i] = 0;
      clow[i] = 0;
      icupp[i] = 0;
      cupp[i] = 0;
    }
  }
}
void sbmlReader::getBounds( int nx, char *&ixlow, double *&xlow, char *&ixupp, double *&xupp, int index){

  if( (index < NANALYSES ) && ( NX == nx ) ) {
    for (int i=0; i< NX; i++)
      {

	ixlow[i] = AD[i].ilowerLimit[index];
	ixupp[i] = AD[i].iupperLimit[index];
	xlow[i] = AD[i].lowerLimit[index];
	xupp[i] = AD[i].upperLimit[index];
	//cout << "ixlow: " << (int)ixlow[i] << " xlow: " << xlow[i] << " ixupp: " << (int)ixupp[i] << " xupp: " << xupp[i] << " index: " << index <<endl;	
      }
  }
}

void sbmlReader::getLinearInequalityMatrix( int * irowC, int nnzC, int * jcolC, double * dC, int index ){

   for( int i = 0; i < nnzC; i++ ){
     irowC[i] = i;
     jcolC[i] = i;
     dC[i] = 0.0;
     //cout << "dC[" << ReactionName[irowC[i]] << "][" << ReactionName[jcolC[i]] << "] = " << dC[i] << endl;
   }

}
void sbmlReader::getLinearEqualityMatrix( int * irowA, int nnzA, int * jcolA, double * dA, double *b, int my, int index ){
  
  if( nnzA != NNZA ){
    //cout << "Number of non-zeros in A is: " << nnzA << ". Should be: " << NNZA<< endl;
  } else {
    for( int i = 0; i < nnzA; i++ ) {
      irowA[i] = IROWA[i];
      jcolA[i] = JCOLA[i];
      dA[i] = DA[i];
      //cout << "dA[" << MetaboliteName[irowA[i]] << "][" << ReactionName[jcolA[i]] << "] = " << dA[i] << endl;
    }
  }
  if( my == MY ){
    for( int j = 0; j < my; j++) {
      b[j] = 0;
    }
  } else {
    //cout << "my is " << my << ". Should be " << MY <<endl;
  }
}
  void sbmlReader::getQuadraticMatrix( int * irowQ, int nnzQ, int * jcolQ, double * dQ,  int index ){
  for( int i = 0; i < nnzQ; i++ ){
    irowQ[i] = i;
    jcolQ[i] = i;
    dQ[i] = 1.0;
    //cout << "dQ[" << irowQ[i] << "][" << jcolQ[i] << "] = " << dQ[i] << endl;
  }

}
void sbmlReader::getObjective( double * c, int nx, int index ){
  if( nx != NX ) {
    //cout << "nx is " << nx << ". Should be " << NX << endl;
  } else {
    if( getAnalysisType( index ) == MOMA ) {
      for( int i = 0; i < NX; i++ ){
	c[i] = -AD[i].prediction[WILD_TYPE];
	//cout << " C[" << i << "]: " << c[i] << endl;
      }
    } else {
      for( int i = 0; i < NX; i++) {
	c[i] = AD[i].objectiveValue[index];
	//cout << " C[" << i << "]: " << c[i] << endl;
      }
    }
  }
}
int sbmlReader::getNumAnalyses(){
  return NANALYSES;
}
void sbmlReader::setObjective(double *objective, int nx, int index){
  if( nx != NX ) {
    //cout << "nx is " << nx << ". Should be " << NX << endl;
  } else {
    for(int i = 0; i < NX; i++){
      AD[i].objectiveValue[index] = objective[i];
    }
  }
}



void sbmlReader::print(){
	int i;
	//cout << " Matrix A: \n";
        for (i=0; i< NNZA;i++)
        {
		//cout << "(row " << IROWA[i] << ", col " << JCOLA[i] << ")= " << DA[i] << endl;
        }
	//cout << "Number of Reactions = " << NX << "; Number of Metabolites = " << MY << "; Number of non-zero elements in matrix A = " << NNZA << endl;
	//cout << " -------- Objective function ------------------- " << endl;
	//cout << "Reaction i,  c[i], ixlow[i], xlow[i], ixupp[i], xupp[i], Reaction Name \n"; 
	//cout << "-----------------------------------------------------------------------\n";
	//        for (i=0; i<NX;i++)
        //{
		//cout << "Reaction [" << i << "]= " <<C[i] << ", " << (int) IXLOW[i] << ", " << XLOW[i] << ", " << (int) IXUPP[i] << ", " << XUPP[i] << ", " << ReactionName[i]<< endl;
        //}
        //for (i=0; i<MY;i++)

}

