#include <iostream.h>
#include <string>
#include <fstream.h>
#include <strstream.h>
#include <math.h>

#include "supergene.h"

SuperGene::SuperGene()
{
	init();
} 

SuperGene::~SuperGene()
{ ; }

void SuperGene::init()
{
	// initialize all array to default values
	for (int i=0; i< NIG; i++)
		geneName[i] = "";

	for (int i=0; i< NIF; i++)
	{
		fluxName[i] = "";
		fluxIndex[i] = -1;

		ixlow[i] = (char) 0;
		xlow[i]  = 0.0;
		ixupp[i] = (char) 0;
		xupp[i]  = 0.0;
	}
	
	currGeneIndex=0;
	currFluxIndex=0;
	globalFluxSize=0;
	SuperGeneFlag=false;
}	
// this clone function is needed for moving valid object to empty object
void SuperGene::cloneSuperGeneData(SuperGene * sg)
{
	currGeneIndex = sg->currGeneIndex;
	currFluxIndex = sg->currFluxIndex;
	globalFluxSize= sg->globalFluxSize;
	SuperGeneFlag = sg->SuperGeneFlag;

	for (int i=0; i< currGeneIndex; i++)
		geneName[i] = sg->geneName[i];
		
	for (int i=0; i< currFluxIndex; i++)
	{
		fluxName[i] = sg->fluxName[i];
		fluxIndex[i]= sg->fluxIndex[i];

		ixlow[i] = sg->ixlow[i];
		xlow[i]  = sg->xlow[i];
		ixupp[i] = sg->ixupp[i];
		xupp[i]  = sg->xupp[i];
	}
}

// given a gen name, cap it and insert it into local geneName array
void SuperGene::insert(const string gene)
{
	string stemp = gene;
	Cap(stemp);

	// insert it in and advance the counter
	geneName[currGeneIndex] = stemp;
	currGeneIndex++;

	if (currGeneIndex>= NIG)
	{
		cerr << " Number of Genes in a SuperGene exceed its limit \n";
		exit(1);
	}
}

void SuperGene::setFluxToZero(double *XLOW, char *IXLOW, double * XUPP, char *IXUPP)
{
	// loop over all fluxes that this superGene represents
	// and set the flux to zero
	int index;
	for (int i=0; i<currFluxIndex; i++)
	{
		// fluxIndex[i] contains index in the IXLOW and IXUPP for all fluxes this SuperGene represents
		index = fluxIndex[i];
		XLOW[index] = 0.0;
		IXLOW[index] = (char) 1;

		XUPP[index] = 0.0;
		IXUPP[index] = (char) 1;
	}
}

void SuperGene::restoreOriginalFluxConstraint(double *XLOW, char *IXLOW, double * XUPP, char *IXUPP)
{
	int index;
	for (int i=0; i<currFluxIndex; i++)
	{
		index = fluxIndex[i];
		IXLOW[index] = ixlow[i] ;
		XLOW[index]  = xlow[i];
		
		IXUPP[index] = ixupp[i] ;
		XUPP[index]  = xupp[i];
	}
}
bool SuperGene::checkGeneIfExist(string &st)
{
	string s=st;
	Cap(s);
	for (int i=0; i< currGeneIndex; i++)
	{
		if (s == geneName[i])
			return true;
	}
	return false;
}	

// this funtion does the following:
// 1. for each gene name in the super gene, find the flux index
// 2. capture the flux index ==> local array of fluxIndex
// 3. copy constraint values 
void SuperGene::computeFluxIndex(const int fluxsize, string *globalFluxArray, double *XLOW, char *IXLOW, double * XUPP, char *IXUPP)
{
	string s;
	int k=0;

	globalFluxSize = fluxsize;
	// go thru each gene in the Supergene
	for (int i=0; i< currGeneIndex; i++)
	{
		// get the gene name, compuete it size, extract the first four letters (max)
		k = geneName[i].size();
		if (k>4)
			k = 4;
		s = geneName[i].substr(0,k);

		// search for the gene name in the 
		for (int j=0; j< globalFluxSize; j++)
		{
			if (s == globalFluxArray[j].substr(0,k))
			{
				// copy the flux name
				fluxName[currFluxIndex]  = globalFluxArray[j];
				// capture the index of the flux ---> fluxIndex
				fluxIndex[currFluxIndex] = j;
				// capture constraints for 
				ixlow[currFluxIndex] =   IXLOW[j];
				ixupp[currFluxIndex] =   IXUPP[j];

				xlow[currFluxIndex]  =   XLOW[j];
				xupp[currFluxIndex]  =   XUPP[j];
					
				currFluxIndex++;
				if (currFluxIndex >=NIF)
				{
					cerr << " Number of Fluxes exceeds its limit NIF \n";
					exit(1);
				}
			}
		}
	}
	if (currFluxIndex > 0)
		SuperGeneFlag=true;
}
void SuperGene::print()
{
	// if (SuperGeneFlag)
	// {
		for (int i=0; i< currGeneIndex; i++)
		{
			cout << geneName[i] << "\t";
		}
		cout << "(";
		for (int i=0; i< currFluxIndex; i++)
			cout << fluxName[i] << "\t";
	
		cout << ")\n"; 
	// }
}

void SuperGene::Cap( string & x)
{
      	int temp;
      	int len = x.size();
     	int i = 0;
     	int offset = ('a' - 'A');
     	while (1)
     	{
           	if (i==len)
              		break;
         	temp = (int) x[i] ;
         	if ((temp >= 'a') && (temp <= 'z'))
             	temp = temp - offset;
             	x[i] = (char) temp;
                i++;
    	}
}
//////////////////////////////////////////////////////////////////////////////////////
// ------------------------ SuperGeneList Class here ------------------------------ //
//////////////////////////////////////////////////////////////////////////////////////

SuperGeneList::SuperGeneList(char *superGeneFileName)
{
	currNumSuperGene=0;
	parseSuperGeneFile(superGeneFileName);
}
int SuperGeneList::copyData(const string *fluxname, const int fluxsize, const double * xlow, const char * ixlow, const double *xupp, const char *ixupp)
{
	// create arrays
	fluxSize = fluxsize;
	oXLOW = new double[fluxSize];
	oXUPP = new double[fluxSize];

	oIXLOW= new char[fluxSize];
	oIXUPP= new char[fluxSize];

	fluxName = new string[fluxSize];

	for (int i=0; i<fluxSize; i++)
	{
		// constraint values
		oXLOW[i] = xlow[i];
		oXUPP[i] = xupp[i];

		// constraint flags
		oIXLOW[i] = ixlow[i];
		oIXUPP[i] = ixupp[i];

		fluxName[i] = fluxname[i];
	}

	// now call initializeSuperGeneFluxes(0 to initialize SuperGene fluxes
	initializeSuperGeneFluxes();
	// cout << "HELLO from SuperGeneList::copyData \n";
	// cout << "Flux size = " << fluxsize << endl;
	return currNumSuperGene;
}
int  SuperGeneList::checkGeneIfExist(string & st)
{
	int i = 0;
	while (1)
	{
		if (listOfSuperGene[i].checkGeneIfExist(st))
			return i;
		if (i>=currNumSuperGene)
			return -1;
		i++;
	}
}
void SuperGeneList::initializeSuperGeneFluxes()
{
	for (int i=0; i< currNumSuperGene; i++)
	{
		listOfSuperGene[i].computeFluxIndex(fluxSize, fluxName, oXLOW, oIXLOW, oXUPP, oIXUPP);
	}

	// now throw away empty genes with no fluxes
	int i=0, j=currNumSuperGene-1;
	int ngene = currNumSuperGene;
	while (1)
	{
		// if listOfSuperGene[i].SuperGeneFlag == false: empty supergene
		if (!(listOfSuperGene[i].SuperGeneFlag))
		{
			// copy .....
			listOfSuperGene[i].cloneSuperGeneData(&listOfSuperGene[j]);
			j--;
			ngene--;
		}
		else
		{
			i++;
		}
		if (i > j)
		{
			currNumSuperGene=ngene;
			break;
		}
	}
}

void SuperGeneList::parseSuperGeneFile(char * superGeneFileName)
{
	ifstream input;
	string s, ss;

	input.open(superGeneFileName);
	if (input.fail())
	{
		cout << "File: " << superGeneFileName << " NOT found, program now exits \n";
		exit(1);
	}

	currNumSuperGene=0;
	// get each line
	while (getline(input, s, '\n'))
	{
	   	if (s.size() > 2)
	     	{
			if ((s[0] != '#') && (s[0] != '?'))
			{
				// make a string object
				istrstream line( (char *) s.c_str() );
				// extract each token (gene) in the line of superGene definition
				while (line >> ss)
				{
					// insert
					listOfSuperGene[currNumSuperGene].insert(ss);
				}
				// advance the current index
				currNumSuperGene++;
		
			}
             	}
	}
	// cout << " Number of super genes = " << currNumSuperGene << endl;

}
void SuperGeneList::setFluxToZero(const int superGeneIndex, double * xlow, char * ixlow, double *xupp, char *ixupp)
{
	listOfSuperGene[superGeneIndex].setFluxToZero(xlow, ixlow, xupp, ixupp);
}

void SuperGeneList::restoreOriginalFluxConstraint(const int superGeneIndex, double * xlow, char * ixlow, double *xupp, char *ixupp)
{
	listOfSuperGene[superGeneIndex].restoreOriginalFluxConstraint(xlow, ixlow, xupp, ixupp);
}

void SuperGeneList::print()
{
	int k=0;
	for (int i=0; i< currNumSuperGene; i++)
	{
		cout << i << "\t";
		listOfSuperGene[i].print();
		k += listOfSuperGene[i].currFluxIndex;
	}
	//cout << "Number of Fluxes used = " << k << endl;
	// cout << "Number of SuperGene   = " << currNumSuperGene << endl;
}
void SuperGeneList::print(int superGeneIndex)
{
	listOfSuperGene[superGeneIndex].print();
}
