#ifndef _SUPER_GENE_H
#define _SUPER_GENE_H

const int NIG      = 30;		// maxinum nimber of genes constituting a supergene
const int NIF      = 100;		// flux array index 
const int MAX_SUPER_GENE = 1000;

using namespace std;
class SuperGene
{
	friend class SuperGeneList;
   private:

	// this constructor take in the global flux array and its bounds
        SuperGene(int fluxSize);
	SuperGene();
	void init();
        ~SuperGene();

	string 	geneName[NIG];		// genes constituting the supergene
	string 	fluxName[NIF];		// flux name corresponding to genes in the supergene
	int	globalFluxSize;		// # of primal variables
	int    	fluxIndex[NIF];		// index of fluxes

	char   	ixlow[NIF]; 
	double  xlow[NIF];
	
	char 	ixupp[NIF];  // keep the original constraint for primal vector
	double  xupp[NIF];

	int currGeneIndex, currFluxIndex;
	bool SuperGeneFlag;
	
	// set of functions to operate
	void insert(const string gene);
	void setFluxToZero(double *XLOW, char *IXLOW, double * XUPP, char *IXUPP);
	void restoreOriginalFluxConstraint(double *XLOW, char *IXLOW, double * XUPP, char *IXUPP);
	void computeFluxIndex(const int fluxsize, string *globalFluxArray, double *XLOW, char *IXLOW, double * XUPP, char *IXUPP);
	void print();
	void Cap(string & x);
	bool checkGeneIfExist(string &st);
	void cloneSuperGeneData(SuperGene * sg);
} ;

class SuperGeneList
{
  public:
	SuperGeneList(char * SuperGeneFileName);
	~SuperGeneList();
		
	// this hold an array of super genes
	SuperGene listOfSuperGene[MAX_SUPER_GENE];
	// these are flux names and constraint for fluxes
	double *oXLOW, *oXUPP;
	char *oIXLOW, *oIXUPP;
	string *fluxName;
	int  fluxSize;

	// current # of superGenes
	int  currNumSuperGene;
	void parseSuperGeneFile(char * superGeneFileName);
	int  copyData(const string *fluxname, const int fluxsize, const double * xlow, const char * ixlow, const double *xupp, const char *ixupp);
	void setFluxToZero(const int superGeneIndex, double * xlow, char * ixlow, double *xupp, char *ixupp);
	void restoreOriginalFluxConstraint(const int superGeneIndex, double * xlow, char * ixlow, double *xupp, char *ixupp);
	int  checkGeneIfExist(string & st);
	void print();
	void print(int superGeneIndex);

  private:
	void initializeSuperGeneFluxes();
	
};

#endif

