package edu.harvard.med.fluxorspreadsheet;



import java.util.Vector;
import java.util.HashMap;
import java.lang.reflect.*;
import java.util.Iterator;

/**
 * This class holds panel specific column metadata.
 * This includes the column ID, index, name, class,and whether the column is
 * editable. This data is encapsulated in {@link FluxorTableColumn} objects.
 */

public class FluxorColumnMetaData {
    HashMap columnIndexMap; 
    HashMap columnNameMap; 
    HashMap columnClassMap;
    HashMap columnEditableMap;
    Vector columnIDs;
	
    public FluxorColumnMetaData(Vector columnIDs,
				HashMap columnIndexMap, 
				HashMap columnNameMap, 
				HashMap columnClassMap,
				HashMap columnEditableMap){
	this.columnIndexMap = columnIndexMap;
	this.columnNameMap = columnNameMap;
	this.columnClassMap = columnClassMap;
	this.columnEditableMap = columnEditableMap;
	this.columnIDs = columnIDs;
    }
    Vector getColumnIDs(){
	return columnIDs;
    }
    int getColumnIndex( String columnID ){
	if( columnIndexMap.containsKey( columnID ) ){
	    Integer index = (Integer)columnIndexMap.get( columnID );
	    return index.intValue();
	} else {
	    return -1;
	}
    }

    public HashMap getTableColumnByIndex() throws FluxorException {
 

	Iterator i = columnIDs.iterator();
	HashMap tableColumnByIndex = new HashMap();
	try {
	    while( i.hasNext() ){
		String columnID = (String)i.next();
		int modelIndex = getColumnIndex( columnID );
		String columnName = getColumnName( columnID );
		Class columnClass = getColumnClass( columnID );
		boolean editable = isColumnEditable( columnID);
		boolean hidden = false;
		int width = 300;
		tableColumnByIndex.put( new Integer( modelIndex ),
					new FluxorTableColumn( modelIndex,
							       columnID,
							       columnName,
							       columnClass,
							       editable,
							       hidden,
							       width ) );
	    }
	} catch( ClassNotFoundException e ){
	    throw new FluxorException( e );
	}
	return tableColumnByIndex;
    }

    public HashMap getColumnIndexByID() {
	return columnIndexMap;
    }

    String getColumnName( String columnID ){
	if( columnNameMap.containsKey( columnID ) ){
	    return (String)columnNameMap.get( columnID );
	} 
	return "None";
    }

   boolean isColumnEditable( String columnID ){
	if( columnEditableMap.containsKey( columnID ) ){
	    return ((Boolean)columnEditableMap.get( columnID )).booleanValue();
	} 
	return false;
    }

    Class getColumnClass( String columnID ) 
	throws java.lang.ClassNotFoundException {
	if( !columnClassMap.containsKey( columnID ) ){
	    throw new ClassNotFoundException();
	}
	return (Class)columnClassMap.get( columnID );	
    }
}

