package edu.harvard.med.fluxorspreadsheet;
import java.util.Vector;
import javax.swing.JTable;
import javax.swing.JScrollPane;
import javax.swing.table.AbstractTableModel;
import javax.swing.SwingUtilities;
import javax.swing.JPanel;
//import javax.swing.*;
import java.awt.Dimension;
import javax.swing.event.TableModelEvent;


/**
 * This class uses the information from a  {@link FluxorPanelInfo} to create 
 * the panel with Swing elements. The fluxor panel is a {@link JPanel}. It
 * contains a  {@link JTable} object which has a {@link FluxorTableModel} as
 * its table model. It contains scroll bars.
 */

class FluxorPanel extends JPanel {
    private String panelName;
    private String panelID;
    private FluxorTableModel tableModel = null;
    private JTable table = null;    
    private JScrollPane scrollPane;


    public FluxorPanel(FluxorPanelInfo panelInfo) throws FluxorException {
	panelName = panelInfo.getPanelName();
	panelID = panelInfo.getPanelID();
	this.tableModel = new FluxorTableModel(panelInfo);
	this.table = new JTable(tableModel);
	this.table.setPreferredScrollableViewportSize(new Dimension(500, 70));

        //Create the scroll pane and add the table to it. 
	JScrollPane scrollPane = new JScrollPane(table);
	this.scrollPane = scrollPane;
    }

    public void updateAllTableValues() {
	tableModel.fireTableDataChanged();
    }

    public void updateTable() throws FluxorException {
	tableModel.updateTable();
    }

    public void clearTable() {
	tableModel.removeAllRows();
    }

    public void stopEditing() {
	table.editCellAt(-1,-1);
    }

    public void clearSelection() {
	table.clearSelection();
    }

    public FluxorTableModel getModel() {
	return (this.tableModel);
    }
    public JScrollPane getScrollPane() {
	return (this.scrollPane);
    }

    /**
     * Gets the value of panelName
     *
     * @return the value of panelName
     */
    public String getPanelName()  {
	return this.panelName;
    }

    /**
     * Sets the value of panelName
     *
     * @param argPanelName Value to assign to this.panelName
     */
    public void setPanelName(String argPanelName) {
	this.panelName = argPanelName;
    }

    /**
     * Gets the value of panelID
     *
     * @return the value of panelID
     */
    public String getPanelID()  {
	return this.panelID;
    }

    /**
     * Sets the value of panelID
     *
     * @param argPanelID Value to assign to this.panelID
     */
    public void setPanelID(String argPanelID) {
	this.panelID = argPanelID;
    }

    /**
     * Gets the value of tableModel
     *
     * @return the value of tableModel
     */
    public FluxorTableModel getTableModel()  {
	return this.tableModel;
    }

    /**
     * Sets the value of tableModel
     *
     * @param argTableModel Value to assign to this.tableModel
     */
    public void setTableModel(FluxorTableModel argTableModel) {
	this.tableModel = argTableModel;
    }

    /**
     * Gets the value of table
     *
     * @return the value of table
     */
    public JTable getTable()  {
	return this.table;
    }

    /**
     * Sets the value of table
     *
     * @param argTable Value to assign to this.table
     */
    public void setTable(JTable argTable) {
	this.table = argTable;
    }

}
