package edu.harvard.med.fluxorspreadsheet;
/*
 * FluxorPanelInfo.java
 *
 * Created on July 4, 2004, 10:25 AM
 */

/**
 *
 * @author  Aaron Brandes
 */


/**
 * This class holds panel specific data: ID, name, and column metadata.
 * For each column this includes the column ID, name, class, preferred
 * width and whether the column is editable or hidden (hidden not currently
 * implemented.)
 * The {@link FluxorTableModel} delegates getting column metadata to this
 * class.
 */

import java.lang.String;
import java.util.HashMap;
import java.lang.Integer;
import javax.swing.table.TableColumnModel;
import java.lang.Class;
public class FluxorPanelInfo {
    
    private String panelName;
    private String panelID;
    private HashMap tableColumnByIndex = null;    
    private HashMap columnIndexByID = null;    

    //    private FluxorTableColumnModel  tableColumnModel = null;

    
    /** Creates a new instance of Class */
    public FluxorPanelInfo(String panelName, 
			   String panelID,
			   HashMap tableColumnByIndex, 
			   HashMap columnIndexByID) throws FluxorException {
        this.panelName = panelName;
	this.panelID = panelID;
	this.tableColumnByIndex = tableColumnByIndex;
	this.columnIndexByID = columnIndexByID;
    }
    

    public FluxorTableColumn getTableColumnByIndex(int columnIndex) {
	return (FluxorTableColumn)
	    tableColumnByIndex.get(new Integer(columnIndex));
    }

    // These methods are used by the FluxorTableModel

    public int getColumnCount() {
	return columnIndexByID.size();
    }


    public String getColumnID(int col) {
	return getTableColumnByIndex(col).getColumnID();
    }

    public String getColumnName(int col) {
	return getTableColumnByIndex(col).getColumnName();
    }

    public boolean isColumnEditable(int col) {
	return getTableColumnByIndex(col).isEditable();
    }


    public Class getColumnClass(int col) {
	return getTableColumnByIndex(col).getColumnClass();
    }

    // Additional Methods
    public boolean isColumnHidden(int col) {
	return getTableColumnByIndex(col).isHidden();
    }

    public int getPreferredWidth(int col) {
	return getTableColumnByIndex(col).getPreferredWidth();
    }

    public String toString() {
	int numCols = getColumnCount();
	StringBuffer sb = new StringBuffer("FluxorTableColumn:\n");
	sb.append("Panel " + getPanelName() + " with ID " + getPanelID() +
		  " has " + numCols + " columns.\n");

	for (int i = 0; i < numCols; i++) {
	    sb.append("Column Index: " + i + "\t");
	    sb.append("Column Name: " + getColumnName(i) + "\t");
	    sb.append("Column ID: " + getColumnID(i) + "\t");
	    sb.append("Column Class: " + getColumnClass(i) + "\t");
	    sb.append("Editable?: " + isColumnEditable(i) + "\t");
	    sb.append("Hidden?: " + isColumnHidden(i) + "\t");
	    sb.append("Preferred Width: " + getPreferredWidth(i) + "\n");
	}
	return sb.toString();
    }
    // Could add some validation to the setter methods
    
    /** Getter for property panelName.
     * @return Value of property panelName.
     *
     */
    public String getPanelName() {
        return panelName;
    }
    
    
    /** Getter for property panelID.
     * @return Value of property panelID.
     *
     */
    public String getPanelID() {
        return panelID;
    }
    


}
