package edu.harvard.med.fluxorspreadsheet;
/**
 * FluxorPanelMetaData.java
 *
 *
 * Created: Thu Jul  8 13:23:09 2004
 *
 * @author <a href="mailto:">Aaron Brandes</a>
 * @version 1.0
 */


import java.lang.String;
import java.lang.Integer;
import java.lang.Class;
import java.util.Iterator;
import java.util.HashMap;
 
/**
 * This class holds panel specific data: ID, name. It also provides access
 * to and column metadata via a {@link FluxorColumnMetaData} member object.
 * For each column this includes the column ID, name, class, and whether 
 * the column is editable
 * It also provides a method for the construction of a 
 * {@link FluxorPanelInfo} object which uses much of the same data.
 */

public class  FluxorPanelMetaData {
    protected String panelName = "";
    protected String panelID = "";
    protected FluxorColumnMetaData columnMetaData = null;
    protected boolean panelHidden; // optional for now

    public FluxorPanelMetaData(String panelID,
			       String panelName, 
			       FluxorColumnMetaData columnMetaData){
	setPanelID( panelID );
	setPanelName( panelName );
	setColumnMetaData( columnMetaData );
	setPanelHidden( false );
    }
    void setColumnMetaData( FluxorColumnMetaData columnMetaData ) {
	this.columnMetaData = columnMetaData;
    }
    /** Getter for property panelName.
     * @return Value of property panelName.
     *
     */
    public String getPanelName() {
        return panelName;
    }
    
    /** Setter for property panelName.
     * @param panelName New value of property panelName.
     *
     */
    public void setPanelName(String panelName) {
        this.panelName = panelName;
    }
    
    /** Getter for property panelID.
     * @return Value of property panelID.
     *
     */
    public String getPanelID() {
        return panelID;
    }
    /** Setter for property panelID
     * @param panelID value to set panelID.
     */
    public void setPanelID(String panelID ){
	this.panelID = panelID;
    }
    
   /**
     * Get the ColumnMetaData value.
     * @return the ColumnMetaData value.
     */
    public FluxorColumnMetaData getColumnMetaData() {
	return columnMetaData;
    }

    public FluxorPanelInfo getPanelInfo() throws FluxorException {
	String panelName = getPanelName();
	String panelID = getPanelID();
	HashMap tableColumnByIndex = columnMetaData.getTableColumnByIndex();
	HashMap columnIndexByID = columnMetaData.getColumnIndexByID();
	return new FluxorPanelInfo( panelName,
				    panelID,
				    tableColumnByIndex,
				    columnIndexByID );
    }
    
    // Optional For now

    /**
     * Get the PanelHidden value.
     * @return the PanelHidden value.
     */
    public boolean isPanelHidden() {
	return panelHidden;
    }

    /**
     * Set the PanelHidden value.
     * @param newPanelHidden The new PanelHidden value.
     */
    public void setPanelHidden(boolean newPanelHidden) {
	this.panelHidden = newPanelHidden;
    }

    

 
}
// FluxorPanelMetaData
