/*
##########################################################################
# Copyright 2004 Department of Genetics, Harvard Medical School and
# and the President and Fellows of Harvard University.
# This code may not be copied without acknowledgement.
# It may be freely distributed for non-commercial use.
#
# The Department of Genetics, Harvard Medical School shall not be
# liable for any damages suffered by anyone  using, modifying
# or distributing this software or its derivatives.
#
##########################################################################
*/
/**
 * Acknowledgement: Some of the code and some of the ideas in this package
 * are based on ideas and code in the BioSpreadsheet
 * written by James M. McCollum <jmccoll2@utk.edu>
 * University of Tennessee - Knoxville
 * http://biocomp.ece.utk.edu
 */
package edu.harvard.med.fluxorspreadsheet;
/**
 * FluxorSpreadsheet.java
 *
 *
 * Created: Tue Jul  6 15:54:37 2004
 *
 * @author <a href="mailto:">Aaron Brandes</a>
 * @version 1.0
 */
import java.io.IOException;
import java.io.PrintWriter;
import java.io.FileOutputStream;
import java.lang.StringBuffer;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.FileReader;
import java.io.File;
import java.io.InputStreamReader;

/**
 * This class is the invoking class of the FluxorSpreadsheet program.
 * It handle the processing of command line arguments. and 
 * calls {@link FluxorFrame} which
 * builds the actually GUI and populates it with data from
 * a libSBML Model object, or from an sbml file (which it uses to populate
 * a libSBML Model object.
 */

public class FluxorSpreadsheet {
    /**
     * Creates a new <code>FluxorSpreadsheet</code> instance.
     *
     * @exception FluxorException if an error occurs
     */
    public FluxorSpreadsheet() throws FluxorException{
	MetabolismReader metabolismReader = new MetabolismReader();
	String displayName = "";
	PrintWriter outputWriter = null;
	Metabolism metabolism = new Metabolism();
	FluxorFrame fluxorFrame;
	String inputFilename = "";
	String outputFilename = "";
 
	try {
	    // Check if output should be sent to std out or to a file
	    // The default is not to have any output when the application exits
	    if (isTrueString(System.getProperty("stdout"))) {
		outputWriter = new PrintWriter(System.out);
		//System.out.println("Output to stdout");
	    }
	    else {
		outputFilename = System.getProperty("of");
		if (isNonEmptyString(outputFilename)) {
		    outputWriter = 
			new PrintWriter(new FileOutputStream(outputFilename));
		}
	    }
	    
	    // Check if input should be come from std in  or from a file
	    // The default is to start with an empty metabolism
	    if (isTrueString(System.getProperty("stdin"))) {
		// read from std to get sbml string
		metabolism =
		    metabolismReader.readMetabolismFromString(stdInToString());
		fluxorFrame = new FluxorFrame(metabolism, outputWriter);
		//System.out.println("Getting metabolism from stdin");
	    }
	    else {
		// If a filename was specified, pass it to the constructor
		inputFilename = System.getProperty("if");

		if (isNonEmptyString(inputFilename)) {
		    fluxorFrame = 
			new FluxorFrame(inputFilename, outputWriter);	
		}
		else {
		    //System.out.println("no input metabolism" );
		    fluxorFrame = new FluxorFrame(outputWriter);
		}
	    } 
	}
	catch (IOException e) {
	}
	catch (Throwable e) {
	    System.out.println("exception: " + e.getMessage());
	    e.printStackTrace();
	}
    } // FluxorSpreadsheet constructor
    
 
    static boolean isTrueString(String string) {
	return (null != string &&
		(string.equalsIgnoreCase("t") ||
		 string.equalsIgnoreCase("true")));
    }

    static boolean isNonEmptyString(String string) {
	return (null != string && 0 != string.length());
    }
 

    private static String fileToString(String filename) throws IOException {
	return bufferedReaderToString(new BufferedReader
				      (new FileReader(filename)));
    }

    private static String stdInToString() throws IOException {
	return bufferedReaderToString(new BufferedReader
				      (new InputStreamReader(System.in)));
    }


    private static String bufferedReaderToString(BufferedReader br) 
	throws IOException {
	String lineSep = System.getProperty("line.separator");
	String nextLine = "";
	StringBuffer sb = new StringBuffer();
	while ((nextLine = br.readLine()) != null) {
	    sb.append(nextLine);
	    //
	    // note:
	    //   BufferedReader strips the EOL character.
	    //
	    sb.append(lineSep);
	}
	return sb.toString();
    }

    /**
     * Main routine for fluxorSpreadsheet
     *
     * @param args not used, arguments are specfied through properties
     * @exception FluxorException if an error occurs
     */
    public static void main(String[] args) throws FluxorException {

	FluxorSpreadsheet fluxorSpreadsheet = new FluxorSpreadsheet();
    }

} // FluxorSpreadsheet
    
