package edu.harvard.med.fluxorspreadsheet;
import javax.swing.table.TableColumn;
import java.lang.String;
/**
 * FluxorTableColumn.java
 *
 *
 * Created: Mon Jul  5 11:02:02 2004
 *
 * @version 1.0
 */
/**
 * This class extends {@link TableColumn} class to include the information
 * the following information: column ID, name, class, preferred
 * width and whether the column is editable or hidden (hidden not currently
 * implemented.)
 */

public class FluxorTableColumn extends TableColumn {
    private boolean editable;
    private boolean hidden;
    private String columnName = "";
    private Class columnClass = null;


    // FluxorTableColumn constructor
    // could use getIdentifier and setIdentifier for the ID

    public FluxorTableColumn(int modelIndex,
			     String columnID,
			     String columnName,
			     Class columnClass,
			     int width) {
	this(modelIndex,
	     columnID,
	     columnName,
	     columnClass,
	     true,   // editable
	     false,  // not hidden
	     width);
    }

    public FluxorTableColumn(int modelIndex,
			     String columnID,
			     String columnName,
			     Class columnClass,
			     boolean editable,
			     boolean hidden,
			     int width) {
	super(modelIndex);
	if (width >= 0) {
	    this.setPreferredWidth(width);
	}
	this.columnName = columnName;
	this.setIdentifier(columnID);
	this.columnClass = columnClass;
	this.editable = editable;
	this.hidden = hidden;
    } // FluxorTableColumn constructor
    
    /**
     * Get the Editable value.
     * @return the Editable value.
     */
    public boolean isEditable() {
	return editable;
    }

    /**
     * Set the Editable value.
     * @param newEditable The new Editable value.
     */
    public void setEditable(boolean newEditable) {
	this.editable = newEditable;
    }


    /**
     * Get the Hidden value.
     * @return the Hidden value.
     */
    public boolean isHidden() {
	return hidden;
    }

    /**
     * Set the Hidden value.
     * @param newHidden The new Hidden value.
     */
    public void setHidden(boolean newHidden) {
	this.hidden = newHidden;
    }


    /**
     * Get the ColumnName value.
     * @return the ColumnName value.
     */
    public String getColumnName() {
	return columnName;
    }

    /**
     * Set the ColumnName value.
     * @param newColumnName The new ColumnName value.
     */
    public void setColumnName(String newColumnName) {
	this.columnName = newColumnName;
    }


    /**
     * Gets the value of columnID
     *
     * @return the value of columnID
     */
    public String getColumnID()  {
	return (String)this.getIdentifier();
    }


    /**
     * Gets the value of columnClass
     *
     * @return the value of columnClass
     */
	public Class getColumnClass()  {
	return this.columnClass;
    }

    /**
     * Sets the value of columnClass
     *
     * @param argColumnClass Value to assign to this.columnClass
     */
    public void setColumnClass(Class argColumnClass) {
	this.columnClass = argColumnClass;
    }
    
    public String toString() {
	StringBuffer sb = new StringBuffer("FluxorTableColumn:\n");
	sb.append("Model Index: " + getModelIndex() + "\t");
	sb.append("Column Name(): " + getColumnName() + "\t");
	sb.append("Column ID: " + getColumnID() + "\t");
	sb.append("ColumnClass: " + getColumnClass() + "\t");
	sb.append("Editable?: " + isEditable() + "\t");
	sb.append("Hidden?: " + isHidden() + "\t");
	sb.append("Preferred Width: " + getPreferredWidth() + "\n");
	return sb.toString();
    }
} // FluxorTableColumn
