package edu.harvard.med.fluxorspreadsheet;
import java.util.Vector;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
//import javax.swing.SwingUtilities;


/**
 * This class implements the table model for a panel in the FluxorSpreadsheet
 * Application. It does not know anything about specific panel types. 
 * It delegates getting column-specific information (metadata) to the 
 * {@link FluxorPanelInfo} class.
 * It delegates the getting and setting of cell data to the{@link Metabolism}
 * class. All data is held in the underlying libSBML Model object. There
 * is no local array or Vector of data.
 */

/**
 * FluxorTableModel.java
 *
 *
 * Created: Mon Jul  5 16:01:51 2004
 *
 * @author Aaron Brandes
 * @version 1.0
 */
public class  FluxorTableModel extends AbstractTableModel {
    private boolean DEBUG = false;
    private Vector tableRows = new Vector();
    private String panelID = "";
    private FluxorPanelInfo panelInfo = null;

    public FluxorTableModel(FluxorPanelInfo panelInfo) 
	throws FluxorException {
	
	this.panelID = panelInfo.getPanelID();
	this.panelInfo = panelInfo;
	tableRows = FluxorFrame.getMetabolism().getRowIDs(panelID);
    }

    public FluxorTableModel(String panelID) throws FluxorException{
	
	this.panelID = panelID;
	tableRows = FluxorFrame.getMetabolism().getRowIDs(panelID);
	
    }
	
    public int getColumnCount() {
	try {
	    int columncount =  
		FluxorFrame.getMetabolism().getColumnIDs( panelID ).size();
	    return columncount;
	} catch (FluxorException e) {
	    return -1;
	}
    }
        
    public int getRowCount() {
	return tableRows.size();
    }

    public String getColumnName(int col) {
	return panelInfo.getColumnName(col);
    }

    public String getColumnID(int col) {
	return panelInfo.getColumnID(col);
    }

    
    public Object getValueAt(int row, int col) {
	try {
	    Object value = FluxorFrame.getMetabolism().getValue(panelID, 
					   (String)tableRows.get(row),
					   getColumnID(col));
	    return value;
	} catch (FluxorException e) {
	    throw new RuntimeException( e.fillInStackTrace() );
	}
    }

    public void removeAllRows() {
	tableRows = new Vector();
	fireTableDataChanged(); 
    }

   public void updateTable() throws FluxorException {
	tableRows = FluxorFrame.getMetabolism().getRowIDs(panelID);
	fireTableDataChanged(); 
    }
    /*
     * JTable uses this method to determine the default renderer/
     * editor for each cell. 
     */
    public Class getColumnClass(int col) {
	return panelInfo.getColumnClass(col);
    }

    /*
     */
    public boolean isCellEditable(int row, int col) {
	return panelInfo.isColumnEditable(col);
    }

    /*
     */
    public void setValueAt(Object value, int row, int col) {
	//check if value has actually changed
	if (DEBUG) {
	    System.out.println("Setting value at " + row + "," + col
			       + " to " + value
			       + " (an instance of " 
			       + value.getClass() + ")");
	}
	try {
	    FluxorFrame.getMetabolism().setValue(panelID, 
						 (String)tableRows.get(row),
						 getColumnID(col),
						 value);
	    fireTableCellUpdated(row, col);
	} catch (FluxorException e) {
	    throw new RuntimeException( e.fillInStackTrace() );
	}
    }

    

    public void printDebugData() {
	int numRows = getRowCount();
	int numCols = getColumnCount();

	for (int i=0; i < numRows; i++) {
	    System.out.print("    row " + i + ":");
	    for (int j=0; j < numCols; j++) {
		System.out.print("\t" + getValueAt(i, j));
	    }
	    System.out.println();
	}
	System.out.println("--------------------------");
    }
}
