package edu.harvard.med.fluxorspreadsheet;
import java.util.HashMap;
import java.util.Vector;

/**
 * The MetabolismPanel class defines the interface of methods necessary to 
 * get and set panel data used in the {@link Metabolism} class. .
 * A MetabolismPanel provides a link to the row data for the panel, which
 * is accessed in the form of a {@link PanelRow}.
 * Each panel type requires a specific implementation. 
 * See {@link SpeciesPanel}, {@link ReactionRow} .
 */

public interface MetabolismPanel {
    public String getPanelID();
    public String getPanelName();
    public FluxorColumnMetaData getColumnMetaData() 
	throws ClassNotFoundException;
    public FluxorPanelMetaData getPanelMetaData() 
	throws ClassNotFoundException;
    public Vector getPanelRowIDs();
    public PanelRow getPanelRow(String rowID);
    public HashMap getPanelRowData(String rowID);
}
