package edu.harvard.med.fluxorspreadsheet;
import libsbml.*;
import java.util.*;
/**
 * The ReactionPanel class  collects (metadata) about the columns of the 
 * Reaction panel and libSBML Reaction objects. These libSBML Reaction 
 * objects are
 * referenced by their IDs, and form the link between the row data in the
 * GUI reaction panel, and the {@link Metabolism} class libSBML Model object.
 * Note: the libSBML Reaction objects as well as the column metadata are
 * encapsulated in the {@link ReactionRow} class.
 */
public class ReactionPanel implements MetabolismPanel {
    Model model;
    static String panelID = "rxns";
    static String panelName = "Reactions";
    private HashMap panelRowMap = null;
    private boolean existPredictions = false;
    FluxorColumnMetaData columnMetaData;

    public ReactionPanel( Model model ) {
	this.model = model;
	// Take special action if no reactions contain predictions,
	try {
	    for( int i = 0; i < model.getNumReactions(); i++ ){
		if (ReactionAnnotation.hasPrediction(model.getReaction(i))) {
		    existPredictions = true;
		    break;
		}
	    } 
	    // If no reactions contain predictions, the reaction rows
	    // should not contain predictions and the reaction annotatations
	    // should not produce default annotations for them
	    if (!existPredictions) {
		ReactionRow.setNoPredictions();
		ReactionAnnotation.setNoPredictions();
	    }
	} catch( Exception e ) {
	    e.printStackTrace();
	}
    }

    public String getPanelID(){
	return panelID;
    }
    public String getPanelName(){
	return panelName;
    }

    public FluxorColumnMetaData getColumnMetaData( ) 
	throws ClassNotFoundException {
	return ReactionRow.getColumnMetaData();
    }


    public FluxorPanelMetaData getPanelMetaData() 
	throws ClassNotFoundException {
	return new FluxorPanelMetaData( panelID, 
					panelName, 
					getColumnMetaData() );
    }


    public Vector getPanelRowIDs() {
	Vector panelIDs = new Vector();
	HashMap panelRows = getPanelData();
	if (null != panelRows) {
	    panelIDs = new Vector(getPanelData().keySet());
	}
	return panelIDs;
    }

    public PanelRow getPanelRow(String rowID) {
	PanelRow panelRow = null;
	HashMap panelRows = getPanelData();
	if (null != panelRows) {
	    panelRow = (PanelRow)(panelRows.get(rowID));
	}
	return panelRow;
    }

    public HashMap getPanelRowData(String rowID) {
	HashMap columnData = null;
	PanelRow panelRow = getPanelRow(rowID);
	if (null != panelRow) {
	    columnData = panelRow.getColumnData();
	}
	return columnData;
    }

    public  HashMap getPanelData() {
	if (null != panelRowMap) {
	    return panelRowMap;
	}

	ListOf listOfReactions = model.getListOfReactions();
	panelRowMap = new HashMap();
	try {
	    for( int i = 0; i < model.getNumReactions(); i++ ){
		ReactionRow reactionRow = 
		    new ReactionRow( model.getReaction( i));
		panelRowMap.put( reactionRow.getID(), reactionRow );	    
	    }
	} catch( Exception e ) {
	    e.printStackTrace();
	}
	return panelRowMap;
    }



    
}
	
