/**
 * The SpeciesPanel class  collects (metadata) about the columns of the 
 * Species panel and libSBML Species objects. These libSBML Species objects are
 * referenced by their IDs, and form the link between the row data in the
 * GUI species panel, and between the {@link Metabolism} class libSBML Model 
 * object.
 * Note: the libSBML Species objects as well as the column metadata are
 * encapsulated in the {@link SpeciesRow} class.
 */
package edu.harvard.med.fluxorspreadsheet;
import libsbml.*;
import java.util.*;
public class SpeciesPanel implements MetabolismPanel {
    Model model;
    String panelID = "species";
    String panelName = "Species";
    private HashMap panelRowMap = null;
    FluxorColumnMetaData columnMetaData;

    public SpeciesPanel( Model model ) {
	this.model = model;
	
    }

    public String getPanelID(){
	return panelID;
    }
    public String getPanelName(){
	return panelName;
    }
    
    public FluxorColumnMetaData getColumnMetaData( ) 
	throws ClassNotFoundException {
	return SpeciesRow.getColumnMetaData();
    }


    public FluxorPanelMetaData getPanelMetaData() 
	throws ClassNotFoundException {
	return new FluxorPanelMetaData( panelID, 
					panelName, 
					getColumnMetaData() );
    }

    public Vector getPanelRowIDs() {
	Vector panelIDs = new Vector();
	HashMap panelRows = getPanelData();
	if (null != panelRows) {
	    panelIDs = new Vector(getPanelData().keySet());
	}
	return panelIDs;
    }

    public PanelRow getPanelRow(String rowID) {
	PanelRow panelRow = null;
	HashMap panelRows = getPanelData();
	if (null != panelRows) {
	    panelRow = (PanelRow)(panelRows.get(rowID));
	}
	return panelRow;
    }

    public HashMap getPanelRowData(String rowID) {
	HashMap columnData = null;
	PanelRow panelRow = getPanelRow(rowID);
	if (null != panelRow) {
	    columnData = panelRow.getColumnData();
	}
	return columnData;
    }

    public  HashMap getPanelData(){
	if (null != panelRowMap) {
	    return panelRowMap;
	}

	ListOf listOfSpecies = model.getListOfSpecies();
	panelRowMap = new HashMap();
	for( int i = 0; i < model.getNumSpecies(); i++ ){
	    try {
		SpeciesRow speciesRow = 
		    new SpeciesRow( model.getSpecies( i ) );
		panelRowMap.put( speciesRow.getID(), speciesRow);
	    } catch( Exception e ) {
		e.printStackTrace();
	    }
	}
	return panelRowMap;
    }
    
}
	
