package edu.harvard.med.fluxorspreadsheet;

import libsbml.*;
import java.util.*;
import java.lang.reflect.*;

       
/**
 * The SpeciesPanel class  collects (metadata) about the columns of the 
 * Species and libSBML Species objects. These libSBML Species objects are
 * referenced by their IDs, and form the link between the row data in the
 * GUI species panel, and the {@link Metabolism} class and the
 * libSBML Model object.
 * Note: the libSBML Species objects as well as the column metadata are
 * encapsulated in the {@link SpeciesRow} class.
 */

public class SpeciesRow implements MetabolismSpecies, PanelRow {
    Species species;
    private static final int ID = 0;
    private static final int NAME = 1;
    private static final int CLASS = 2;
    private static final int COMPARTMENT = 2;
    private static final int TYPE = 3;

    private static final String [][] speciesColumnMetaData =
	//     ID                   Name             Class                
    {
	{"speciesID",          "Species ID",     "java.lang.String"}, 
	{"speciesName",        "Species Name",   "java.lang.String"}, 
	{"speciesCompartment", "Compartment",    "java.lang.String"}, 
	{"speciesType",        "Species Type",   "java.lang.String"}, 
    };
 
    private static final Boolean[] speciesColumnEditableData =
	//  Editable            ID  
    {
	new Boolean(false), //"speciesID"          
	new Boolean(false), //"speciesName"        
	new Boolean(false), //"speciesCompartment" 
	new Boolean(false), //"speciesType" 
    };
    private static final String[] editableColumns = {};
    private static final String speciesID = speciesColumnMetaData[ID][ID];
    private static final String speciesName = speciesColumnMetaData[NAME][ID];
    private static final String speciesCompartment = 
	speciesColumnMetaData[COMPARTMENT][ID];
    private static final String speciesType = speciesColumnMetaData[TYPE][ID];

    public SpeciesRow( Species species ) {
	this.species = species;
    }

    public HashMap getColumnData() {
	HashMap columnData = new HashMap();
	columnData.put( speciesID, getSpeciesID() );
	columnData.put( speciesName, getSpeciesName() );
	columnData.put( speciesCompartment, getSpeciesCompartment() );
	columnData.put( speciesType, getSpeciesType() );
	return columnData;
    }
    public static FluxorColumnMetaData getColumnMetaData() 
	throws ClassNotFoundException{

	Vector columnIDs = new Vector();
	HashMap columnIndexMap = new HashMap();
	HashMap columnNameMap = new HashMap();
	HashMap columnClassMap = new HashMap();
	HashMap columnEditableMap = new HashMap();
	
	for(int i=0; i < speciesColumnMetaData.length; i++ ){
	    columnIDs.add( speciesColumnMetaData[i][ID] );
	    columnIndexMap.put( speciesColumnMetaData[i][ID], 
				new Integer(i) );
	    columnNameMap.put( speciesColumnMetaData[i][ID], 
			       speciesColumnMetaData[i][NAME] );
	    columnClassMap.put( speciesColumnMetaData[i][ID], 
				Class.forName( speciesColumnMetaData
					       [i][CLASS]));
	    columnEditableMap.put(speciesColumnMetaData[i][ID],
				  speciesColumnEditableData[i]);
	}
	return new FluxorColumnMetaData(columnIDs,
					columnIndexMap,
					columnNameMap,
					columnClassMap,
					columnEditableMap);
	
    }

    
    public void setColumnData( HashMap columnData ){
	if( columnData.containsKey( speciesID ) ){
	    setSpeciesID( (String)columnData.get( speciesID ) );
	} 
	if( columnData.containsKey( speciesName ) ){
	    setSpeciesName( (String)columnData.get( speciesName ) );
	} 
	if( columnData.containsKey( speciesType ) ){
	    setSpeciesType( (String)columnData.get( speciesType ) );
	} 
	if( columnData.containsKey( speciesCompartment ) ){
	    setSpeciesCompartment( (String)columnData.get( speciesCompartment ) );
	} 
    }

    public String getID(){
	return getSpeciesID();
    }
    public String getSpeciesID(){
	return species.getId();
    } 
    public void setID( String sid ) {
	setSpeciesID( sid );
    }
    public void setSpeciesID( String sid ) {
	species.setId( sid );
    }

    public String getSpeciesName( ){
	return species.getName();
    }
    public String getSpeciesCompartment( ){
	return species.getCompartment();
    }
    public String getSpeciesType( ){
	return species.getMetaId();
    }
    
    public void setSpeciesName( String speciesName ){
	species.setName( speciesName );
    }
    public void setSpeciesCompartment( String speciesCompartment ){
	species.setCompartment( speciesCompartment );
    }
    public void setSpeciesType( String speciesType ){
	species.setMetaId( speciesType );
    }
    public String toString(){
	return species.toSBML(2, 1);
    }
    static {
	System.loadLibrary("sbmlj");
    }
}




