#!/usr/local/bin/perl
# 
# writesbml: Uses ecolifluxlist.txt file created by writenative to create an sbml 2 
# representation of the same information.
#
# Copyright (c) 2003 by Wayne Rindone and the President and Fellows of Harvard University

AnalyzeParameters(@ARGV);

# set up the reference list of defined abbreviations

processabbreviationfile(\%abblist,"newabbreviations.txt");

open (IN,$infile);
open (OUT,">$outfile");
open (EXC,">$excfile");

writefilestart(OUT); 

$incount = 0;
$fluxcount = 0;
$filepart = 1;
@fluxlist = ();

# Write the initial annotations as we read through the file

$firstline = <IN>;  # skip header line

print "first line is '$firstline'\n";

while ($line = <IN>) {
   chomp($line);
   @cols = split "\t",$line;
   $numcols = @cols;
   $fluxcount++;
   print $line."\n";
   print " has $numcols fields\n";

   $thisfluxname = $cols[0];
   $firstchar = substr($thisfluxname,0,1);
   if ($firstchar lt  "0" || $firstchar gt "9") {
       $thisfluxname = $thisfluxname;
   }
   else {
       $thisfluxname = "_".$thisfluxname;
   }
   $thisfluxname = replacecharacter($thisfluxname,"-","_");
   $thisfluxname = replacecharacter($thisfluxname,".","_");
   print "final flux name is '".$thisfluxname."'\n";
   $fluxlist[$fluxcount-1] = $thisfluxname;
   $thisgenelist = $cols[1];
   if ($thisgenelist eq "") {
      @thesegenesymbols = ();
   }
   else {
      @thesegenesymbols = split ",",$thisgenelist;
   }
   $thisdescription = $cols[2];
   $thisecnumber = $cols[3];
   $thisreactiontext = $cols[4];
   $thislowerlimit = $cols[5];
   $thisupperlimit = $cols[6];
   $thisobjectivecoeff = $cols[7];

   $thisdescription = replacecharacter($thisdescription,"&","&amp;");
   $thisdescription = replacecharacter($thisdescription,">","&gt;");
   $thisdescription = replacecharacter($thisdescription,"<","&lt;");
   print "transformed flux description is ".$thisdescription."\n";

   $firstnoteline = "           <p>".$thisdescription.", ";
   if ($thisgenelist ne "") {
      $firstnoteline = $firstnoteline.$thisgenelist.", ";
   }
   $firstnoteline = $firstnoteline."flux ".$thisfluxname." </p>";

   print OUT $firstnoteline."\n";

   for ($i=0; $i<@thesegenesymbols; ++$i) {
       while (substr($thesegenesymbols[$i],0,1) eq " ") {
	   $thesegenesymbols[$i] = substr($thesegenesymbols[$i],1);
       }
   }

   $thisreactiontext = convertreaction($thisreactiontext);

   $thisreactiontext = processreaction($thisreactiontext,\%abblist,\%compoundlist,\%genelist,\%fluxreactants,\%fluxproducts,\%fluxmodifiers,\%fluxreverse,\@thesegenesymbols);

   addcompoundspecies(\@reactantnamearray,\%abblist,\%compoundlist);
   addcompoundspecies(\@productnamearray,\%abblist,\%compoundlist);
   addmodifierspecies(\@thesegenesymbols,$thisdescription,\%modifierlist);


   $secondnoteline = "           <p>".$thisfluxname.": ";
   if ($isexternalreaction == 1) {
       $secondnoteline = $secondnoteline."[e]".$thisreactiontext."</p>";
   }
   else {
       $secondnoteline = $secondnoteline.$thisreactiontext."</p>";
   }

   print OUT $secondnoteline."\n";

   if ($thislowerlimit eq "Inf") {
       $thislowerlimit = "INF";
   }
   elsif ($thislowerlimit eq "-Inf") {
       $thislowerlimit = "-INF";
   }

   if ($thisupperlimit eq "Inf") {
       $thisupperlimit = "INF";
   }
   elsif ($thisupperlimit eq "-Inf") {
       $thisupperlimit = "-INF";
   }

   $fluxlimits{$thisfluxname} = '             <annotation xmlns:flux="http://arep.med.harvard.edu/fluxns">
                     <flux:limit analysis="wild_type" upper="'.$thisupperlimit.'" lower="'.$thislowerlimit.'" objective="'.$thisobjectivecoeff.'"';
   if ($thisecnumber ne "") {
       $fluxlimits{$thisfluxname} = $fluxlimits{$thisfluxname}.' enzymecommission="'.$thisecnumber.'"';
   }
   $fluxlimits{$thisfluxname} = $fluxlimits{$thisfluxname}.'/>
 ';
   $fluxlimits{$thisfluxname} = $fluxlimits{$thisfluxname}.'                    <flux:limit analysis="mutant_fba" upper="'.$thisupperlimit.'" lower="'.$thislowerlimit.'" objective="'.$thisobjectivecoeff.'"';
   $fluxlimits{$thisfluxname} = $fluxlimits{$thisfluxname}.'/>
';
   $fluxlimits{$thisfluxname} = $fluxlimits{$thisfluxname}.'                     <flux:limit analysis="mutant_moma" upper="'.$thisupperlimit.'" lower="'.$thislowerlimit.'"';
   $fluxlimits{$thisfluxname} = $fluxlimits{$thisfluxname}.'/>
	     ';
   $fluxlimits{$thisfluxname} = $fluxlimits{$thisfluxname}.'</annotation>
';
   $fluxupper{$thisfluxname} = $thisupperlimit;

   $fluxdescription{$thisfluxname} = $thisdescription;

   $incount++;
}

close (IN);

# finish off the annotations and produce the compartment list and first line of Species list


print OUT '         </body>
      </notes>
      <annotation xmlns:flux="http://arep.med.harvard.edu/fluxns">
       <flux:listOfAnalysisTypes>
         <flux:analysisType id="fba" name="Flux Balance Analysis"/>
         <flux:analysisType id="moma" name="Minimization of Metabolic Adjustment"/>
       </flux:listOfAnalysisTypes>
       <flux:listOfAnalyses>
         <flux:analysis id="wild_type" name="Wild-Type FBA" analysisType="fba"/>
         <flux:analysis id="mutant_fba" name="Mutant FBA" analysisType="fba"/>
         <flux:analysis id="mutant_moma" name="Mutant MOMA" analysisType="moma"/>
       </flux:listOfAnalyses>
      </annotation>
      <listOfCompartments>
         <compartment id="cytoplasm"/>
         <compartment id="extracellular"/>
      </listOfCompartments>
      <listOfSpecies>
';

while (($compoundkeyname, $compoundvalue) = each %compoundlist) {
   print $compoundkeyname." is ".$compoundvalue."\n";
   $compoundvalue = replacecharacter($compoundvalue,">","&gt;");
   $compoundvalue = replacecharacter($compoundvalue,"<","&lt;");
   $compoundvalue = replacecharacter($compoundvalue,"&","&amp;");
   print "transformed ".$compoundkeyname." is ".$compoundvalue." and substr with offset -2 is '".substr($compoundkeyname,-2)."\n";
   if (substr($compoundkeyname,-2) eq "xt" || substr($compoundkeyname,-2) eq "XT") {
       $compartment = "extracellular";
       $boundarycondition = "true";
   }
   else {
       $compartment = "cytoplasm";
       $boundarycondition = "false";
   }

   print OUT '	 <species id="'.$compoundkeyname.'" name="'.$compoundvalue.' (biopax:smallMolecule)" 
		  compartment="'.$compartment.'" boundaryCondition="'.$boundarycondition.'"/>
';
}
while (($modifierkeyname, $modifiervalue) = each %modifierlist) {
   print $modifierkeyname." is ".$modifiervalue."\n";

   print OUT '	 <species id="'.$modifierkeyname.'" name="'.$modifiervalue.' (biopax:gene)"
		  compartment="cytoplasm" boundaryCondition="false"/>
';
}
# finish Species list and start Reaction list

print OUT '      </listOfSpecies>
      <listOfReactions>
';

for ($i=0; $i<@fluxlist; ++$i) {
    print OUT '         <reaction id="'.$fluxlist[$i].'" name="'.$fluxdescription{$fluxlist[$i]}.
'" reversible="'.$fluxreverse{$fluxlist[$i]}.'">
'.$fluxlimits{$fluxlist[$i]}.$fluxreactants{$fluxlist[$i]}.$fluxproducts{$fluxlist[$i]}.
$fluxmodifiers{$fluxlist[$i]}.'         </reaction>
';
}

print OUT '      </listOfReactions>
   </model>
</sbml>
';


$totalinputlines = $incount;

close (OUT);
close (EXC);

print "Statistics generated by writenative\n\n";
print "  A total of $incount lines were processed, listing a total of $fluxcount different fluxes\n";
print "Processing complete.\n";


sub AnalyzeParameters {
  my $i;

  if (@_==0) {
    Usage();
    exit;
  }

  for ($i=0; $i<@_; ++$i) {
    if ($_[$i] eq "-i") {
      $infile = $_[$i+1];
      ++$i;
    }
    if ($_[$i] eq "-o") {
      $outfile = $_[$i+1];
      ++$i;
    }
    if ($_[$i] eq "-e") {
      $excfile = $_[$i+1];
      ++$i;
    }
  }

  if ($infile eq "") {
      print "\nERROR: no input load file specified.  Use -i argument.\n";
      exit;
  }

  if ($outfile eq "") {
    print STDERR "Output file name defaulting to ".$infile."sbml2.xml.\n";
    $outfile = $infile."sbml2.xml";
  }

  if ($excfile eq "") {
    print STDERR "Exception list file name defaulting to $infile.exception.\n";
    $excfile = $infile.".exception";
  }
}

sub Usage {
  print <<_USAGE
    writesbml: Creates the sbml 2 representation of the native flux balance model
               file that is provided

Syntax: 
  writesbml -i native-load-file -o smbl-2-file -e list-of-exceptions

  if -o not provided, smbl-2-file name defaults to input
  native-load-file name followed by "sbml2.xml"

  if -e not provided, list-of-exceptions name defaults to 
  input native-load-file name followed by ".exceptions"

_USAGE
;
} 

sub processreaction {

   my $reactiontext = $_[0];
   my $abblist_ref = $_[1];
   my $compoundlist_ref = $_[2];
   my $genelist_ref = $_[3];
   my $fluxreactants_ref = $_[4];
   my $fluxproducts_ref = $_[5];
   my $fluxmodifiers_ref = $_[6];
   my $fluxreverse_ref = $_[7];
   local *genenames = $_[8];

#   local *reactioncols = $_[0];
#   my $thesenumcols = $_[1];
#   my $inreversible = $_[2];

   $newreactiontext = "";
   $numreactants = 0;
   $numproducts = 0;

    $$fluxreverse_ref{$thisfluxname} = "false";
    $newreactiontext = replacecharacter($reactiontext,"<","&lt;");
    if ($newreactiontext ne $reactiontext) {
        $$fluxreverse_ref{$thisfluxname} = "true";
    }
    $newreactiontext= replacecharacter($newreactiontext,">","&gt;");
    $delimiterloc = index($reactiontext,"-->");
    if ($delimiterloc > 0) {
        $reactantstring = substr($reactiontext,0,$delimiterloc-1);
    }
    else {
        $reactantstring = "";
    }
print "reactantstring is '".$reactantstring."'\n";
    $reactantstring = truncatestring($reactantstring);
print "reactantstring is '".$reactantstring."'\n";
    @reactantcols = split ' \\+ ',$reactantstring;
print "reactant cols are ";
    @reactantnamearray = ();
    @reactcoeffarray = ();
    for ($i=0; $i<@reactantcols; ++$i) {
	if ($isexternalreaction == 1) {
           $reactantcols[$i] = $reactantcols[$i]."xt";
        }
print $reactantcols[$i].":";
        if (index($reactantcols[$i]," ") > -1) {
            @reactantpieces = split " ",$reactantcols[$i];
            $reactcoeffarray[$i] = $reactantpieces[0];
            $reactantnamearray[$i] = $reactantpieces[1];
        }
        else {
            $reactantnamearray[$i] = $reactantcols[$i];
        }
    }
print "\n";
    if (@reactantnamearray > 0) {
        $$fluxreactants_ref{$thisfluxname} = "            <listOfReactants>\n";
        for ($i=0; $i<@reactantnamearray; ++$i) {
            $reactantnamearray[$i] = prependdigits($reactantnamearray[$i]);
print "reactantnamearray ".$i." is '".$reactantnamearray[$i]."'\n";
print "reactcoeffarray ".$i." is '".$reactcoeffarray[$i]."'\n";
#print "reactdenominatorarray ".$i." is '".$reactdenominatorarray[$i]."'\n";
            if ($reactcoeffarray[$i] eq "") {
                $$fluxreactants_ref{$thisfluxname} = $$fluxreactants_ref{$thisfluxname}.'               <speciesReference species="'.$reactantnamearray[$i].'" stoichiometry="1"';
            }
            else {
                $$fluxreactants_ref{$thisfluxname} = $$fluxreactants_ref{$thisfluxname}.'               <speciesReference species="'.$reactantnamearray[$i].'" stoichiometry="'.$reactcoeffarray[$i].'"';
#	    if ($reactdenominatorarray[$i] ne "") {
#	        $$fluxreactants_ref{$thisfluxname} = $$fluxreactants_ref{$thisfluxname}.' denominator = "'.$reactdenominatorarray[$i].'"';
#	    }
            }
            $$fluxreactants_ref{$thisfluxname} = $$fluxreactants_ref{$thisfluxname}.'/>
';
        }
        $$fluxreactants_ref{$thisfluxname} = $$fluxreactants_ref{$thisfluxname}.'            </listOfReactants>
';
    }
    $productstring = substr($reactiontext,-(length($reactiontext)-$delimiterloc-4));
print "productstring is '".$productstring."'\n";
    $productstring = truncatestring($productstring);
print "productstring is '".$productstring."'\n";
    @productcols = split ' \\+ ',$productstring;
print "product cols are ";
    @productnamearray = ();
    @productcoeffarray = ();
#    @productdenominatorarray = ();
    for ($i=0; $i<@productcols; ++$i) {
	if ($isexternalreaction == 1) {
           $productcols[$i] = $productcols[$i]."xt";
        }
print $productcols[$i].":";
        if (index($productcols[$i]," ") > -1) {
            @productpieces = split " ",$productcols[$i];
            $productcoeffarray[$i] = $productpieces[0];
#	    $pdelimiterloc = index($productcoeffarray[$i],".");
#	    if ($pdelimiterloc > -1) {
# print "starting coefficient is ".$productcoeffarray[$i]."\nperiod location is ".$pdelimiterloc."\n";
#                $numdecimals = length($productcoeffarray[$i])-$pdelimiterloc-1;
#		$productdenominatorarray[$i] = 10**$numdecimals;
#print "denominator is ".$productdenominatorarray[$i]."\n";
#                $productcoeffarray[$i] = $productdenominatorarray[$i]*$productcoeffarray[$i];
#print "numerator is ".$productcoeffarray[$i];
#            }
            $productnamearray[$i] = $productpieces[1];
        }
        else {
            $productnamearray[$i] = $productcols[$i];
        }
    }
print "\n";
    $$fluxproducts_ref{$thisfluxname} = "            <listOfProducts>\n";
    for ($i=0; $i<@productnamearray; ++$i) {
        $productnamearray[$i] = prependdigits($productnamearray[$i]);
print "productnamearray ".$i." is '".$productnamearray[$i]."'\n";
print "productcoeffarray ".$i." is '".$productcoeffarray[$i]."'\n";
#print "productdenominatorarray ".$i." is '".$productdenominatorarray[$i]."'\n";
        if ($productcoeffarray[$i] eq "") {
            $$fluxproducts_ref{$thisfluxname} = $$fluxproducts_ref{$thisfluxname}.'               <speciesReference species="'.$productnamearray[$i].'" stoichiometry="1"';
        }
        else {
            $$fluxproducts_ref{$thisfluxname} = $$fluxproducts_ref{$thisfluxname}.'               <speciesReference species="'.$productnamearray[$i].'" stoichiometry="'.$productcoeffarray[$i].'"';
#	    if ($productdenominatorarray[$i] ne "") {
#	        $$fluxproducts_ref{$thisfluxname} = $$fluxproducts_ref{$thisfluxname}.' denominator = "'.$productdenominatorarray[$i].'"';
#	    }
        }
        $$fluxproducts_ref{$thisfluxname} = $$fluxproducts_ref{$thisfluxname}.'/>
';
    }
    $$fluxproducts_ref{$thisfluxname} = $$fluxproducts_ref{$thisfluxname}."            </listOfProducts>
";
    if (@genenames > 0) {
        $$fluxmodifiers_ref{$thisfluxname} = "            <listOfModifiers>\n";
        for ($i=0; $i<@genenames; ++$i) {
print "genename ".$i." is '".$genenames[$i]."'\n";
            $$fluxmodifiers_ref{$thisfluxname} = $$fluxmodifiers_ref{$thisfluxname}.'               <modifierSpeciesReference species="'.$genenames[$i].'"/>
';
        }
        $$fluxmodifiers_ref{$thisfluxname} = $$fluxmodifiers_ref{$thisfluxname}."            </listOfModifiers>
";
    }
    return $newreactiontext;
}

sub convertreaction {

   my $reactiontext = $_[0];
   local *genenames = $_[8];

#   local *reactioncols = $_[0];
#   my $thesenumcols = $_[1];
#   my $inreversible = $_[2];

   $newreactiontext = $reactiontext;

   $isexternalreaction = 0;
   if (index($reactiontext,"[") == 0) {
       if (index($reactiontext,"[e]") == 0) {
          $isexternalreaction = 1;
       }
       $newreactiontext = substr($reactiontext,3);
    }

    $newreactiontext = replacecharacter($newreactiontext,"(","");
    $newreactiontext = replacecharacter($newreactiontext,")","");
    $newreactiontext = replacecharacter($newreactiontext,"[e]","xt");
    $newreactiontext = replacecharacter($newreactiontext,"[c]","");
    $newreactiontext = replacecharacter($newreactiontext,"<==>","<-->");

    return $newreactiontext;
}

sub processabbreviationfile {

    my $abblist_ref = $_[0];
    my $abbfilename = $_[1];

    open (INABB,$abbfilename);

    my $line = <INABB>;
    my @cols = ();

    while ($line = <INABB>) {
        chomp($line);
        @cols = split "\t",$line;
        $$abblist_ref{$cols[0]} = $cols[1];
    }    

    close (INABB);
}

sub addcompoundspecies {

   local *compoundarray = $_[0];
   my $abblist_ref = $_[1];
   my $compoundlist_ref = $_[2];
   my $i;

   for ($i=0; $i<@compoundarray; ++$i) {
print "addcompoundspecies: checking ".$compoundarray[$i]."\n";      
       if (!exists $$compoundlist_ref{$compoundarray[$i]}) {
# strip initial underscore if necessary
           $thiscompound = $compoundarray[$i];
	   if (index($thiscompound,"13") > -1) {
               print "thiscompound is '".$thiscompound."' with first character '".substr($thiscompound,0,1)."'\n";
	   }
           if (substr($thiscompound,0,1) eq "_") {
               $thiscompound = substr($thiscompound,1);
	   }
	   if (index($thiscompound,"13") > -1) {
               print "now thiscompound is '".$thiscompound."' with first character '".substr($thiscompound,0,1)."'\n";
	   }
           if (index($thiscompound,"_") > 0) {
               $thiscompound = replacecharacter($thiscompound,"_","-");
               $thiscompound = replacecharacter($thiscompound,"-EC","_EC");
               $thiscompound = replacecharacter($thiscompound,"-EC","_EC");
               $thiscompound = replacecharacter($thiscompound,"D-D","D,D");
               $thiscompound = replacecharacter($thiscompound,"L-D","L,D");
	       print "we changed ".$compoundarray[$i]." to ".$thiscompound." to look it up.\n";
           }
           if (exists $$abblist_ref{$thiscompound}) {
               $$compoundlist_ref{$compoundarray[$i]} = $$abblist_ref{$thiscompound};
           }
           else {
               print "WARNING: Compound '".$thiscompound."' was not found in newabbreviations.txt\n";
               $$compoundlist_ref{$compoundarray[$i]} = $thiscompound;
           }  
       }
   }
}
sub addmodifierspecies {

   local *genearray = $_[0];
   my $fullname = $_[1];
   my $modifierlist_ref = $_[2];
   my $i;

   for ($i=0; $i<@genearray; ++$i) {
print "addmodifierspecies: checking ".$genearray[$i]."\n";      
       if (!exists $$modifierlist_ref{$genearray[$i]}) {
           $$modifierlist_ref{$genearray[$i]} = $fullname;
       }
       elsif ($$modifierlist_ref{$genearray[$i]} ne $fullname) {
           print "WARNING: Original ".$genearray[$i]." description was ".$$modifierlist_ref{$genearray[$i]}.
              ", but it is now described as ".$fullname."\n";
       }
   }
}
sub arrayloc {

    local *thisarray = $_[0];
    $lookupword = $_[1];
    $arraysize = $_[2];

    $foundindex = -1;

    for ($i=0; $i<@thisarray && $foundindex == -1; ++$i) {

        if ($foundindex < 0) {

            if ($lookupword eq $thisarray[$i]) {

                $foundindex = $i;
            }

        }

    }

    return $foundindex;

}

sub replacecharacter {

    my $fullstring  = $_[0];
    my $lookupchar = $_[1];
    my $newstring = $_[2];

    print "original fullstring is "."'".$fullstring."'\n";
    $lookuplength = length ($lookupchar);
    print "string to replace is "."'".$lookupchar."', which contains ".$lookuplength." characters.\n";
    print "and it will be replaced with '".$newstring."'\n";

    $newfullstring = $fullstring;
    $firstcharloc = index($fullstring,$lookupchar);
    print "first location is $firstcharloc\n";
    while ($firstcharloc > -1) {
        $newfullstring = substr($fullstring,0,$firstcharloc);
        print "new fullstring is "."'".$newfullstring."'\n";
        $newfullstring = $newfullstring.$newstring.substr($fullstring,$firstcharloc+$lookuplength);
        $fullstring = $newfullstring;
        $firstcharloc = index($fullstring,$lookupchar);
        print "first location is $firstcharloc\n";
    }

    print "new fullstring is "."'".$newfullstring."'\n";

    return $newfullstring;

}
    
sub prependdigits {

    my $fullstring  = $_[0];

    print "original compound abbreviation is "."'".$fullstring."'\n";

    $firstchar = substr($fullstring,0,1);
    print "firstchar is '".$firstchar."'\n";
    if ($firstchar lt  "0" || $firstchar gt "9") {
        $newfullstring =$fullstring;
    }
    else {
        $newfullstring = "_".$fullstring;
    }

    $newfullstring = replacecharacter($newfullstring,"-","_");
    $newfullstring = replacecharacter($newfullstring,",","_");

    print "new compound abbreviation is "."'".$newfullstring."'\n";

    return $newfullstring;

}
    
sub truncatestring {

    my $fullstring  = $_[0];

    my $newstring = $fullstring;

    print "original fullstring is "."'".$fullstring."'\n";

    while (substr($newstring,-1) eq " ") {
        $newstring = substr($newstring,0,length($newstring)-1);
    }

    print "new string is "."'".$newstring."'\n";

    return $newstring;

}
    
sub writefilestart {

    print OUT '<?xml version="1.0" encoding="utf-8"?>
<sbml xmlns="http://www.sbml.org/sbml/level2" version="1" level="2">
   <model id="JR904" name="Ecoli Palsson Model JR904">
      <notes>
         <body xmlns="http://www.w3.org/1999/xhtml">
';

}
