#ifndef SBW_BROKER
#define SBW_BROKER

#include "Host.h"
#include "SBWBrokerRPC.h"
#include "InstanceTable.h"
#include "ModuleRegistry.h"
#include "Instance.h"
#include "BrokerInterface.h"
#include "BrokerInstance.h"

#include "SBW/MethodTable.h"
#include "SBW/ModuleDescriptor.h"
#include "SBW/ServiceDescriptor.h"

#include <string>
#include <vector>

using namespace SystemsBiologyWorkbench;
using namespace SystemsBiologyWorkbench::Broker;
using namespace std;

namespace SystemsBiologyWorkbench
{
	namespace Broker
	{

		class Broker
		{
		public:
			Broker();
			int getModuleInstance(std::string name);
			DataBlockWriter getModuleDescriptor(std::string name, bool includeRunning);
			std::vector<int> getExistingModuleInstanceIds();
			std::vector<std::string> getServiceCategories(std::string parentCategory);
			std::vector < DataBlockWriter > findServices(std::string category, bool recursive);
			void registerModule(std::string moduleName, std::string moduleDisplayName,
				int moduleType, std::string commandLine, std::string help);
			void changeModuleName(int moduleId, std::string newModuleName);
			void registerService(std::string moduleName, std::string serviceName,
				std::string serviceDisplayName, std::string category,
				std::string help);
			void unregisterModule(std::string moduleName);
			int linkBroker(std::string hostName);
			void shutdownBroker();
			std::string getVersion();
			DataBlockWriter getModuleDescriptor(int moduleId);
			std::vector< DataBlockWriter > getModuleDescriptors(bool localOnly, bool includeRunning);
			DataBlockWriter getServiceDescriptor(int moduleId, int serviceId);
			DataBlockWriter getServiceDescriptor(int moduleId, std::string servName);
			std::vector<DataBlockWriter> getServiceDescriptors(int moduleId);
			std::vector<DataBlockWriter> getServiceDescriptors(std::string name);
			DataBlockWriter getListOfModules();
			DataBlockWriter getMethodIds(int moduleId,int serviceId);
			DataBlockWriter getMethodIds(string moduleName,string serviceName);
			bool isRemoteInstance(int nModuleId);

		private:
			int getModuleInstanceLocal(std::string moduleName);
			int getModuleInstanceRemote(std::string moduleName, Host *host);
			DataBlockWriter getModuleDescriptorLocal(std::string moduleName,
				bool includeRunning);
			DataBlockWriter getModuleDescriptorRemote(std::string moduleName, Host *host,
				bool includeRunning);
			DataBlockWriter getModuleDescriptorLocal(int moduleId, Instance *moduleInst);
			DataBlockWriter getModuleDescriptorRemote(int moduleId, Instance *moduleInst);
			DataBlockWriter getServiceDescriptorLocal(int moduleId, std::string servName,
				Instance *moduleInst);
			DataBlockWriter getServiceDescriptorRemote(int moduleId, std::string servName,
				Instance *moduleInst);
			std::vector<DataBlockWriter> getServiceDescriptorsLocal(std::string moduleName);
			std::vector<DataBlockWriter> getServiceDescriptorsRemote(std::string moduleName, Host *host);
			void changeModuleNameLocal(int moduleId, Instance *moduleInst,
				std::string newModuleName);
			void changeModuleNameRemote(int moduleId, Instance *moduleInst,
				std::string newModuleName);
			int startNewLocalModuleInstance(ModuleDescriptor *md);
			DataBlockWriter listForModuleDescriptor(ModuleDescriptor *md);
			DataBlockWriter listForModuleDescriptor(std::string moduleName, std::string displayName,
				int mgmtType, std::string cmdLine, std::string help);
			DataBlockWriter listForServiceDescriptor(std::string moduleName, std::string serviceName,
				std::string serviceDisplayName, std::string category,
				std::string helpString);
			DataBlockWriter listForServiceDescriptor(std::string moduleName, ServiceDescriptor *sd);
			bool matchCategory(std::string category, std::string toMatch, int start);
			std::string normalizeModuleName(std::string moduleName);
			std::string normalizeServiceName(std::string serviceName);
			void checkModuleId(int moduleId);
			void checkBrokerId(int brokerId);
			Host *getHostPart(std::string name);
			std::string getModuleNamePart(std::string name);
			Instance *getInstanceForModuleId(int moduleId);
			ModuleDescriptor *getDescriptor(int brokerId, std::string moduleName);
			std::vector<ModuleDescriptor *> getDescriptors();
			std::vector<ModuleDescriptor *> getDescriptors(int brokerId);
			int getBrokerId(Host *host);
			std::vector<int> getBrokerIds();
			BrokerInterface *getBrokerInterface(Host *host);
			BrokerInterface *getBrokerInterface(int bId);
			DataBlockWriter mungeModuleDescriptor(DataBlockWriter list, std::string modName, Host *host);
			DataBlockWriter mungeServiceDescriptor(DataBlockWriter list, std::string modName, Host *host);
			void addBrokerInstance();
			void addBrokerRegistryEntry();
			static SBWBrokerRPC *rpc;
			static InstanceTable *instanceTable;
			static ModuleRegistry *moduleRegistry;
		};

	}
}

#endif
