#ifndef SBW_BROKER_HOST
#define SBW_BROKER_HOST

#include <string>

namespace SystemsBiologyWorkbench
{
	namespace Broker
	{

		class Host
		{
		private:
			/** The host name used in a query; typically something a user supplied. **/
			std::string givenHostName;

			/** The fully qualified domain name. **/
			std::string fullHostName;

			/** The host IPv4 address as a string; e.g., "123.123.123.123". **/
			std::string hostAddressString;
		public:
			/**
			* Class constructor.  This takes a host name and constructs an object
			* that internally records the name, the fully qualified name, the
			* IP address, and possibly other data.
			* <p>
			* @param name the name of the host
			**/
			Host(std::string  name);

			/**
			* Class constructor.  This takes a host name and IP address and
			* constructs an object that internally records the name, the fully
			* qualified name, the IP address, and possibly other data.
			* <p>
			* @param name the name of the host
			**/
			Host(std::string name, std::string address);
			/**
			* Returns the name of this host.
			**/
			std::string getName();
			/**
			* Returns the fully qualified domain name of this host.
			**/
			std::string getFullName();

			/**
			* Returns this host's IP address as a string.
			**/
			std::string getAddress();
			/**
			* Indicates whether some other object is "equal to" this one.  This
			* method implements equality for {@link Host} objects,
			* overriding the method of the same name on the basic Java {@link
			* Object} class.  If the given object is a {@link Host}
			* object and its various fields are equal to this object's,
			* then this method returns true, else it returns false.
			* <p>
			* @param other the object with which to compare this one.
			* @return true if the argument is a {@link Host} object that
			* refers to the same module as this object, false otherwise.
			* @see java.lang.Object#equals
			**/
			bool equals(Host *other);
		};

	}
}

#endif

