#ifndef SBW_BROKER_INSTANCE_BASE
#define SBW_BROKER_INSTANCE_BASE

#include "SBW/SBWOSSocket.h"

#include "Instance.h"
#include "Host.h"

#include "SBW/portableOS.h"

#include <string>
#include <sstream>

using namespace std;

namespace SystemsBiologyWorkbench
{
	namespace Broker
	{
		class InstanceBase : public Instance
		{
		public:
			/** The local identifier for this instance.  **/		
			int myId;
		protected:
			/** The identifier of the broker responsible for this instance. **/
			int myBrokerId;
			/** The host on which this instance is running. **/
			Host *myHost;
			/** The full name of this instance sans host prefix. **/
			std::string myName;
			/** The full name, including host prefix, of this instance. **/
			std::string myFullName;
			/** The full name, including host prefix, of this instance. **/
			std::string myInternalName;
			/** The type of this instance, either "module" or "broker". **/
			std::string myType;
			/**
			* Protected constructor for this class.  There never needs to be
			* an actual instance of {@link InstanceBase} per se, since all
			* the work is done in subclasses.
			**/
			InstanceBase() {}
		public: 
			virtual int getId() { return myId; }
			virtual int getBrokerId() { return myBrokerId; }
			virtual Host *getHost() { return myHost; }
			virtual std::string getName() { return myName; }
			virtual std::string getFullName() { return myFullName; }
			virtual std::string getInternalName() { return myInternalName; }
			virtual std::string getNameForMessage() 
			{ 
				std::stringstream sResult;
				sResult << myType << " " << myId << " ('" << myFullName << "')";
				return sResult.str(); 
			}
		};
	}
}

#endif
