#ifndef SBW_INSTANCE_TABLE
#define SBW_INSTANCE_TABLE

#include "Instance.h"
#include "IntegerHashtable.h"
#include "Host.h"

#include <string>
#include <vector>
#ifndef WIN32
// don't forget to add a std::string to stl_has_fun.h
#include <ext/hash_map>
using namespace std;
using namespace __gnu_cxx;
#else
#include <hash_map>
using namespace std;
#endif

namespace SystemsBiologyWorkbench
{
	namespace Broker
	{
		class InstanceTable
		{
		private:
			/** The actual instance table, a singleton. **/
			static InstanceTable *table;
			/** The hash table mapping identifers to instances. **/
			IntegerHashtable<Instance *> idHash;
			/** The hash table mapping module names to instances. **/
			typedef pair< string , Instance *> StringInstance;
			hash_map< string , Instance *> nameHash;			
			/** The internal counter of identifers used by {@link #getNextId()}. **/
			int nextId;
			InstanceTable() : instanceMutex("instanceTableMutex")	{nextId = 0;}
			SBWOSMutex instanceMutex ;
		public:
			static InstanceTable *getTable();
			void addInstance(Instance *inst);

			void removeInstance(Instance *inst);
			void removeInstance(int moduleId);

			Instance *getInstance(int moduleId);
			Instance *getInstance(Host *host, string moduleName);

			vector<Instance *> getInstances(int brokerId);
			vector<Instance *> getInstances();
			
			bool exists(int moduleId);
			int getNextId();
		private:
			InstanceTable *readResolve();

		};
	}
}
#endif
