#include "SBW.h"
#include "SBWBrokerRPC.h"

#include "SBW/SBWCommunicationException.h"
#include "SBW/SBWApplicationException.h"

using namespace SystemsBiologyWorkbench::Broker;
using namespace SystemsBiologyWorkbench;

SBWRPC *Broker::SBW::getInternalAPI()
{
	if (rpc == NULL)
	{
		rpc = new SBWBrokerRPC(); 
	}
	return rpc;
}
void Broker::SBW::setInternalAPI(SBWBrokerRPC *r)
{
	rpc = r;
}

std::string Broker::SBW::normalizeCategoryName(std::string category)
{
	if (StringUtil::empty(category))
		return "/";

	//category = category.trim();
	if (category == "")
		return "/";
	else
	{
		try
		{
			std::string newCategoryStr = "";
			int lastPos = 0;
			int currPos = 0;

			if (category.find('/') == 0)
				lastPos = 1;

			while ((currPos = category.find('/', lastPos)) != category.npos )
			{
				std::string piece = category.substr(lastPos, currPos);
				newCategoryStr = newCategoryStr + "/" + piece;
				lastPos = currPos + 1;
			}
			currPos = category.length();

			newCategoryStr =
				newCategoryStr + "/"
				+ category.substr(lastPos, currPos);

			return newCategoryStr;
		}
		catch (...)
		{
			return category;
		}
	}
}
std::vector<std::string> Broker::SBW::getServiceNamesFromModule(int moduleId)
{
	std::vector<std::string> oTemp;
	std::string sTemp;
	TRACE("Querying module " << moduleId << " about its services");

	DataBlockWriter args;

	try
	{
		DataBlockReader result = rpc->call(moduleId, SYSTEM_SERVICE,
			GET_SERVICES_METHOD, args);

		// Normalize the strings returned by trimming possible whitespace.

		result >> oTemp;
	}
	catch (SBWException *e)
	{
		std::string msg = "Unable to get services list from module " + Properties::toString(moduleId);
		TRACE(msg);
		throw new SBWCommunicationException(e->getMessage(),
			e->getDetailedMessage());
	}
	return oTemp;
}

SBWBrokerRPC *Broker::SBW::rpc = NULL;
