#include "Sys.h"

#include "SBW/config.h"
#include "SBW/SBWOSSocket.h"
#include "SBW/DataBlockReader.h"
#include "SBW/DataBlockWriter.h"
#include "SBW/SBWApplicationException.h"
#include "SBW/SBWCommunicationException.h"
#include "SBW/portableOS.h"
#ifndef WIN32
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>
#endif

using namespace SystemsBiologyWorkbench::Broker;
using namespace SystemsBiologyWorkbench;
using namespace std;

std::string Sys::readString (SOCKET sock)
{
	std::string sResult;	
	int length = readInteger(sock);
	char *oBuffer = (char *) malloc ( length+1 * sizeof(char));
	memset(oBuffer,0,length * sizeof(char));
	try 
	{
		receive(sock,(unsigned char*)oBuffer,length);
	}
	catch (SBWCommunicationException *exception)
	{
		free (oBuffer);
		throw exception;
	}
	oBuffer[length]=0;
	sResult = oBuffer;
	free (oBuffer);
	oBuffer = NULL;
	return sResult;
}
int Sys::readInteger(SOCKET sock)
{
	int result = -1;
	unsigned char *buffer = new unsigned char[4];
	memset(buffer,0,4);
	try 
	{
		receive(sock, buffer, 4);
	}
	catch (SBWCommunicationException *exception)
	{
		delete[] buffer;
		throw exception;
	}

	DataBlockReader *oTemp = new DataBlockReader(buffer, 4, false);
	oTemp->getImpl()->getWithoutType(result);
	delete oTemp;
	buffer = NULL;
	return result; 
}
void Sys::receive(SOCKET sock, unsigned char *data, Integer length)
{
	Integer cursor = 0 ;

	while (cursor != length)
	{
		Integer result = recv(sock, (char *)data + cursor, length - cursor, 0);

		if (result == 0)
			throw new SBWCommunicationException("graceful disconnect while reading");

		if (result == SOCKET_ERROR)
			throw new SBWCommunicationException("SOCKET_ERROR");

		cursor += result ;
	}
}

std::string Sys::fullyQualifiedHostName(std::string host)
{	
	hostent* oHost = gethostbyname(host.c_str());
	if (oHost != NULL)
		return oHost->h_name;
	else
	{
		TRACE("couldn't resolve host")
			return host;
	}
}
std::string Sys::getHostName()
{
	std::string thisHost;
	char shortname[256];
	int nResult = gethostname(shortname, 255);
	if (  nResult  >= 0)
	{
		thisHost = shortname;		
	} 
	return thisHost;
}
std::string Sys::getHostAddress(std::string host)
{
	hostent* oHost = gethostbyname(host.c_str());
	if (oHost == NULL)
	{
		TRACE("couldn't get host address")
			return host;
	}
	unsigned long addr = *(( unsigned long *) oHost->h_addr_list[0]);

	in_addr oTemp;
#ifdef WIN32
	oTemp.S_un.S_addr = addr;
#else
	oTemp.s_addr = (in_addr_t) addr;
#endif
	std::string sResult ( inet_ntoa(oTemp ));	
	return sResult;
}
std::string Sys::getHostAddress()
{	
	return getHostAddress("localhost");
}
bool Sys::isAddress(std::string str)
{
	throw new SBWApplicationException("not yet implemented");
}
bool Sys::portInUse(int port, bool remoteAllowed)
{
	return SBWOSSocket::portInUse(port);
}
int Sys::findFreePort(std::vector<int> portRange, bool remoteAccessible)
{
	int low = portRange[0];
	int high = portRange[1];

	for (int port = low; port <= high; port++) {
		if (!  portInUse(port, remoteAccessible))
			return port;
		else
			TRACE("Port " << port <<  " appears to be in use");
	}
	return -1;                      // Didn't find one.
}
bool Sys::isSecure()
{
#ifdef INSECURE
	return false;
#else
	return true;
#endif
}
