/**
  * Filename    : NOMImpl.c
  * Description : SBW Interface to NOM
  * Author(s)   : Sri Paladugu <spaladug@kgi.edu>
  * Organization: Keck Graduate Institute (KGI) 
  * Created     : 
  * Revision    : $Id: NomImpl.c,v 1.1 2004/09/17 00:35:34 zucker Exp $ 
  * Source      : $Source: /var/lib/cvs/Biospice/FluxorPipeline/Metatool/src/SBW-NOM/NOM/src/NomImpl.c,v $ 
  * 
  * Copyright 2004 Keck Graduate Institute 
  * 
  * This library is free software; you can redistribute it and/or modify it 
  * under the terms of the GNU Lesser General Public License as published 
  * by the Free Software Foundation; either version 2.1 of the License, or 
  * any later version. 
  * 
  * This library is distributed in the hope that it will be useful, but 
  * WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF 
  * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  The software and 
  * documentation provided hereunder is on an "as is" basis, and the 
  * California Institute of Technology and Japan Science and Technology 
  * Corporation have no obligations to provide maintenance, support, 
  * updates, enhancements or modifications.  In no event shall the 
  * California Institute of Technology or the Japan Science and Technology 
  * Corporation be liable to any party for direct, indirect, special, 
  * incidental or consequential damages, including lost profits, arising 
  * out of the use of this software and its documentation, even if the 
  * California Institute of Technology and/or Japan Science and Technology 
  * Corporation have been advised of the possibility of such damage.  See 
  * the GNU Lesser General Public License for more details. 
  * 
  * You should have received a copy of the GNU Lesser General Public License 
  * along with this library; if not, write to the Free Software Foundation, 
  * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA. 
  * 
  * The original code contained here was initially developed by: 
  * 
  *     Sri Paladugu 
  *     The Systems Biology Markup Language Development Group 
  *     ERATO Kitano Symbiotic Systems Project 
  *     Keck Graduate Institute of Applied Life Sciences 
  *     535 Watson Drive 
  *     Claremont, CA, 91711, USA 
  * 
  *     http://www.cds.caltech.edu/erato 
  * 	http://www.kgi.edu
  *     mailto:sysbio-team@caltech.edu 
  * Contributor(s): 
  */ 
#include "NomImpl.h"
#ifndef SBMLREADER_H
#include "sbml/SBMLReader.h"
#endif
#ifndef MATHMLDOCUMENT_H
#include "sbml/MathMLDocument.h"
#endif
#ifndef MATHMLREADER_H
#include "sbml/MathMLReader.h"
#endif
#ifndef MATHMLWRITER_H
#include "sbml/MathMLWriter.h"
#endif
#ifndef SBMLTYPES_H
#include "sbml/SBMLTypes.h"
#endif
#ifndef STRING_H
#include <string.h>
#endif

extern SBMLDocument_t* d;
extern Model_t* m;

#define FUNCDATAROWS 44
#define FUNCDATACOLS 16

static const char* functiondata[FUNCDATAROWS][FUNCDATACOLS] = {
{ "massi1", "Irreversible Mass Action Kinetics for 1 substrate", "S", "k", "k * S" },
{ "massi2", "Irreversible Mass Action Kinetics for 2 substrates", "S1", "S2", "k", "k * S1 * S2" },
{ "massi3", "Irreversible Mass Action Kinetics for 3 substrates", "S1", "S2", "S3", "k", "k * S1 * S2 * S3" },
{ "massr11", "Reversible Mass Action Kinetics for 1 substrate and 1 product", "S", "P", "k_1", "k_2",
    "k_1 * S - k_2 * P"},
{ "massr12", "Reversible Mass Action Kinetics for 1 substrate and 2 products", "S", "P1", "P2", "k_1", "k_2",
    "k_1 * S - k_2 * P1 * P2"},
{ "massr13", "Reversible Mass Action Kinetics for 1 substrate and 3 products", "S", "P1", "P2", "P3", "k_1", "k_2",
    "k_1 * S - k_2 * P1 * P2 * P3"},
{ "massr21", "Reversible Mass Action Kinetics for 2 substrates and 1 product", "S1", "S2", "P", "k_1", "k_2",
    "k_1 * S1 * S2 - k_2 * P"},
{ "massr22", "Reversible Mass Action Kinetics for 2 substrates and 2 products", "S1", "S2", "P1", "P2", "k_1", "k_2",
    "k_1 * S1 * S2 - k_2 * P1 * P2"},
{ "massr23", "Reversible Mass Action Kinetics for 2 substrates and 3 products", "S1", "S2", "P1", "P2", "P3", "k_1", "k_2",
    "k_1 * S1 * S2 - k_2 * P1 * P2 * P3"},
{ "massr31", "Reversible Mass Action Kinetics for 3 substrates and 1 product", "S1", "S2", "S3", "P", "k_1", "k_2",
    "k_1 * S1 * S2 * S3 - k_2 * P"},
{ "massr32", "Reversible Mass Action Kinetics for 3 substrates and 2 products", "S1", "S2", "S3", "P1", "P2", "k_1", "k_2",
    "k_1 * S1 * S2 * S3 - k_2 * P1 * P2"},
{ "massr33", "Reversible Mass Action Kinetics for 3 substrates and 3 products", "S1", "S2", "S3", "P1", "P2", "P3", "k_1", "k_2",
    "k_1 * S1 * S2 * S3 - k_2 * P1 * P2 * P3"},
{ "uui", "Irreversible Simple Michaelis-Menten ", "S", "V_m", "K_m", "(V_m * S)/(K_m + S)" },
{ "uur", "Uni-Uni Reversible Simple Michaelis-Menten", "S", "P", "V_f", "V_r", "K_ms", "K_mp",
    "(V_f * S / K_ms - V_r * P / K_mp)/(1 + S / K_ms +  P / K_mp)"},
{ "uuhr", "Uni-Uni Reversible Simple Michaelis-Menten with Haldane adjustment", "S", "P", "V_f", "K_m1", "K_m2", "K_eq",
    "( V_f / K_m1 * (S - P / K_eq ))/(1 + S / K_m1 + P / K_m2)"},
{ "isouur", "Iso Uni-Uni", "S", "P", "V_f", "K_ms", "K_mp", "K_ii", "K_eq",
    "(V_f * (S - P / K_eq ))/(S * (1 + P / K_ii ) + K_ms * (1 + P / K_mp))"},
{ "hilli", "Hill Kinetics", "S", "V", "S_0_5", "h", "(V * pow(S,h))/(pow(S_0_5,h) + pow(S,h))"},
{ "hillr", "Reversible Hill Kinetics", "S", "P", "V_f", "S_0_5", "P_0_5", "h", "K_eq",
"(V_f * (S / S_0_5) * (1 - P / (S * K_eq) ) * pow(S / S_0_5 + P / P_0_5, h-1))/(1 + pow(S / S_0_5 + P / P_0_5, h))"},
{ "hillmr", "Reversible Hill Kinetics with One Modifier", "S", "M", "P", "V_f", "K_eq", "k", "h", "alpha",
"(V_f * (S / S_0_5) * (1 - P / (S * K_eq) ) * pow(S / S_0_5 + P / P_0_5, h-1))/( pow(S / S_0_5 + P / P_0_5, h) + (1 + pow(M / M_0_5, h))/(1 + alpha * pow(M/M_0_5,h)))"},
{ "hillmmr", "Reversible Hill Kinetics with Two Modifiers", "S", "P", "M", "V_f", "K_eq", "k", "h", "a", "b", "alpha_1", "alpha_2", "alpha_12",
"(V_f * (S / S_0_5) * (1 - P / (S * K_eq) ) * pow(S / S_0_5 + P / P_0_5, h-1)) / (pow(S / S_0_5 + P / P_0_5, h) + ((1 + pow(Ma/Ma_0_5,h) + pow(Mb/Mb_0_5,h))/( 1 + alpha_1 * pow(Ma/Ma_0_5,h) + alpha_2 * pow(Mb/Mb_0_5,h) + alpha_1 * alpha_2 * alpha_12 * pow(Ma/Ma_0_5,h) * pow(Mb/Mb_0_5,h))))"},
{ "usii", "Substrate Inhibition Kinetics (Irreversible)", "S", "V", "K_m", "K_i", "V*(S/K_m)/(1 + S/K_m + sqr(S)/K_i)"},
{ "usir", "Substrate Inhibition Kinetics (Reversible)", "S", "P", "V_f", "V_r", "K_ms", "K_mp", "K_i",
  "(V_f*S/K_ms + V_r*P/K_mp)/(1 + S/K_ms + P/K_mp + sqr(S)/K_i)"},
{ "usai", "Substrate Activation", "S", "V", "K_sa", "K_sc", "V * sqr(S/K_sa)/(1 + S/K_sc + sqr(S/K_sa) + S/K_sa)"},
{ "ucii", "Competitive Inhibition (Irreversible)", "S", "V", "K_m", "K_i", "(V * S/K_m)/(1 + S/K_m + I/K_i)"},
{ "ucir", "Competitive Inhibition (Reversible)", "S", "P", "V_f", "V_r", "K_ms", "K_mp", "K_i",
  "(V_f*S/K_ms - V_r*P/K_mp)/(1 + S/K_ms + P/K_mp + I/K_i)"},
{ "unii", "Noncompetitive Inhibition (Irreversible)", "S", "I", "V", "K_m", "K_i", "(V*S/K_m)/(1 + I/K_i + (S/K_m)*(1 + I/K_i))"},
{ "unir", "Noncompetitive Inhibition (Reversible)", "S", "P", "I", "V_f", "K_ms", "K_mp", "K_i",
  "(V_f*S/K_ms - V_r*P/K_mp)/(1 + I/K_i + (S/K_ms + P/K_mp )*(1 + I/K_i))"},
{ "uuci", "Uncompetitive Inhibition (Irreversible)", "S", "I", "V", "K_m", "K_i", "(V*S/K_m)/(1 + (S/K_m)*(1 + I/K_i))"},
{ "uucr", "Uncompetitive Inhibition (Reversible)", "S", "P", "I", "V_f", "V_r", "K_ms", "K_mp", "K_i",
  "(V_f*S/K_ms - V_r*P/K_mp)/(1 + ( S/K_ms + P/K_mp )*( 1 + I/K_i))"},
{ "umi", "Mixed Inhibition Kinetics (Irreversible)", "S", "I", "V", "K_m", "K_is", "K_ic",
  "(V*S/K_m)/(1 + I/K_is + (S/K_m)*(1 + I/K_ic))"},
{ "umr", "Mixed Inhibition Kinetics (Reversible)", "S", "P", "I", "V_f", "V_r", "K_ms", "K_mp", "K_is", "K_ic",
  "(V_f*S/K_ms - V_r*P/K_mp)/(1 + I/K_is + ( S/K_ms + P/K_mp )*( 1 + I/K_ic ))"},
{ "uai", "Specific Activation Kinetics - irreversible", "S", "A_c", "V", "K_m", "K_a", "(V*S/K_m)/(1 + S/K_m + K_a/A_c)"},
{ "uar", "Specific Activation Kinetics (Reversible)", "S", "P", "A_c", "V_f", "V_r", "K_ms", "K_mp", "K_a",
  "(V_f*S/K_ms - V_r*P/K_mp)/(1 + S/K_ms + P/K_mp + K_a/A_c)"},
{ "ucti", "Catalytic Activation (Irreversible)", "S", "A_c", "V", "K_m", "K_a", "(V*S/K_m)/(1 + K_a/A_c + (S/K_m)*(1 + K_a/A_c))"},
{ "uctr", "Catalytic Activation (Reversible)", "S", "P", "A_c", "V_f", "V_r", "K_ms", "K_mp", "K_a",
  "(V_f*S/K_ms - V_r*P/K_mp)/(1 + K_a/A_c + (S/K_ms + P/K_mp)*(1 + K_a/A_c))"},
{ "umai", "Mixed Activation Kinetics (Irreversible)", "S", "A_c", "V", "K_m", "Kas", "Kac",
  "(V*S/K_m)/(1 + Kas/A_c + (S/K_m)*(1 + Kac/A_c))"},
{ "umar", "Mixed Activation Kinetics (Reversible)", "S", "P", "A_c", "V_f", "V_r", "K_ms", "K_mp", "K_as", "K_ac",
  "(V_f*S/K_ms - V_r*P/K_mp)/(1 + K_as/A_c + (S/K_ms + P/K_mp)*(1 + K_ac/A_c))"},
{ "uhmi", "General Hyperbolic Modifier Kinetics (Irreversible)", "S", "M", "V", "K_m", "K_d", "a", "b",
  "(V*(S/K_m)*(1 + b * M / (a*K_d)))/(1 + M/K_d + (S/K_m)*(1 + M/(a*K_d)))"},
{ "uhmr", "General Hyperbolic Modifier Kinetics (Reversible)", "S", "P", "M", "V_f", "V_r", "K_ms", "K_mp", "K_d", "a", "b",
  "((V_f*S/K_ms - V_r*P/K_mp)*(1 + b*M/(a*K_d)))/(1 + M/K_d + (S/K_ms + P/K_mp)*(1 + M/(a*K_d)))"},
{ "ualii", "Allosteric inhibition (Irreversible)", "S", "I", "V", "K_s", "K_ii", "n", "L",
  "(V*pow(1 + S/K_s, n-1))/(L*pow(1 + I/K_ii,n) + pow(1 + S/K_s,n))"},
{ "ordubr", "Ordered Uni Bi Kinetics", "A", "P", "Q", "V_f", "V_r", "K_ma", "K_mq", "K_mp", "K_ip", "K_eq",
  "(V_f*( A - P*Q/K_eq))/(K_ma + A*(1 + P/K_ip) + (V_f/(V_r*K_eq))*(K_mq*P + K_mp*Q + P*Q))"},
{ "ordbur", "Ordered Bi Uni Kinetics", "A", "B", "P", "V_f", "V_r", "K_ma", "Kmb", "K_mp", "K_ia", "K_eq",
  "(V_f*(A*B - P/K_eq))/(A*B + K_ma*B + Kmb*A + (V_f/(V_r*K_eq))*(K_mp + P*(1 + A/K_ia)))"},
{ "ordbbr", "Ordered Bi Bi Kinetics", "A", "B", "P", "Q", "V_f", "K_ma", "K_mb", "K_mp", "K_ia", "K_ib", "K_ip", "K_eq",
  "(V_f*(A*B - P*Q/K_eq))/(A*B*(1 + P/K_ip) + K_mb*(A + K_ia) + K_ma*B + ((V_f / (V_r*K_eq)) * (K_mq*P*( 1 + A/K_ia) + Q*(K_mp*( 1 + (K_ma*B)/(K_ia*K_mb) + P*(1 + B/K_ib))))))"},
{ "ppbr", "Ping Pong Bi Bi Kinetics", "A", "B", "P", "Q", "V_f", "V_r", "K_ma", "K_mb", "K_mp", "K_mq", "K_ia", "K_iq", "K_eq",
  "(V_f*(A*B - P*Q/K_eq))/(A*B + K_mb*A + K_ma*B*(1 + Q/K_iq) + ((V_f/(V_r*K_eq))*(K_mq*P*(1 + A/K_ia) + Q*(K_mp + P))))"}};

void getValueImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
	char* arg;
	int level;
	SBWDouble result;
	ListOf_t* speciesList;
	ListOf_t* compartmentsList;
	ListOf_t* parametersList;
	Species_t* s;
	const char* speciesName;
	Compartment_t* c;
	const char* compartmentName;
	Parameter_t* p;
	const char* parameterName;
	int isExists = 0;
	int i;

	if (m == NULL)
	{
		SBWSetException("You need to load the model first", "Exception raised in method getValueImpl");
        return;
	}

	if(!SBWRead(argObj,"string",&arg))
		return;

	level = SBMLDocument_getLevel(d);

	speciesList = Model_getListOfSpecies(m);
        for(i = 0; i<ListOf_getNumItems(speciesList); i++)
	{
		s = (Species_t *) ListOf_get(speciesList,i);
		if (level == 1)
		{
			speciesName = (char *) Species_getName(s);
		}
		else
		{
			speciesName = (char *) Species_getId(s);
		}
		if(strcmp(speciesName, arg) == 0)
		{
			isExists = 1;
			if(Species_isSetInitialAmount(s) == 1)
			{
                		result = Species_getInitialAmount(s);
			}
			else
			{
				result = Species_getInitialConcentration(s);
			}
			break;
		}
	}

	if(isExists == 0)
	{
		compartmentsList = Model_getListOfCompartments(m);
		for(i = 0; i<ListOf_getNumItems(compartmentsList); i++)
		{
			c = (Compartment_t *) ListOf_get(compartmentsList,i);
			if (level == 1)
			{
				compartmentName = (char* ) Compartment_getName(c);
			}
			else
			{
				compartmentName = (char* ) Compartment_getId(c);
			}
			if(strcmp(compartmentName, arg) == 0)
			{
				isExists = 1;
				result = Compartment_getVolume(c);
				break;
			}
		}
	}

	if(isExists == 0)
	{
		parametersList = Model_getListOfParameters(m);
		for(i = 0; i<ListOf_getNumItems(parametersList); i++)
		{
			p = (Parameter_t* ) ListOf_get(parametersList,i);
			if (level == 1)
			{
				parameterName = (char* ) Parameter_getName(p);
			}
			else
			{
				parameterName = (char *) Parameter_getId(p);
			}
			if(strcmp(parameterName, arg) == 0)
			{
				isExists = 1;
				result = Parameter_getValue(p);
				break;
			}
		}
	}

	if(isExists == 1)
	{
	  SBWWrite(resultObj,"double",result);
	}
	else
	{
		SBWSetException("Invalid string name. The name does not exist in the model", "Exception raised in method getValueImpl");
        return;	  
	}
}

void existsImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
	char* arg;
	int level;
	SBWBoolean result;
	ListOf_t* speciesList;
	const char* speciesName;
	ListOf_t* compartmentsList;
	const char* compartmentName;
	ListOf_t* parametersList;
	const char* parameterName;
	Species_t* s;
	Compartment_t* c;
	Parameter_t* p;
	int isExists = 0;
	int i;

	if (m == NULL)
	{
		SBWSetException("You need to load the model first", "Exception raised in method existsImpl");
        return;
	}
	if(!SBWRead(argObj,"string",&arg))
		return;

	level = SBMLDocument_getLevel(d);

	speciesList = Model_getListOfSpecies(m);
    for(i = 0; i<ListOf_getNumItems(speciesList); i++)
	{
		s = (Species_t *) ListOf_get(speciesList,i);
		if (level == 1)
		{
			speciesName = (char* ) Species_getName(s);
		}
		else
		{
			speciesName = (char* ) Species_getId(s);
		}
		if(strcmp(speciesName, arg) == 0)
		{
			isExists = 1;
			break;
		}
	}

	if(isExists == 0)
	{
		compartmentsList = Model_getListOfCompartments(m);
		for(i = 0; i<ListOf_getNumItems(compartmentsList); i++)
		{
			c = (Compartment_t *) ListOf_get(compartmentsList,i);
			if (level == 1)
			{
				compartmentName = (char* ) Compartment_getName(c);
			}
			else
			{
				compartmentName = (char* ) Compartment_getId(c);
			}
			if(strcmp(compartmentName, arg) == 0)
			{
				isExists = 1;
				break;
			}
		}
	}

	if(isExists == 0)
	{
		parametersList = Model_getListOfParameters(m);
		for(i = 0; i<ListOf_getNumItems(parametersList); i++)
		{
			p = (Parameter_t* ) ListOf_get(parametersList,i);
			if (level == 1)
			{
				parameterName = (char* ) Parameter_getName(p);
			}
			else
			{
				parameterName = (char* ) Parameter_getId(p);
			}
			if(strcmp(parameterName, arg) == 0)
			{
				isExists = 1;
				break;
			}
		}
	}

	if(isExists == 1)
	{
	  result = 1;
	  SBWWrite(resultObj,"boolean",result);
	}
	else
	{
	  result = 0;
	  SBWWrite(resultObj,"boolean",result);
	}
}

void loadSBMLImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
	const char* arg;
	if(!SBWRead(argObj,"string",&arg))
		return;
	if (strcmp(arg,"") == 0)
	{
		SBWSetException("The input string cannot be blank", "Exception raised in method loadSBMLImpl");
        	return;
	}
	d = readSBMLFromString(arg);
	m = SBMLDocument_getModel(d);
	if (m == NULL)
	{
		SBWSetException("Error in sbml input. ", "Exception raised in method loadSBMLImpl");
        	return;
	}
}

void getSBMLImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
	char* result;
	if (m == NULL)
	{
		SBWSetException("You need to load the model first", "Exception raised in method getSBMLImpl");
        return;
	}
	result = writeSBMLToString(d);
	SBWWrite(resultObj,"string",result);
}

void getModelNameImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{	
	char* result;
	SBWInteger level;
	if (m == NULL)
	{
		SBWSetException("You need to load the model first", "Exception raised in method getModelNameImpl");
        return;
	}

	level = SBMLDocument_getLevel(d);
	if (level == 1)
	{
		result = (char* ) Model_getName(m);
	}
	else
	{
		result = (char* ) Model_getId(m);
	}
	SBWWrite(resultObj,"string",result);
}

void getNumCompartmentsImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
	SBWInteger result;
	if (m == NULL)
	{
		SBWSetException("You need to load the model first", "Exception raised in method getNumCompartmentsImpl");
        return;
	}
	result = Model_getNumCompartments(m);
	SBWWrite(resultObj,"int",result);
}

void getNumReactionsImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
	SBWInteger result;
	if (m == NULL)
	{
		SBWSetException("You need to load the model first", "Exception raised in method getNumReactionsImpl");
        return;
	}
	result = Model_getNumReactions(m);
	SBWWrite(resultObj,"int",result);
}

void getNumFloatingSpeciesImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
	SBWInteger result;
	SBWInteger numSpecies;
	SBWInteger numBoundarySpecies;
	if (m == NULL)
	{
		SBWSetException("You need to load the model first", "Exception raised in method getNumFloatingSpeciesImpl");
        return;
	}
	numSpecies = Model_getNumSpecies(m);
	numBoundarySpecies = Model_getNumSpeciesWithBoundaryCondition(m);
	result = numSpecies - numBoundarySpecies;
	SBWWrite(resultObj,"int",result);
}

void getNumBoundarySpeciesImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
	SBWInteger result;
	if (m == NULL)
	{
		SBWSetException("You need to load the model first", "Exception raised in method getNumBoundarySpeciesImpl");
        return;
	}
	result = Model_getNumSpeciesWithBoundaryCondition(m);
	SBWWrite(resultObj,"int",result);
}

void getNumGlobalParametersImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
	SBWInteger result;
	if (m == NULL)
	{
		SBWSetException("You need to load the model first", "Exception raised in method getNumGlobalParametersImpl");
        return;
	}
	result = Model_getNumParameters(m);
	SBWWrite(resultObj,"int",result);
}

void getNthCompartmentNameImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
	SBWInteger arg;
	char* result;
	int level;
	Compartment_t* c;
	if(m == NULL)
	{
		SBWSetException("You need to load the model first", "Exception raised in method getNthCompartmentNameImpl");
        return;
	}
	if(!SBWRead(argObj,"int",&arg))
		return;
	if(arg < 0 || arg >= Model_getNumCompartments(m))
	{
		SBWSetException("Invalid input - Argument should be >= 0 and should be less than total number of compartments in the model", "Exception raised in method getNthCompartmentNameImpl");
        return;
	}

	c = Model_getCompartment(m, arg);
	level = SBMLDocument_getLevel(d);
	
	if (level == 1)
	{
		result = (char* ) Compartment_getName(c);
	}
	else
	{
		result = (char* ) Compartment_getId(c);
	}

	SBWWrite(resultObj,"string",result);
}

void getNthFloatingSpeciesNameImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
	SBWInteger arg;
	char* result;
	int i;
	int level;
	SBWInteger numOfFltSpecies = 0;
	int isFndNthFltSpecies = 0;
	ListOf_t* speciesList;
	Species_t* s;
	if(m == NULL)
	{
		SBWSetException("You need to load the model first", "Exception raised in method getNthFloatingSpeciesNameImpl");
        return;
	}
	if(!SBWRead(argObj,"int",&arg))
		return;
    speciesList = Model_getListOfSpecies(m);
	for(i=0; i<Model_getNumSpecies(m); i++)
	{
		s = (Species_t* ) ListOf_get(speciesList, i);
		if(Species_getBoundaryCondition(s) == 0)
		{
			if(numOfFltSpecies == arg)
			{
				isFndNthFltSpecies = 1;
				break;
			}
			else
			{
				++numOfFltSpecies;
			}
		}
	}
	if(isFndNthFltSpecies == 1)
	{
		level = SBMLDocument_getLevel(d);
	
		if (level == 1)
		{
			result = (char* ) Species_getName(s);
		}
		else
		{
			result = (char* ) Species_getId(s);
		}
	}
	else
	{
		SBWSetException("The model does not have a floating species corresponding to the index provided", "Exception raised in method getNthFloatingSpeciesNameImpl");
		return;
	}
	SBWWrite(resultObj,"string",result);
}

void getNthBoundarySpeciesNameImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
	SBWInteger arg;
	char* result;
	int i;
	int level;
	SBWInteger numOfBdySpecies = 0;
	int isFndNthBdySpecies = 0;
	ListOf_t* speciesList;
	Species_t* s;

	if(m == NULL)
	{
		SBWSetException("You need to load the model first", "Exception raised in method getNthBoundarySpeciesNameImpl");
        return;
	}

	if(!SBWRead(argObj,"int",&arg))
		return;

    speciesList = Model_getListOfSpecies(m);
	for(i=0; i<Model_getNumSpecies(m); i++)
	{
		s = (Species_t* ) ListOf_get(speciesList, i);
		if(Species_getBoundaryCondition(s) == 1)
		{
			if(numOfBdySpecies == arg)
			{
				isFndNthBdySpecies = 1;
				break;
			}
			else
			{
				++numOfBdySpecies;
			}
		}
	}

	if(isFndNthBdySpecies == 1)
	{
		level = SBMLDocument_getLevel(d);
	
		if (level == 1)
		{
			result = (char* ) Species_getName(s);
		}
		else
		{
			result = (char* ) Species_getId(s);
		}
	}
	else
	{
		SBWSetException("The model does not have a boundary species corresponding to the index provided", "Exception raised in method getNthBoundarySpeciesNameImpl");
		return;
	}
	SBWWrite(resultObj,"string",result);
}

void getNthFloatingSpeciesCompartmentNameImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
	SBWInteger arg;
	char* result;
	int i;
	SBWInteger numOfFltSpecies = 0;
	int isFndNthFltSpecies = 0;
	ListOf_t* speciesList;
	Species_t* s;
	if(m == NULL)
	{
		SBWSetException("You need to load the model first", "Exception raised in method getNthFloatingSpeciesCompartmentNameImpl");
        return;
	}

	if(!SBWRead(argObj,"int",&arg))
		return;

    speciesList = Model_getListOfSpecies(m);
	for(i=0; i<Model_getNumSpecies(m); i++)
	{
		s = (Species_t* ) ListOf_get(speciesList, i);
		if(Species_getBoundaryCondition(s) == 0)
		{
			if(numOfFltSpecies == arg)
			{
				isFndNthFltSpecies = 1;
				break;
			}
			else
			{
				++numOfFltSpecies;
			}
		}

	}

	if(isFndNthFltSpecies == 1)
	{
		result = (char* ) Species_getCompartment(s);
	}
	else
	{
		SBWSetException("The model does not have a floating species corresponding to the index provided", "Exception raised in method getNthFloatingSpeciesCompartmentNameImpl");
		return;
	}
	SBWWrite(resultObj,"string",result);
}

void getNthBoundarySpeciesCompartmentNameImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
	SBWInteger arg;
	char* result;
	int i;
	SBWInteger numOfBdySpecies = 0;
	int isFndNthBdySpecies = 0;
	ListOf_t* speciesList;
	Species_t* s;
	
	if(m == NULL)
	{
		SBWSetException("You need to load the model first", "Exception raised in method getNthBoundarySpeciesCompartmentNameImpl");
        return;
	}

	if(!SBWRead(argObj,"int",&arg))
		return;

    speciesList = Model_getListOfSpecies(m);
	for(i=0; i < Model_getNumSpecies(m); i++)
	{
		s = (Species_t* ) ListOf_get(speciesList, i);
		if(Species_getBoundaryCondition(s) == 1)
		{
			if(numOfBdySpecies == arg)
			{
				isFndNthBdySpecies = 1;
				break;
			}
			else
			{
				++numOfBdySpecies;
			}
		}
	}

	if(isFndNthBdySpecies == 1)
	{
		result = (char* ) Species_getCompartment(s);
	}
	else
	{
		SBWSetException("The model does not have a boundary species corresponding to the index provided", "Exception raised in method getNthBoundarySpeciesCompartmentNameImpl");
		return;
	}
	SBWWrite(resultObj,"string",result);
}

void isReactionReversibleImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
	SBWInteger arg;
	SBWBoolean result;
	Reaction_t* r;

	if(m == NULL)
	{
		SBWSetException("You need to load the model first", "Exception raised in method isReactionReversibleImpl");
        return;
	}

	if(!SBWRead(argObj,"int",&arg))
		return;

	if(arg >= Model_getNumReactions(m))
	{
		SBWSetException("There is no reaction corresponding to the index you provided", "Exception raised in method isReactionReversibleImpl");
        return;
	}

	r = Model_getReaction(m,arg);
	result = Reaction_getReversible(r);
	SBWWrite(resultObj,"boolean",result);
}

void getNthReactionNameImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
	SBWInteger arg;
	char* result;
	int level;
	Reaction_t* r;

	if(m == NULL)
	{
		SBWSetException("You need to load the model first", "Exception raised in method getNthReactionNameImpl");
        return;
	}

	if(!SBWRead(argObj,"int",&arg))
		return;

	if(arg >= Model_getNumReactions(m))
	{
		SBWSetException("There is no reaction corresponding to the index you provided", "Exception raised in method getNthReactionNameImpl");
        return;
	}

	r = Model_getReaction(m,arg);
	level = SBMLDocument_getLevel(d);
	
	if (level == 1)
	{
		result = (char* ) Reaction_getName(r);
	}
	else
	{
		result = (char* ) Reaction_getId(r);
	}
	
	SBWWrite(resultObj,"string",result);
}

void getNumReactantsImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
	SBWInteger arg;
	SBWInteger result;
	Reaction_t* r;

	if(m == NULL)
	{
		SBWSetException("You need to load the model first", "Exception raised in method getNumReactantsImpl");
        return;
	}

	if(!SBWRead(argObj,"int",&arg))
		return;
	r = Model_getReaction(m,arg);
	result = Reaction_getNumReactants(r);
	SBWWrite(resultObj,"int",result);
}

void getNumProductsImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
	SBWInteger arg;
	SBWInteger result;
	Reaction_t* r;
	
	if(m == NULL)
	{
		SBWSetException("You need to load the model first", "Exception raised in method getNumProductsImpl");
        return;
	}

	if(!SBWRead(argObj,"int",&arg))
		return;
	r = Model_getReaction(m,arg);
	result = Reaction_getNumProducts(r);
	SBWWrite(resultObj,"int",result);
}

void getNthReactantNameImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
	SBWInteger arg1;
	SBWInteger arg2;
	char* result;
	ListOf_t* reactantsList;
	SpeciesReference_t* reactant;
	Reaction_t* r;
	
	if(m == NULL)
	{
		SBWSetException("You need to load the model first", "Exception raised in method getNthReactantNameImpl");
        return;
	}
	
	if(!SBWRead(argObj,"int,int",&arg1,&arg2))
		return;
	
	if(arg1 >= Model_getNumReactions(m))
	{
		SBWSetException("There is no reaction corresponding to the index you provided", "Exception raised in method getNthReactantNameImpl");
        return;
	}

	r = Model_getReaction(m,arg1);
	reactantsList = Reaction_getListOfReactants(r);
		
	if(arg2 >= ListOf_getNumItems(reactantsList))
	{
		SBWSetException("Index exceeds the number of reactants in the list", "Exception raised in method getNthReactantNameImpl");
        return;
	}

	reactant = (SpeciesReference_t* ) ListOf_get(reactantsList,arg2);
	result = (char* ) SpeciesReference_getSpecies(reactant);
	SBWWrite(resultObj,"string",result);
}

void getNthProductNameImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
	SBWInteger arg1;
	SBWInteger arg2;
	char* result;
	ListOf_t* productsList;
	SpeciesReference_t* product;
	Reaction_t* r;

	if(m == NULL)
	{
		SBWSetException("You need to load the model first", "Exception raised in method getNthProductNameImpl");
        return;
	}

	if(!SBWRead(argObj,"int,int",&arg1,&arg2))
		return;
		
	if(arg1 >= Model_getNumReactions(m))
	{
		SBWSetException("There is no reaction corresponding to the index you provided", "Exception raised in method getNthProductNameImpl");
        return;
	}

	r = Model_getReaction(m,arg1);
	productsList = Reaction_getListOfProducts(r);
		
	if(arg2 >= ListOf_getNumItems(productsList))
	{
		SBWSetException("Index exceeds the number of products in the list", "Exception raised in method getNthProductNameImpl");
        return;
	}

	product = (SpeciesReference_t* ) ListOf_get(productsList,arg2);
	result = (char* ) SpeciesReference_getSpecies(product);
	SBWWrite(resultObj,"string",result);
}

void getKineticLawImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
	SBWInteger arg;
	char* result;
	Reaction_t* r;
	KineticLaw_t* kl;

	if(m == NULL)
	{
		SBWSetException("You need to load the model first", "Exception raised in method getKineticLawImpl");
        return;
	}

	if(!SBWRead(argObj,"int",&arg))
		return;
			
	if(arg >= Model_getNumReactions(m))
	{
		SBWSetException("There is no reaction corresponding to the index you provided", "Exception raised in method getKineticLawImpl");
        return;
	}

	r = Model_getReaction(m,arg);
	kl = Reaction_getKineticLaw(r);
	result = (char* ) KineticLaw_getFormula(kl);
	SBWWrite(resultObj,"string",result); 
}

void getNthReactantStoichiometryImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
	SBWInteger arg1;
	SBWInteger arg2;
	SBWInteger result;
	ListOf_t* reactantsList;
	SpeciesReference_t* reactant;
	Reaction_t* r;

	if(m == NULL)
	{
		SBWSetException("You need to load the model first", "Exception raised in method getNthReactantStoichiometryImpl");
        return;
	}

	if(!SBWRead(argObj,"int,int",&arg1,&arg2))
		return;

	if(arg1 >= Model_getNumReactions(m))
	{
		SBWSetException("There is no reaction corresponding to the index you provided", "Exception raised in method getNthReactantStoichiometryImpl");
        return;
	}

	r = Model_getReaction(m,arg1);
	reactantsList = Reaction_getListOfReactants(r);

	if(arg2 >= ListOf_getNumItems(reactantsList))
	{
		SBWSetException("Index exceeds the number of reactants in the list", "Exception raised in method getNthReactantStoichiometryImpl");
        return;
	}

	reactant = (SpeciesReference_t* ) ListOf_get(reactantsList,arg2);//Is this right I am assuming the list contents are of type SpeciesReference
	result = (SBWInteger) SpeciesReference_getStoichiometry(reactant);
	SBWWrite(resultObj,"int",result);
}

void getNthProductStoichiometryImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
	SBWInteger arg1;
	SBWInteger arg2;
	SBWInteger result;
	ListOf_t* productsList;
	SpeciesReference_t* product;
	Reaction_t* r;

	if(m == NULL)
	{
		SBWSetException("You need to load the model first", "Exception raised in method getNthProductStoichiometryImpl");
        return;
	}

	if(!SBWRead(argObj,"int,int",&arg1,&arg2))
		return;
	
	if(arg1 >= Model_getNumReactions(m))
	{
		SBWSetException("There is no reaction corresponding to the index you provided", "Exception raised in method getNthProductStoichiometryImpl");
        return;
	}

	r = Model_getReaction(m,arg1);
	productsList = Reaction_getListOfProducts(r);
	
	if(arg2 >= ListOf_getNumItems(productsList))
	{
		SBWSetException("Index exceeds the number of products in the list", "Exception raised in method getNthProductStoichiometryImpl");
        return;
	}
	
	product = (SpeciesReference_t* ) ListOf_get(productsList,arg2);
	result = (SBWInteger) SpeciesReference_getStoichiometry(product);
	SBWWrite(resultObj,"int",result);
}

void getNumParametersImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
	SBWInteger arg;
	SBWInteger result;
	Reaction_t* r;
	KineticLaw_t* kl;

	if(m == NULL)
	{
		SBWSetException("You need to load the model first", "Exception raised in method getNumParametersImpl");
        return;
	}

	if(!SBWRead(argObj,"int",&arg))
		return;

	if(arg >= Model_getNumReactions(m))
	{
		SBWSetException("There is no reaction corresponding to the index you provided", "Exception raised in method getNumParametersImpl");
        return;
	}

	r = Model_getReaction(m,arg);
	kl = Reaction_getKineticLaw(r);
	result = KineticLaw_getNumParameters(kl);
	SBWWrite(resultObj,"int",result);
}

void getNthParameterNameImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
	SBWInteger arg1;
	SBWInteger arg2;
	char* result;
	int level;
	Reaction_t* r;
	KineticLaw_t* kl;
	ListOf_t* parametersList;
	Parameter_t* parameter;

	if(m == NULL)
	{
		SBWSetException("You need to load the model first", "Exception raised in method getNthParameterNameImpl");
        return;
	}

	if(!SBWRead(argObj,"int,int",&arg1,&arg2))
		return;
	
	if(arg1 >= Model_getNumReactions(m))
	{
		SBWSetException("There is no reaction corresponding to the index you provided", "Exception raised in method getNthParameterNameImpl");
        return;
	}

	r = Model_getReaction(m,arg1);
	kl = Reaction_getKineticLaw(r);
	parametersList = KineticLaw_getListOfParameters(kl);
	
	if(arg2 >= ListOf_getNumItems(parametersList))
	{
		SBWSetException("Index exceeds the number of parameters in the list", "Exception raised in method getNthParameterNameImpl");
        return;
	}
	
	parameter = (Parameter_t *) ListOf_get(parametersList, arg2);
	level = SBMLDocument_getLevel(d);

	if (level == 1)
	{
		result = (char* ) Parameter_getName(parameter);
	}
	else
	{
		result = (char* ) Parameter_getId(parameter);
	}
	SBWWrite(resultObj,"string",result);
}

void getNthParameterValueImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
	SBWInteger arg1;
	SBWInteger arg2;
	SBWDouble result;
	Reaction_t* r;
	KineticLaw_t* kl;
	ListOf_t* parametersList;
	Parameter_t* parameter;

	if(m == NULL)
	{
		SBWSetException("You need to load the model first", "Exception raised in method getNthParameterValueImpl");
        return;
	}

	if(!SBWRead(argObj,"int,int",&arg1,&arg2))
		return;
	
	if(arg1 >= Model_getNumReactions(m))
	{
		SBWSetException("There is no reaction corresponding to the index you provided", "Exception raised in method getNthParameterValueImpl");
        return;
	}

	r = Model_getReaction(m,arg1);
	kl = Reaction_getKineticLaw(r);
	parametersList = KineticLaw_getListOfParameters(kl);
	
	if(arg2 >= ListOf_getNumItems(parametersList))
	{
		SBWSetException("Index exceeds the number of products in the list", "Exception raised in method getNthParameterValueImpl");
        return;
	}

	parameter = (Parameter_t *) ListOf_get(parametersList, arg2);
	result = Parameter_getValue(parameter);
	SBWWrite(resultObj,"double",result);
}

void getNthParameterHasValueImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
	SBWInteger arg1;
	SBWInteger arg2;
	SBWBoolean result;
	Reaction_t* r;
	KineticLaw_t* kl;
	ListOf_t* parametersList;
	Parameter_t* parameter;
	
	if(m == NULL)
	{
		SBWSetException("You need to load the model first", "Exception raised in method getNthParameterHasValueImpl");
        return;
	}
	
	if(!SBWRead(argObj,"int,int",&arg1,&arg2))
		return;

	if(arg1 >= Model_getNumReactions(m))
	{
		SBWSetException("There is no reaction corresponding to the index you provided", "Exception raised in method getNthParameterHasValueImpl");
        return;
	}

	r = Model_getReaction(m,arg1);
	kl = Reaction_getKineticLaw(r);
	parametersList = KineticLaw_getListOfParameters(kl);

	if(arg2 >= ListOf_getNumItems(parametersList))
	{
		SBWSetException("Index exceeds the number of products in the list", "Exception raised in method getNthParameterHasValueImpl");
        return;
	}

	parameter = (Parameter_t *) ListOf_get(parametersList, arg2);
	result = Parameter_isSetValue(parameter);
	SBWWrite(resultObj,"boolean",result);
}

void getNthGlobalParameterNameImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
	SBWInteger arg;
	char* result;
	int level;
	Parameter_t* parameter;
	
	if(m == NULL)
	{
		SBWSetException("You need to load the model first", "Exception raised in method getNthGlobalParameterNameImpl");
        return;
	}
	
	if(!SBWRead(argObj,"int",&arg))
		return;

	if(arg >= Model_getNumParameters(m))
	{
		SBWSetException("Index exceeds the number of Global Parameters in the model", "Exception raised in method getNthGlobalParameterNameImpl");
        return;
	}

	parameter = (Parameter_t *) Model_getParameter(m, arg);
	level = SBMLDocument_getLevel(d);
	
	if (level == 1)
	{
		result = (char* ) Parameter_getName(parameter);
	}
	else
	{
		result = (char* ) Parameter_getId(parameter);
	}
	SBWWrite(resultObj,"string",result);
}

void hasValueImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
	char* arg;
	int level;
	SBWBoolean result = 0;
	ListOf_t* speciesList;
	char* speciesName;
	ListOf_t* compartmentsList;
	char* compartmentName;
	ListOf_t* parametersList;
	char* parameterName;
	Species_t* s;
	Compartment_t* c;
	Parameter_t* p;
	int isExists = 0;
	int i;

	if (m == NULL)
	{
		SBWSetException("You need to load the model first", "Exception raised in method hasValueImpl");
        return;
	}

	if(!SBWRead(argObj,"string",&arg))
		return;

	level = SBMLDocument_getLevel(d);

	speciesList = Model_getListOfSpecies(m);
    for(i = 0; i<ListOf_getNumItems(speciesList); i++)
	{
		s = (Species_t *) ListOf_get(speciesList,i);
		if (level == 1)
		{
			speciesName = (char* ) Species_getName(s);
		}
		else
		{
			speciesName = (char* ) Species_getId(s);
		}
		if(strcmp(speciesName, arg) == 0)
		{
			isExists = 1;
			if(Species_isSetInitialAmount(s) == 1)
			{
                result = 1;
			}
			if(Species_isSetInitialConcentration(s) == 1)
			{
				result = 1;
			}
			break;
		}
	}

	if(isExists == 0)
	{
		compartmentsList = Model_getListOfCompartments(m);
		for(i = 0; i<ListOf_getNumItems(compartmentsList); i++)
		{
			c = (Compartment_t *) ListOf_get(compartmentsList,i);
			if (level == 1)
			{
				compartmentName = (char* ) Compartment_getName(c);
			}
			else
			{
				compartmentName = (char* ) Compartment_getId(c);
			}
			if(strcmp(compartmentName, arg) == 0)
			{
				isExists = 1;
				if(Compartment_isSetVolume(c) == 1)
				{
					result = 1;
				}
				break;
			}
		}
	}

	if(isExists == 0)
	{
		parametersList = Model_getListOfParameters(m);
		for(i = 0; i<ListOf_getNumItems(parametersList); i++)
		{
			p = (Parameter_t* ) ListOf_get(parametersList,i);
			if (level == 1)
			{
				parameterName = (char* ) Parameter_getName(p);
			}
			else
			{
				parameterName = (char* ) Parameter_getId(p);
			}
			if(strcmp(parameterName, arg) == 0)
			{
				isExists = 1;
				if(Parameter_isSetValue(p) == 1)
				{
					result = 1;
				}
				break;
			}
		}
	}

	if(isExists == 1)
	{
		SBWWrite(resultObj,"boolean",result);
	}
	else
	{
	  	SBWSetException("Invalid string name. The name does not exist in the model", "Exception raised in method hasValueImpl");
        return;	
	}
}

void getBuiltinFunctionInfoImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
	char* arg;
	char* result[FUNCDATAROWS];
	int funcFound = 0;
	int i = 0;
	int j = 0;

	if(!SBWRead(argObj,"string",&arg))
		return;
	while(i < FUNCDATAROWS)
	{
		if(strcmp(arg,functiondata[i][0]) == 0)
		{
			funcFound = 1;
			break;
		}
		else
		{
			i++;
		}
	}

	if(funcFound)
	{
		while(functiondata[i][j] != NULL)
		{
            result[j] = (char* ) functiondata[j];
			j++;
		}
	}
	else
	{
		SBWSetException("Invalid string name. There is no inbuilt function with that name", "Exception raised in method getBuiltinFunctionInfoImpl");
        return;
	}
    
	SBWWrite(resultObj,"int",result);
}

void getBuiltinFunctionsImpl(SBWInteger from,SBWDataBlockReader *argObj,SBWDataBlockWriter *resultObj,void *usrData)
{
    char* result[FUNCDATAROWS];
	int i;

	for(i = 0; i < FUNCDATAROWS; i++)
	{
		result[i] = (char* ) functiondata[i][0];
	}
	SBWWrite(resultObj,"int",result);
}

/*
 *Converts MathML to String
 */
char* convertMathMLToString (const char *xml )
{
	MathMLDocument_t 	*mdoc;
	const ASTNode_t  	*mtree_root;
	char			*mchar;
	
	mdoc 		= readMathMLFromString (xml);
	mtree_root 	= (ASTNode_t *) MathMLDocument_getMath (mdoc);
	mchar 		= SBML_formulaToString (mtree_root);
	
	return mchar;
}

void convertMathMLToStringImpl ( SBWInteger from , SBWDataBlockReader *argObject, SBWDataBlockWriter *resultObject, void *userData)

{	
	const char* arg;
	const char* result;
	if (!SBWRead (argObject, "string", &arg))
		return;
	result = convertMathMLToString (arg);
	SBWWrite (resultObject, "string", result);
}

/*
 *Converts String to MathML
 */
char* convertStringToMathML ( const char *str)
{
	MathMLDocument_t	*mdoc;
	ASTNode_t		*mtree_root;
	char			*mathml;

	mdoc 		= MathMLDocument_create ();
	mtree_root	= (ASTNode_t *) SBML_parseFormula (str);
	MathMLDocument_setMath(mdoc,mtree_root);
	mathml		= writeMathMLToString (mdoc);
	
	return mathml;
}

void convertStringToMathMLImpl ( SBWInteger from , SBWDataBlockReader *argObject, SBWDataBlockWriter *resultObject, void *userData)

{	
	const char* arg;
	const char* result;
	if (!SBWRead (argObject, "string", &arg))
		return;
	result = convertStringToMathML (arg);
	SBWWrite (resultObject, "string", result);
}

int addNomImpl()
{
	if(!SBWModuleImplAddService("NOM","wrapper to NOM","NOM","NOM"))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",getValueImpl,0,"double getValue(string)",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",existsImpl,0,"boolean exists(string)",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",loadSBMLImpl,0,"void loadSBML(string)",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",getSBMLImpl,0,"string getSBML()",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",getModelNameImpl,0,"string getModelName()",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",getNumCompartmentsImpl,0,"int getNumCompartments()",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",getNumReactionsImpl,0,"int getNumReactions()",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",getNumFloatingSpeciesImpl,0,"int getNumFloatingSpecies()",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",getNumBoundarySpeciesImpl,0,"int getNumBoundarySpecies()",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",getNumGlobalParametersImpl,0,"int getNumGlobalParameters()",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",getNthCompartmentNameImpl,0,"string getNthCompartmentName(int)",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",getNthFloatingSpeciesNameImpl,0,"string getNthFloatingSpeciesName(int)",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",getNthBoundarySpeciesNameImpl,0,"string getNthBoundarySpeciesName(int)",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",getNthFloatingSpeciesCompartmentNameImpl,0,"string getNthFloatingSpeciesCompartmentName(int)",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",getNthBoundarySpeciesCompartmentNameImpl,0,"string getNthBoundarySpeciesCompartmentName(int)",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",isReactionReversibleImpl,0,"boolean isReactionReversible(int)",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",getNthReactionNameImpl,0,"string getNthReactionName(int)",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",getNumReactantsImpl,0,"int getNumReactants(int)",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",getNumProductsImpl,0,"int getNumProducts(int)",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",getNthReactantNameImpl,0,"string getNthReactantName(int,int)",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",getNthProductNameImpl,0,"string getNthProductName(int,int)",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",getKineticLawImpl,0,"string getKineticLaw(int)",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",getNthReactantStoichiometryImpl,0,"int getNthReactantStoichiometry(int,int)",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",getNthProductStoichiometryImpl,0,"int getNthProductStoichiometry(int,int)",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",getNumParametersImpl,0,"int getNumParameters(int)",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",getNthParameterNameImpl,0,"string getNthParameterName(int,int)",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",getNthParameterValueImpl,0,"double getNthParameterValue(int,int)",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",getNthParameterHasValueImpl,0,"boolean getNthParameterHasValue(int,int)",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",getNthGlobalParameterNameImpl,0,"string getNthGlobalParameterName(int)",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",hasValueImpl,0,"boolean hasValue(string)",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",getBuiltinFunctionInfoImpl,0,"int getBuiltinFunctionInfo(string)",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",getBuiltinFunctionsImpl,0,"int getBuiltinFunctions()",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",convertMathMLToStringImpl,0,"string convertMathMLToString(string)",0,""))
		return 0;
	else
	if(!SBWModuleImplSetHandler("NOM",convertStringToMathMLImpl,0,"string convertStringToMathML(string)",0,""))
		return 0;
	else
	{
		SBWSetException("Method Implementation not found", "Exception raised in NomImpl");
        return 1;
	}
        
}

