/** 
  * Filename    : NOMModule.c 
  * Description : SBW interface to NOM
  * Author(s)   : Sri Paladugu <spaladug@kgi.edu> 
  * Author(s)   : Herbert Sauro <Herbert_Sauro@kgi.edu> 
  * Organization: Keck Graduate Institute (KGI) 
  * Created     : 
  * Revision    : $Id: NOMModule.c,v 1.1 2004/09/17 00:35:34 zucker Exp $ 
  * Source      : $Source: /var/lib/cvs/Biospice/FluxorPipeline/Metatool/src/SBW-NOM/NOMModule/src/NOMModule.c,v $ 
  * 
  * Copyright 2004 Keck Graduate Institute 
  * 
  * This library is free software; you can redistribute it and/or modify it 
  * under the terms of the GNU Lesser General Public License as published 
  * by the Free Software Foundation; either version 2.1 of the License, or 
  * any later version. 
  * 
  * This library is distributed in the hope that it will be useful, but 
  * WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF 
  * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  The software and 
  * documentation provided hereunder is on an "as is" basis, and the 
  * California Institute of Technology and Japan Science and Technology 
  * Corporation have no obligations to provide maintenance, support, 
  * updates, enhancements or modifications.  In no event shall the 
  * California Institute of Technology or the Japan Science and Technology 
  * Corporation be liable to any party for direct, indirect, special, 
  * incidental or consequential damages, including lost profits, arising 
  * out of the use of this software and its documentation, even if the 
  * California Institute of Technology and/or Japan Science and Technology 
  * Corporation have been advised of the possibility of such damage.  See 
  * the GNU Lesser General Public License for more details. 
  * 
  * You should have received a copy of the GNU Lesser General Public License 
  * along with this library; if not, write to the Free Software Foundation, 
  * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA. 
  * 
  * The original code contained here was initially developed by: 
  * 
  *     Sri Paladugu 
  *     The Systems Biology Markup Language Development Group 
  *     ERATO Kitano Symbiotic Systems Project 
  *     Keck Graduate Institute of Applied Life Sciences 
  *     535 Watson Drive 
  *     Claremont, CA, 91711, USA 
  * 
  *     http://www.cds.caltech.edu/erato 
  * 	http://www.kgi.edu
  *     mailto:sysbio-team@caltech.edu 
  * Contributor(s): 
  */
#ifdef WIN32
#include "windows.h"
#endif
#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "SBWC.h"

#include "NomImpl.h"
#include "NOMClipboardImpl.h"

int cserver(int argc, char* argv[])
{
	int i;
	int isUnique = 0;
	int isRegister = 0;

	/*Parse the command line arguments to check whether the executable is
	  invoked in -sbwmodule mode or -sbwregister mode at the same it also
	  checks whether the module instance should be "unique" or "selfmanaged".
	  */
	for(i=0; i<argc; i++)
	{
		if( strcmp(argv[i], "-unique")==0 )
		{
			isUnique = 1;
		}
		if( strcmp(argv[i], "-sbwregister")==0)
		{
			isRegister = 1;
		}
	}

	/*If the module instance is unique, then it starts NomClipboard otherwise
	  it starts NOM.
	  */
	if(isUnique)
	{
		if(!SBWCreateModuleImpl("edu.caltech.NOMClipboard","SBW-NOM Bridge C Implementation",SBW_UniqueModule,"provides NOM functions through SBW"))
			return 0;
	}
	else
	{
		if(!SBWCreateModuleImpl("edu.caltech.NOM","SBW-NOM Bridge C Implementation",SBW_SelfManagedModule,"provides NOM functions through SBW"))
			return 0;
	}

	/*----------NomImpl-------------------*/
	addNomImpl();

	if(isUnique)
	{
        addNOMClipboardImpl();
	}
	
	/*If the module is invoked with -sbwregister then the module simply
	  registers itself and terminates otherwise the module runs in module mode
	  providing services.
	  */
	if(isRegister)
	{
		char commandLine[100];
		strcpy(commandLine,argv[0]);

		if(isUnique)
		{
			if(!SBWModuleImplSetCommandLine(strcat(commandLine," -sbwmodule -unique")))
				return 0;
		}
		else
		{
			if(!SBWModuleImplSetCommandLine(strcat(commandLine," -sbwmodule")))
			return 0;
		}
		if(!SBWModuleImplRegister())
			return 0;
	}
	else
	{
		if(!SBWModuleImplRun(argc,argv,1))
		return 0;
	}
	
	return 1;
}

/*
* module entry point WinMain/main
* On windows using WinMain as an entry point means that the executable can be an
* invisible process rather than a console application.
* arguments: in windows case all are ignored, see windows documentation
* for details of arguments. In linux case standard main arguments
* result: int - -1 if error occurred 0 otherwise.
*/
#if defined(WIN32)
int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
LPSTR lpCmdLine, int nCmdShow)
{
	int argc ;
	char **argv ;
	SBWWindowsExtractCommandLine(&argc, &argv); /* get windows commandline */
#elif defined(linux)
	int main(int argc, char* argv[])
	{
#endif
	if (!cserver(argc, argv))
	{
#ifdef WIN32
/* display error dialog */
		MessageBox(NULL, SBWExceptionGetMessage(),"SBW-NOM",MB_OK | MB_ICONEXCLAMATION);
#else
		fprintf(stderr, "cserver: %s\n", SBWExceptionGetMessage());
#endif
		return -1;
	}
	return 0;
}

